# This file is autogenerated by make
# values can be changed in the Makefile

# Version of this python package, used in setup.py
__version__ = "0.14.1"

# Version of the grammar pulled down during packaging
#   https://github.com/tree-sitter/tree-sitter-python/archive/refs/tags/v0.19.0.tar.gz
__grammar_version__ = "0.19.0"
