#include <tree_sitter/parser.h>

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 13
#define STATE_COUNT 1088
#define LARGE_STATE_COUNT 101
#define SYMBOL_COUNT 227
#define ALIAS_COUNT 0
#define TOKEN_COUNT 101
#define EXTERNAL_TOKEN_COUNT 6
#define FIELD_COUNT 25
#define MAX_ALIAS_SEQUENCE_LENGTH 9
#define PRODUCTION_ID_COUNT 91

enum {
  sym_identifier = 1,
  anon_sym_import = 2,
  anon_sym_DOT = 3,
  anon_sym_from = 4,
  anon_sym___future__ = 5,
  anon_sym_LPAREN = 6,
  anon_sym_RPAREN = 7,
  anon_sym_COMMA = 8,
  anon_sym_as = 9,
  anon_sym_STAR = 10,
  anon_sym_print = 11,
  anon_sym_GT_GT = 12,
  anon_sym_assert = 13,
  anon_sym_COLON_EQ = 14,
  anon_sym_return = 15,
  anon_sym_del = 16,
  anon_sym_raise = 17,
  anon_sym_pass = 18,
  anon_sym_break = 19,
  anon_sym_continue = 20,
  anon_sym_if = 21,
  anon_sym_COLON = 22,
  anon_sym_elif = 23,
  anon_sym_else = 24,
  anon_sym_async = 25,
  anon_sym_for = 26,
  anon_sym_in = 27,
  anon_sym_while = 28,
  anon_sym_try = 29,
  anon_sym_except = 30,
  anon_sym_finally = 31,
  anon_sym_with = 32,
  anon_sym_def = 33,
  anon_sym_DASH_GT = 34,
  anon_sym_STAR_STAR = 35,
  anon_sym_global = 36,
  anon_sym_nonlocal = 37,
  anon_sym_exec = 38,
  anon_sym_class = 39,
  anon_sym_AT = 40,
  anon_sym_LBRACK = 41,
  anon_sym_RBRACK = 42,
  anon_sym_EQ = 43,
  anon_sym_not = 44,
  anon_sym_and = 45,
  anon_sym_or = 46,
  anon_sym_PLUS = 47,
  anon_sym_DASH = 48,
  anon_sym_SLASH = 49,
  anon_sym_PERCENT = 50,
  anon_sym_SLASH_SLASH = 51,
  anon_sym_PIPE = 52,
  anon_sym_AMP = 53,
  anon_sym_CARET = 54,
  anon_sym_LT_LT = 55,
  anon_sym_TILDE = 56,
  anon_sym_LT = 57,
  anon_sym_LT_EQ = 58,
  anon_sym_EQ_EQ = 59,
  anon_sym_BANG_EQ = 60,
  anon_sym_GT_EQ = 61,
  anon_sym_GT = 62,
  anon_sym_LT_GT = 63,
  anon_sym_is = 64,
  anon_sym_lambda = 65,
  anon_sym_PLUS_EQ = 66,
  anon_sym_DASH_EQ = 67,
  anon_sym_STAR_EQ = 68,
  anon_sym_SLASH_EQ = 69,
  anon_sym_AT_EQ = 70,
  anon_sym_SLASH_SLASH_EQ = 71,
  anon_sym_PERCENT_EQ = 72,
  anon_sym_STAR_STAR_EQ = 73,
  anon_sym_GT_GT_EQ = 74,
  anon_sym_LT_LT_EQ = 75,
  anon_sym_AMP_EQ = 76,
  anon_sym_CARET_EQ = 77,
  anon_sym_PIPE_EQ = 78,
  anon_sym_yield = 79,
  sym_ellipsis = 80,
  anon_sym_LBRACE = 81,
  anon_sym_RBRACE = 82,
  sym_escape_sequence = 83,
  sym__not_escape_sequence = 84,
  aux_sym_format_specifier_token1 = 85,
  sym_type_conversion = 86,
  sym_integer = 87,
  sym_float = 88,
  anon_sym_await = 89,
  sym_true = 90,
  sym_false = 91,
  sym_none = 92,
  sym_comment = 93,
  sym__semicolon = 94,
  sym__newline = 95,
  sym__indent = 96,
  sym__dedent = 97,
  sym__string_start = 98,
  sym__string_content = 99,
  sym__string_end = 100,
  sym_module = 101,
  sym__statement = 102,
  sym__simple_statements = 103,
  sym_import_statement = 104,
  sym_import_prefix = 105,
  sym_relative_import = 106,
  sym_future_import_statement = 107,
  sym_import_from_statement = 108,
  sym__import_list = 109,
  sym_aliased_import = 110,
  sym_wildcard_import = 111,
  sym_print_statement = 112,
  sym_chevron = 113,
  sym_assert_statement = 114,
  sym_expression_statement = 115,
  sym_named_expression = 116,
  sym_return_statement = 117,
  sym_delete_statement = 118,
  sym_raise_statement = 119,
  sym_pass_statement = 120,
  sym_break_statement = 121,
  sym_continue_statement = 122,
  sym_if_statement = 123,
  sym_elif_clause = 124,
  sym_else_clause = 125,
  sym_for_statement = 126,
  sym_while_statement = 127,
  sym_try_statement = 128,
  sym_except_clause = 129,
  sym_finally_clause = 130,
  sym_with_statement = 131,
  sym_with_clause = 132,
  sym_with_item = 133,
  sym_function_definition = 134,
  sym_parameters = 135,
  sym_lambda_parameters = 136,
  sym_list_splat = 137,
  sym_dictionary_splat = 138,
  sym_global_statement = 139,
  sym_nonlocal_statement = 140,
  sym_exec_statement = 141,
  sym_class_definition = 142,
  sym_parenthesized_list_splat = 143,
  sym_argument_list = 144,
  sym_decorated_definition = 145,
  sym_decorator = 146,
  sym_block = 147,
  sym_expression_list = 148,
  sym_dotted_name = 149,
  sym__parameters = 150,
  sym__patterns = 151,
  sym_parameter = 152,
  sym_pattern = 153,
  sym_tuple_pattern = 154,
  sym_list_pattern = 155,
  sym_default_parameter = 156,
  sym_typed_default_parameter = 157,
  sym_list_splat_pattern = 158,
  sym_dictionary_splat_pattern = 159,
  sym__expression_within_for_in_clause = 160,
  sym_expression = 161,
  sym_primary_expression = 162,
  sym_not_operator = 163,
  sym_boolean_operator = 164,
  sym_binary_operator = 165,
  sym_unary_operator = 166,
  sym_comparison_operator = 167,
  sym_lambda = 168,
  sym_lambda_within_for_in_clause = 169,
  sym_assignment = 170,
  sym_augmented_assignment = 171,
  sym_pattern_list = 172,
  sym__right_hand_side = 173,
  sym_yield = 174,
  sym_attribute = 175,
  sym_subscript = 176,
  sym_slice = 177,
  sym_call = 178,
  sym_typed_parameter = 179,
  sym_type = 180,
  sym_keyword_argument = 181,
  sym_list = 182,
  sym_set = 183,
  sym_tuple = 184,
  sym_dictionary = 185,
  sym_pair = 186,
  sym_list_comprehension = 187,
  sym_dictionary_comprehension = 188,
  sym_set_comprehension = 189,
  sym_generator_expression = 190,
  sym__comprehension_clauses = 191,
  sym_parenthesized_expression = 192,
  sym__collection_elements = 193,
  sym_for_in_clause = 194,
  sym_if_clause = 195,
  sym_conditional_expression = 196,
  sym_concatenated_string = 197,
  sym_string = 198,
  sym_interpolation = 199,
  sym_format_specifier = 200,
  sym_format_expression = 201,
  sym_await = 202,
  aux_sym_module_repeat1 = 203,
  aux_sym__simple_statements_repeat1 = 204,
  aux_sym_import_prefix_repeat1 = 205,
  aux_sym__import_list_repeat1 = 206,
  aux_sym_print_statement_repeat1 = 207,
  aux_sym_assert_statement_repeat1 = 208,
  aux_sym_if_statement_repeat1 = 209,
  aux_sym_try_statement_repeat1 = 210,
  aux_sym_with_clause_repeat1 = 211,
  aux_sym_global_statement_repeat1 = 212,
  aux_sym_argument_list_repeat1 = 213,
  aux_sym_decorated_definition_repeat1 = 214,
  aux_sym_dotted_name_repeat1 = 215,
  aux_sym__parameters_repeat1 = 216,
  aux_sym__patterns_repeat1 = 217,
  aux_sym_comparison_operator_repeat1 = 218,
  aux_sym_subscript_repeat1 = 219,
  aux_sym_dictionary_repeat1 = 220,
  aux_sym__comprehension_clauses_repeat1 = 221,
  aux_sym__collection_elements_repeat1 = 222,
  aux_sym_for_in_clause_repeat1 = 223,
  aux_sym_concatenated_string_repeat1 = 224,
  aux_sym_string_repeat1 = 225,
  aux_sym_format_specifier_repeat1 = 226,
};

static const char *ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [anon_sym_import] = "import",
  [anon_sym_DOT] = ".",
  [anon_sym_from] = "from",
  [anon_sym___future__] = "__future__",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_COMMA] = ",",
  [anon_sym_as] = "as",
  [anon_sym_STAR] = "*",
  [anon_sym_print] = "print",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_assert] = "assert",
  [anon_sym_COLON_EQ] = ":=",
  [anon_sym_return] = "return",
  [anon_sym_del] = "del",
  [anon_sym_raise] = "raise",
  [anon_sym_pass] = "pass",
  [anon_sym_break] = "break",
  [anon_sym_continue] = "continue",
  [anon_sym_if] = "if",
  [anon_sym_COLON] = ":",
  [anon_sym_elif] = "elif",
  [anon_sym_else] = "else",
  [anon_sym_async] = "async",
  [anon_sym_for] = "for",
  [anon_sym_in] = "in",
  [anon_sym_while] = "while",
  [anon_sym_try] = "try",
  [anon_sym_except] = "except",
  [anon_sym_finally] = "finally",
  [anon_sym_with] = "with",
  [anon_sym_def] = "def",
  [anon_sym_DASH_GT] = "->",
  [anon_sym_STAR_STAR] = "**",
  [anon_sym_global] = "global",
  [anon_sym_nonlocal] = "nonlocal",
  [anon_sym_exec] = "exec",
  [anon_sym_class] = "class",
  [anon_sym_AT] = "@",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_EQ] = "=",
  [anon_sym_not] = "not",
  [anon_sym_and] = "and",
  [anon_sym_or] = "or",
  [anon_sym_PLUS] = "+",
  [anon_sym_DASH] = "-",
  [anon_sym_SLASH] = "/",
  [anon_sym_PERCENT] = "%",
  [anon_sym_SLASH_SLASH] = "//",
  [anon_sym_PIPE] = "|",
  [anon_sym_AMP] = "&",
  [anon_sym_CARET] = "^",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_TILDE] = "~",
  [anon_sym_LT] = "<",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_GT] = ">",
  [anon_sym_LT_GT] = "<>",
  [anon_sym_is] = "is",
  [anon_sym_lambda] = "lambda",
  [anon_sym_PLUS_EQ] = "+=",
  [anon_sym_DASH_EQ] = "-=",
  [anon_sym_STAR_EQ] = "*=",
  [anon_sym_SLASH_EQ] = "/=",
  [anon_sym_AT_EQ] = "@=",
  [anon_sym_SLASH_SLASH_EQ] = "//=",
  [anon_sym_PERCENT_EQ] = "%=",
  [anon_sym_STAR_STAR_EQ] = "**=",
  [anon_sym_GT_GT_EQ] = ">>=",
  [anon_sym_LT_LT_EQ] = "<<=",
  [anon_sym_AMP_EQ] = "&=",
  [anon_sym_CARET_EQ] = "^=",
  [anon_sym_PIPE_EQ] = "|=",
  [anon_sym_yield] = "yield",
  [sym_ellipsis] = "ellipsis",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [sym_escape_sequence] = "escape_sequence",
  [sym__not_escape_sequence] = "_not_escape_sequence",
  [aux_sym_format_specifier_token1] = "format_specifier_token1",
  [sym_type_conversion] = "type_conversion",
  [sym_integer] = "integer",
  [sym_float] = "float",
  [anon_sym_await] = "await",
  [sym_true] = "true",
  [sym_false] = "false",
  [sym_none] = "none",
  [sym_comment] = "comment",
  [sym__semicolon] = "_semicolon",
  [sym__newline] = "_newline",
  [sym__indent] = "_indent",
  [sym__dedent] = "_dedent",
  [sym__string_start] = "\"",
  [sym__string_content] = "_string_content",
  [sym__string_end] = "\"",
  [sym_module] = "module",
  [sym__statement] = "_statement",
  [sym__simple_statements] = "_simple_statements",
  [sym_import_statement] = "import_statement",
  [sym_import_prefix] = "import_prefix",
  [sym_relative_import] = "relative_import",
  [sym_future_import_statement] = "future_import_statement",
  [sym_import_from_statement] = "import_from_statement",
  [sym__import_list] = "_import_list",
  [sym_aliased_import] = "aliased_import",
  [sym_wildcard_import] = "wildcard_import",
  [sym_print_statement] = "print_statement",
  [sym_chevron] = "chevron",
  [sym_assert_statement] = "assert_statement",
  [sym_expression_statement] = "expression_statement",
  [sym_named_expression] = "named_expression",
  [sym_return_statement] = "return_statement",
  [sym_delete_statement] = "delete_statement",
  [sym_raise_statement] = "raise_statement",
  [sym_pass_statement] = "pass_statement",
  [sym_break_statement] = "break_statement",
  [sym_continue_statement] = "continue_statement",
  [sym_if_statement] = "if_statement",
  [sym_elif_clause] = "elif_clause",
  [sym_else_clause] = "else_clause",
  [sym_for_statement] = "for_statement",
  [sym_while_statement] = "while_statement",
  [sym_try_statement] = "try_statement",
  [sym_except_clause] = "except_clause",
  [sym_finally_clause] = "finally_clause",
  [sym_with_statement] = "with_statement",
  [sym_with_clause] = "with_clause",
  [sym_with_item] = "with_item",
  [sym_function_definition] = "function_definition",
  [sym_parameters] = "parameters",
  [sym_lambda_parameters] = "lambda_parameters",
  [sym_list_splat] = "list_splat",
  [sym_dictionary_splat] = "dictionary_splat",
  [sym_global_statement] = "global_statement",
  [sym_nonlocal_statement] = "nonlocal_statement",
  [sym_exec_statement] = "exec_statement",
  [sym_class_definition] = "class_definition",
  [sym_parenthesized_list_splat] = "parenthesized_list_splat",
  [sym_argument_list] = "argument_list",
  [sym_decorated_definition] = "decorated_definition",
  [sym_decorator] = "decorator",
  [sym_block] = "block",
  [sym_expression_list] = "expression_list",
  [sym_dotted_name] = "dotted_name",
  [sym__parameters] = "_parameters",
  [sym__patterns] = "_patterns",
  [sym_parameter] = "parameter",
  [sym_pattern] = "pattern",
  [sym_tuple_pattern] = "tuple_pattern",
  [sym_list_pattern] = "list_pattern",
  [sym_default_parameter] = "default_parameter",
  [sym_typed_default_parameter] = "typed_default_parameter",
  [sym_list_splat_pattern] = "list_splat_pattern",
  [sym_dictionary_splat_pattern] = "dictionary_splat_pattern",
  [sym__expression_within_for_in_clause] = "_expression_within_for_in_clause",
  [sym_expression] = "expression",
  [sym_primary_expression] = "primary_expression",
  [sym_not_operator] = "not_operator",
  [sym_boolean_operator] = "boolean_operator",
  [sym_binary_operator] = "binary_operator",
  [sym_unary_operator] = "unary_operator",
  [sym_comparison_operator] = "comparison_operator",
  [sym_lambda] = "lambda",
  [sym_lambda_within_for_in_clause] = "lambda",
  [sym_assignment] = "assignment",
  [sym_augmented_assignment] = "augmented_assignment",
  [sym_pattern_list] = "pattern_list",
  [sym__right_hand_side] = "_right_hand_side",
  [sym_yield] = "yield",
  [sym_attribute] = "attribute",
  [sym_subscript] = "subscript",
  [sym_slice] = "slice",
  [sym_call] = "call",
  [sym_typed_parameter] = "typed_parameter",
  [sym_type] = "type",
  [sym_keyword_argument] = "keyword_argument",
  [sym_list] = "list",
  [sym_set] = "set",
  [sym_tuple] = "tuple",
  [sym_dictionary] = "dictionary",
  [sym_pair] = "pair",
  [sym_list_comprehension] = "list_comprehension",
  [sym_dictionary_comprehension] = "dictionary_comprehension",
  [sym_set_comprehension] = "set_comprehension",
  [sym_generator_expression] = "generator_expression",
  [sym__comprehension_clauses] = "_comprehension_clauses",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym__collection_elements] = "_collection_elements",
  [sym_for_in_clause] = "for_in_clause",
  [sym_if_clause] = "if_clause",
  [sym_conditional_expression] = "conditional_expression",
  [sym_concatenated_string] = "concatenated_string",
  [sym_string] = "string",
  [sym_interpolation] = "interpolation",
  [sym_format_specifier] = "format_specifier",
  [sym_format_expression] = "format_expression",
  [sym_await] = "await",
  [aux_sym_module_repeat1] = "module_repeat1",
  [aux_sym__simple_statements_repeat1] = "_simple_statements_repeat1",
  [aux_sym_import_prefix_repeat1] = "import_prefix_repeat1",
  [aux_sym__import_list_repeat1] = "_import_list_repeat1",
  [aux_sym_print_statement_repeat1] = "print_statement_repeat1",
  [aux_sym_assert_statement_repeat1] = "assert_statement_repeat1",
  [aux_sym_if_statement_repeat1] = "if_statement_repeat1",
  [aux_sym_try_statement_repeat1] = "try_statement_repeat1",
  [aux_sym_with_clause_repeat1] = "with_clause_repeat1",
  [aux_sym_global_statement_repeat1] = "global_statement_repeat1",
  [aux_sym_argument_list_repeat1] = "argument_list_repeat1",
  [aux_sym_decorated_definition_repeat1] = "decorated_definition_repeat1",
  [aux_sym_dotted_name_repeat1] = "dotted_name_repeat1",
  [aux_sym__parameters_repeat1] = "_parameters_repeat1",
  [aux_sym__patterns_repeat1] = "_patterns_repeat1",
  [aux_sym_comparison_operator_repeat1] = "comparison_operator_repeat1",
  [aux_sym_subscript_repeat1] = "subscript_repeat1",
  [aux_sym_dictionary_repeat1] = "dictionary_repeat1",
  [aux_sym__comprehension_clauses_repeat1] = "_comprehension_clauses_repeat1",
  [aux_sym__collection_elements_repeat1] = "_collection_elements_repeat1",
  [aux_sym_for_in_clause_repeat1] = "for_in_clause_repeat1",
  [aux_sym_concatenated_string_repeat1] = "concatenated_string_repeat1",
  [aux_sym_string_repeat1] = "string_repeat1",
  [aux_sym_format_specifier_repeat1] = "format_specifier_repeat1",
};

static TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [anon_sym_import] = anon_sym_import,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_from] = anon_sym_from,
  [anon_sym___future__] = anon_sym___future__,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_as] = anon_sym_as,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_print] = anon_sym_print,
  [anon_sym_GT_GT] = anon_sym_GT_GT,
  [anon_sym_assert] = anon_sym_assert,
  [anon_sym_COLON_EQ] = anon_sym_COLON_EQ,
  [anon_sym_return] = anon_sym_return,
  [anon_sym_del] = anon_sym_del,
  [anon_sym_raise] = anon_sym_raise,
  [anon_sym_pass] = anon_sym_pass,
  [anon_sym_break] = anon_sym_break,
  [anon_sym_continue] = anon_sym_continue,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_elif] = anon_sym_elif,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_async] = anon_sym_async,
  [anon_sym_for] = anon_sym_for,
  [anon_sym_in] = anon_sym_in,
  [anon_sym_while] = anon_sym_while,
  [anon_sym_try] = anon_sym_try,
  [anon_sym_except] = anon_sym_except,
  [anon_sym_finally] = anon_sym_finally,
  [anon_sym_with] = anon_sym_with,
  [anon_sym_def] = anon_sym_def,
  [anon_sym_DASH_GT] = anon_sym_DASH_GT,
  [anon_sym_STAR_STAR] = anon_sym_STAR_STAR,
  [anon_sym_global] = anon_sym_global,
  [anon_sym_nonlocal] = anon_sym_nonlocal,
  [anon_sym_exec] = anon_sym_exec,
  [anon_sym_class] = anon_sym_class,
  [anon_sym_AT] = anon_sym_AT,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_not] = anon_sym_not,
  [anon_sym_and] = anon_sym_and,
  [anon_sym_or] = anon_sym_or,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [anon_sym_SLASH_SLASH] = anon_sym_SLASH_SLASH,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_LT_LT] = anon_sym_LT_LT,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [anon_sym_BANG_EQ] = anon_sym_BANG_EQ,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_LT_GT] = anon_sym_LT_GT,
  [anon_sym_is] = anon_sym_is,
  [anon_sym_lambda] = anon_sym_lambda,
  [anon_sym_PLUS_EQ] = anon_sym_PLUS_EQ,
  [anon_sym_DASH_EQ] = anon_sym_DASH_EQ,
  [anon_sym_STAR_EQ] = anon_sym_STAR_EQ,
  [anon_sym_SLASH_EQ] = anon_sym_SLASH_EQ,
  [anon_sym_AT_EQ] = anon_sym_AT_EQ,
  [anon_sym_SLASH_SLASH_EQ] = anon_sym_SLASH_SLASH_EQ,
  [anon_sym_PERCENT_EQ] = anon_sym_PERCENT_EQ,
  [anon_sym_STAR_STAR_EQ] = anon_sym_STAR_STAR_EQ,
  [anon_sym_GT_GT_EQ] = anon_sym_GT_GT_EQ,
  [anon_sym_LT_LT_EQ] = anon_sym_LT_LT_EQ,
  [anon_sym_AMP_EQ] = anon_sym_AMP_EQ,
  [anon_sym_CARET_EQ] = anon_sym_CARET_EQ,
  [anon_sym_PIPE_EQ] = anon_sym_PIPE_EQ,
  [anon_sym_yield] = anon_sym_yield,
  [sym_ellipsis] = sym_ellipsis,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [sym_escape_sequence] = sym_escape_sequence,
  [sym__not_escape_sequence] = sym__not_escape_sequence,
  [aux_sym_format_specifier_token1] = aux_sym_format_specifier_token1,
  [sym_type_conversion] = sym_type_conversion,
  [sym_integer] = sym_integer,
  [sym_float] = sym_float,
  [anon_sym_await] = anon_sym_await,
  [sym_true] = sym_true,
  [sym_false] = sym_false,
  [sym_none] = sym_none,
  [sym_comment] = sym_comment,
  [sym__semicolon] = sym__semicolon,
  [sym__newline] = sym__newline,
  [sym__indent] = sym__indent,
  [sym__dedent] = sym__dedent,
  [sym__string_start] = sym__string_start,
  [sym__string_content] = sym__string_content,
  [sym__string_end] = sym__string_start,
  [sym_module] = sym_module,
  [sym__statement] = sym__statement,
  [sym__simple_statements] = sym__simple_statements,
  [sym_import_statement] = sym_import_statement,
  [sym_import_prefix] = sym_import_prefix,
  [sym_relative_import] = sym_relative_import,
  [sym_future_import_statement] = sym_future_import_statement,
  [sym_import_from_statement] = sym_import_from_statement,
  [sym__import_list] = sym__import_list,
  [sym_aliased_import] = sym_aliased_import,
  [sym_wildcard_import] = sym_wildcard_import,
  [sym_print_statement] = sym_print_statement,
  [sym_chevron] = sym_chevron,
  [sym_assert_statement] = sym_assert_statement,
  [sym_expression_statement] = sym_expression_statement,
  [sym_named_expression] = sym_named_expression,
  [sym_return_statement] = sym_return_statement,
  [sym_delete_statement] = sym_delete_statement,
  [sym_raise_statement] = sym_raise_statement,
  [sym_pass_statement] = sym_pass_statement,
  [sym_break_statement] = sym_break_statement,
  [sym_continue_statement] = sym_continue_statement,
  [sym_if_statement] = sym_if_statement,
  [sym_elif_clause] = sym_elif_clause,
  [sym_else_clause] = sym_else_clause,
  [sym_for_statement] = sym_for_statement,
  [sym_while_statement] = sym_while_statement,
  [sym_try_statement] = sym_try_statement,
  [sym_except_clause] = sym_except_clause,
  [sym_finally_clause] = sym_finally_clause,
  [sym_with_statement] = sym_with_statement,
  [sym_with_clause] = sym_with_clause,
  [sym_with_item] = sym_with_item,
  [sym_function_definition] = sym_function_definition,
  [sym_parameters] = sym_parameters,
  [sym_lambda_parameters] = sym_lambda_parameters,
  [sym_list_splat] = sym_list_splat,
  [sym_dictionary_splat] = sym_dictionary_splat,
  [sym_global_statement] = sym_global_statement,
  [sym_nonlocal_statement] = sym_nonlocal_statement,
  [sym_exec_statement] = sym_exec_statement,
  [sym_class_definition] = sym_class_definition,
  [sym_parenthesized_list_splat] = sym_parenthesized_list_splat,
  [sym_argument_list] = sym_argument_list,
  [sym_decorated_definition] = sym_decorated_definition,
  [sym_decorator] = sym_decorator,
  [sym_block] = sym_block,
  [sym_expression_list] = sym_expression_list,
  [sym_dotted_name] = sym_dotted_name,
  [sym__parameters] = sym__parameters,
  [sym__patterns] = sym__patterns,
  [sym_parameter] = sym_parameter,
  [sym_pattern] = sym_pattern,
  [sym_tuple_pattern] = sym_tuple_pattern,
  [sym_list_pattern] = sym_list_pattern,
  [sym_default_parameter] = sym_default_parameter,
  [sym_typed_default_parameter] = sym_typed_default_parameter,
  [sym_list_splat_pattern] = sym_list_splat_pattern,
  [sym_dictionary_splat_pattern] = sym_dictionary_splat_pattern,
  [sym__expression_within_for_in_clause] = sym__expression_within_for_in_clause,
  [sym_expression] = sym_expression,
  [sym_primary_expression] = sym_primary_expression,
  [sym_not_operator] = sym_not_operator,
  [sym_boolean_operator] = sym_boolean_operator,
  [sym_binary_operator] = sym_binary_operator,
  [sym_unary_operator] = sym_unary_operator,
  [sym_comparison_operator] = sym_comparison_operator,
  [sym_lambda] = sym_lambda,
  [sym_lambda_within_for_in_clause] = sym_lambda,
  [sym_assignment] = sym_assignment,
  [sym_augmented_assignment] = sym_augmented_assignment,
  [sym_pattern_list] = sym_pattern_list,
  [sym__right_hand_side] = sym__right_hand_side,
  [sym_yield] = sym_yield,
  [sym_attribute] = sym_attribute,
  [sym_subscript] = sym_subscript,
  [sym_slice] = sym_slice,
  [sym_call] = sym_call,
  [sym_typed_parameter] = sym_typed_parameter,
  [sym_type] = sym_type,
  [sym_keyword_argument] = sym_keyword_argument,
  [sym_list] = sym_list,
  [sym_set] = sym_set,
  [sym_tuple] = sym_tuple,
  [sym_dictionary] = sym_dictionary,
  [sym_pair] = sym_pair,
  [sym_list_comprehension] = sym_list_comprehension,
  [sym_dictionary_comprehension] = sym_dictionary_comprehension,
  [sym_set_comprehension] = sym_set_comprehension,
  [sym_generator_expression] = sym_generator_expression,
  [sym__comprehension_clauses] = sym__comprehension_clauses,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym__collection_elements] = sym__collection_elements,
  [sym_for_in_clause] = sym_for_in_clause,
  [sym_if_clause] = sym_if_clause,
  [sym_conditional_expression] = sym_conditional_expression,
  [sym_concatenated_string] = sym_concatenated_string,
  [sym_string] = sym_string,
  [sym_interpolation] = sym_interpolation,
  [sym_format_specifier] = sym_format_specifier,
  [sym_format_expression] = sym_format_expression,
  [sym_await] = sym_await,
  [aux_sym_module_repeat1] = aux_sym_module_repeat1,
  [aux_sym__simple_statements_repeat1] = aux_sym__simple_statements_repeat1,
  [aux_sym_import_prefix_repeat1] = aux_sym_import_prefix_repeat1,
  [aux_sym__import_list_repeat1] = aux_sym__import_list_repeat1,
  [aux_sym_print_statement_repeat1] = aux_sym_print_statement_repeat1,
  [aux_sym_assert_statement_repeat1] = aux_sym_assert_statement_repeat1,
  [aux_sym_if_statement_repeat1] = aux_sym_if_statement_repeat1,
  [aux_sym_try_statement_repeat1] = aux_sym_try_statement_repeat1,
  [aux_sym_with_clause_repeat1] = aux_sym_with_clause_repeat1,
  [aux_sym_global_statement_repeat1] = aux_sym_global_statement_repeat1,
  [aux_sym_argument_list_repeat1] = aux_sym_argument_list_repeat1,
  [aux_sym_decorated_definition_repeat1] = aux_sym_decorated_definition_repeat1,
  [aux_sym_dotted_name_repeat1] = aux_sym_dotted_name_repeat1,
  [aux_sym__parameters_repeat1] = aux_sym__parameters_repeat1,
  [aux_sym__patterns_repeat1] = aux_sym__patterns_repeat1,
  [aux_sym_comparison_operator_repeat1] = aux_sym_comparison_operator_repeat1,
  [aux_sym_subscript_repeat1] = aux_sym_subscript_repeat1,
  [aux_sym_dictionary_repeat1] = aux_sym_dictionary_repeat1,
  [aux_sym__comprehension_clauses_repeat1] = aux_sym__comprehension_clauses_repeat1,
  [aux_sym__collection_elements_repeat1] = aux_sym__collection_elements_repeat1,
  [aux_sym_for_in_clause_repeat1] = aux_sym_for_in_clause_repeat1,
  [aux_sym_concatenated_string_repeat1] = aux_sym_concatenated_string_repeat1,
  [aux_sym_string_repeat1] = aux_sym_string_repeat1,
  [aux_sym_format_specifier_repeat1] = aux_sym_format_specifier_repeat1,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_import] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_from] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___future__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_as] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_print] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_assert] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_del] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_raise] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_pass] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_continue] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_elif] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_async] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_in] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_while] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_try] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_except] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_finally] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_with] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_def] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_global] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_nonlocal] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_exec] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_class] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_not] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_and] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_or] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_is] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_lambda] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_yield] = {
    .visible = true,
    .named = false,
  },
  [sym_ellipsis] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [sym_escape_sequence] = {
    .visible = true,
    .named = true,
  },
  [sym__not_escape_sequence] = {
    .visible = false,
    .named = true,
  },
  [aux_sym_format_specifier_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_type_conversion] = {
    .visible = true,
    .named = true,
  },
  [sym_integer] = {
    .visible = true,
    .named = true,
  },
  [sym_float] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_await] = {
    .visible = true,
    .named = false,
  },
  [sym_true] = {
    .visible = true,
    .named = true,
  },
  [sym_false] = {
    .visible = true,
    .named = true,
  },
  [sym_none] = {
    .visible = true,
    .named = true,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym__semicolon] = {
    .visible = false,
    .named = true,
  },
  [sym__newline] = {
    .visible = false,
    .named = true,
  },
  [sym__indent] = {
    .visible = false,
    .named = true,
  },
  [sym__dedent] = {
    .visible = false,
    .named = true,
  },
  [sym__string_start] = {
    .visible = true,
    .named = false,
  },
  [sym__string_content] = {
    .visible = false,
    .named = true,
  },
  [sym__string_end] = {
    .visible = true,
    .named = false,
  },
  [sym_module] = {
    .visible = true,
    .named = true,
  },
  [sym__statement] = {
    .visible = false,
    .named = true,
  },
  [sym__simple_statements] = {
    .visible = false,
    .named = true,
  },
  [sym_import_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_import_prefix] = {
    .visible = true,
    .named = true,
  },
  [sym_relative_import] = {
    .visible = true,
    .named = true,
  },
  [sym_future_import_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_import_from_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__import_list] = {
    .visible = false,
    .named = true,
  },
  [sym_aliased_import] = {
    .visible = true,
    .named = true,
  },
  [sym_wildcard_import] = {
    .visible = true,
    .named = true,
  },
  [sym_print_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_chevron] = {
    .visible = true,
    .named = true,
  },
  [sym_assert_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_named_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_return_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_delete_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_raise_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_pass_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_break_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_elif_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_else_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_for_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_while_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_try_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_except_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_finally_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_with_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_with_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_with_item] = {
    .visible = true,
    .named = true,
  },
  [sym_function_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_lambda_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_list_splat] = {
    .visible = true,
    .named = true,
  },
  [sym_dictionary_splat] = {
    .visible = true,
    .named = true,
  },
  [sym_global_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_nonlocal_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_exec_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_class_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_list_splat] = {
    .visible = true,
    .named = true,
  },
  [sym_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_decorated_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_decorator] = {
    .visible = true,
    .named = true,
  },
  [sym_block] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_list] = {
    .visible = true,
    .named = true,
  },
  [sym_dotted_name] = {
    .visible = true,
    .named = true,
  },
  [sym__parameters] = {
    .visible = false,
    .named = true,
  },
  [sym__patterns] = {
    .visible = false,
    .named = true,
  },
  [sym_parameter] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_pattern] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_tuple_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_list_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_default_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_typed_default_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_list_splat_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_dictionary_splat_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__expression_within_for_in_clause] = {
    .visible = false,
    .named = true,
  },
  [sym_expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_primary_expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_not_operator] = {
    .visible = true,
    .named = true,
  },
  [sym_boolean_operator] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_operator] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_operator] = {
    .visible = true,
    .named = true,
  },
  [sym_comparison_operator] = {
    .visible = true,
    .named = true,
  },
  [sym_lambda] = {
    .visible = true,
    .named = true,
  },
  [sym_lambda_within_for_in_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_augmented_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_pattern_list] = {
    .visible = true,
    .named = true,
  },
  [sym__right_hand_side] = {
    .visible = false,
    .named = true,
  },
  [sym_yield] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym_subscript] = {
    .visible = true,
    .named = true,
  },
  [sym_slice] = {
    .visible = true,
    .named = true,
  },
  [sym_call] = {
    .visible = true,
    .named = true,
  },
  [sym_typed_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_type] = {
    .visible = true,
    .named = true,
  },
  [sym_keyword_argument] = {
    .visible = true,
    .named = true,
  },
  [sym_list] = {
    .visible = true,
    .named = true,
  },
  [sym_set] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple] = {
    .visible = true,
    .named = true,
  },
  [sym_dictionary] = {
    .visible = true,
    .named = true,
  },
  [sym_pair] = {
    .visible = true,
    .named = true,
  },
  [sym_list_comprehension] = {
    .visible = true,
    .named = true,
  },
  [sym_dictionary_comprehension] = {
    .visible = true,
    .named = true,
  },
  [sym_set_comprehension] = {
    .visible = true,
    .named = true,
  },
  [sym_generator_expression] = {
    .visible = true,
    .named = true,
  },
  [sym__comprehension_clauses] = {
    .visible = false,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym__collection_elements] = {
    .visible = false,
    .named = true,
  },
  [sym_for_in_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_if_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_conditional_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_concatenated_string] = {
    .visible = true,
    .named = true,
  },
  [sym_string] = {
    .visible = true,
    .named = true,
  },
  [sym_interpolation] = {
    .visible = true,
    .named = true,
  },
  [sym_format_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_format_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_await] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_module_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__simple_statements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_import_prefix_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__import_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_print_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_assert_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_if_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_try_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_with_clause_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_global_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_decorated_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_dotted_name_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__patterns_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_comparison_operator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_subscript_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_dictionary_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__comprehension_clauses_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__collection_elements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_for_in_clause_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_concatenated_string_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_format_specifier_repeat1] = {
    .visible = false,
    .named = false,
  },
};

enum {
  field_alias = 1,
  field_alternative = 2,
  field_argument = 3,
  field_arguments = 4,
  field_attribute = 5,
  field_body = 6,
  field_cause = 7,
  field_code = 8,
  field_condition = 9,
  field_consequence = 10,
  field_definition = 11,
  field_function = 12,
  field_key = 13,
  field_left = 14,
  field_module_name = 15,
  field_name = 16,
  field_object = 17,
  field_operator = 18,
  field_parameters = 19,
  field_return_type = 20,
  field_right = 21,
  field_subscript = 22,
  field_superclasses = 23,
  field_type = 24,
  field_value = 25,
};

static const char *ts_field_names[] = {
  [0] = NULL,
  [field_alias] = "alias",
  [field_alternative] = "alternative",
  [field_argument] = "argument",
  [field_arguments] = "arguments",
  [field_attribute] = "attribute",
  [field_body] = "body",
  [field_cause] = "cause",
  [field_code] = "code",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_definition] = "definition",
  [field_function] = "function",
  [field_key] = "key",
  [field_left] = "left",
  [field_module_name] = "module_name",
  [field_name] = "name",
  [field_object] = "object",
  [field_operator] = "operator",
  [field_parameters] = "parameters",
  [field_return_type] = "return_type",
  [field_right] = "right",
  [field_subscript] = "subscript",
  [field_superclasses] = "superclasses",
  [field_type] = "type",
  [field_value] = "value",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [2] = {.index = 0, .length = 1},
  [3] = {.index = 1, .length = 1},
  [5] = {.index = 2, .length = 1},
  [6] = {.index = 3, .length = 1},
  [7] = {.index = 4, .length = 1},
  [8] = {.index = 5, .length = 2},
  [10] = {.index = 7, .length = 2},
  [11] = {.index = 9, .length = 1},
  [12] = {.index = 10, .length = 2},
  [13] = {.index = 12, .length = 1},
  [14] = {.index = 13, .length = 2},
  [15] = {.index = 15, .length = 1},
  [16] = {.index = 16, .length = 1},
  [17] = {.index = 17, .length = 2},
  [18] = {.index = 19, .length = 2},
  [19] = {.index = 21, .length = 2},
  [20] = {.index = 23, .length = 3},
  [21] = {.index = 26, .length = 2},
  [22] = {.index = 28, .length = 1},
  [23] = {.index = 29, .length = 2},
  [24] = {.index = 31, .length = 2},
  [25] = {.index = 33, .length = 1},
  [26] = {.index = 34, .length = 2},
  [28] = {.index = 36, .length = 1},
  [29] = {.index = 37, .length = 2},
  [30] = {.index = 39, .length = 1},
  [31] = {.index = 40, .length = 2},
  [32] = {.index = 42, .length = 2},
  [33] = {.index = 16, .length = 1},
  [34] = {.index = 44, .length = 1},
  [35] = {.index = 45, .length = 2},
  [36] = {.index = 47, .length = 2},
  [37] = {.index = 49, .length = 1},
  [38] = {.index = 50, .length = 2},
  [39] = {.index = 52, .length = 2},
  [40] = {.index = 54, .length = 2},
  [41] = {.index = 56, .length = 1},
  [42] = {.index = 57, .length = 3},
  [43] = {.index = 60, .length = 3},
  [44] = {.index = 63, .length = 3},
  [45] = {.index = 66, .length = 1},
  [46] = {.index = 67, .length = 3},
  [47] = {.index = 70, .length = 3},
  [48] = {.index = 73, .length = 2},
  [49] = {.index = 75, .length = 2},
  [50] = {.index = 77, .length = 3},
  [51] = {.index = 80, .length = 3},
  [52] = {.index = 83, .length = 3},
  [53] = {.index = 86, .length = 3},
  [54] = {.index = 17, .length = 2},
  [55] = {.index = 89, .length = 1},
  [56] = {.index = 90, .length = 2},
  [57] = {.index = 92, .length = 1},
  [58] = {.index = 93, .length = 2},
  [59] = {.index = 95, .length = 2},
  [60] = {.index = 97, .length = 4},
  [61] = {.index = 101, .length = 2},
  [62] = {.index = 103, .length = 4},
  [63] = {.index = 107, .length = 4},
  [64] = {.index = 111, .length = 2},
  [65] = {.index = 113, .length = 3},
  [66] = {.index = 116, .length = 3},
  [67] = {.index = 119, .length = 4},
  [69] = {.index = 123, .length = 4},
  [70] = {.index = 127, .length = 4},
  [71] = {.index = 131, .length = 3},
  [72] = {.index = 134, .length = 3},
  [73] = {.index = 137, .length = 2},
  [74] = {.index = 139, .length = 3},
  [75] = {.index = 142, .length = 5},
  [76] = {.index = 147, .length = 3},
  [77] = {.index = 150, .length = 4},
  [78] = {.index = 154, .length = 4},
  [79] = {.index = 158, .length = 4},
  [81] = {.index = 162, .length = 4},
  [82] = {.index = 166, .length = 3},
  [83] = {.index = 169, .length = 4},
  [84] = {.index = 173, .length = 4},
  [85] = {.index = 177, .length = 4},
  [86] = {.index = 181, .length = 5},
  [87] = {.index = 186, .length = 5},
  [88] = {.index = 191, .length = 5},
  [89] = {.index = 196, .length = 5},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_name, 1, .inherited = true},
  [1] =
    {field_name, 0},
  [2] =
    {field_argument, 1},
  [3] =
    {field_value, 0},
  [4] =
    {field_code, 1},
  [5] =
    {field_argument, 1},
    {field_operator, 0},
  [7] =
    {field_arguments, 1},
    {field_function, 0},
  [9] =
    {field_definition, 1},
  [10] =
    {field_name, 0},
    {field_name, 1, .inherited = true},
  [12] =
    {field_argument, 2, .inherited = true},
  [13] =
    {field_argument, 1},
    {field_argument, 2, .inherited = true},
  [15] =
    {field_cause, 2},
  [16] =
    {field_body, 2},
  [17] =
    {field_name, 0},
    {field_value, 2},
  [19] =
    {field_left, 0},
    {field_type, 2},
  [21] =
    {field_left, 0},
    {field_right, 2},
  [23] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [26] =
    {field_attribute, 2},
    {field_object, 0},
  [28] =
    {field_name, 1},
  [29] =
    {field_name, 0, .inherited = true},
    {field_name, 1, .inherited = true},
  [31] =
    {field_alias, 2},
    {field_name, 0},
  [33] =
    {field_name, 3, .inherited = true},
  [34] =
    {field_module_name, 1},
    {field_name, 3, .inherited = true},
  [36] =
    {field_body, 1},
  [37] =
    {field_argument, 0, .inherited = true},
    {field_argument, 1, .inherited = true},
  [39] =
    {field_cause, 3},
  [40] =
    {field_condition, 1},
    {field_consequence, 3},
  [42] =
    {field_body, 3},
    {field_condition, 1},
  [44] =
    {field_body, 3},
  [45] =
    {field_alias, 2},
    {field_value, 0},
  [47] =
    {field_body, 3},
    {field_name, 1},
  [49] =
    {field_type, 2},
  [50] =
    {field_body, 3},
    {field_parameters, 1},
  [52] =
    {field_key, 0},
    {field_value, 2},
  [54] =
    {field_subscript, 2},
    {field_value, 0},
  [56] =
    {field_alternative, 0},
  [57] =
    {field_alternative, 4},
    {field_condition, 1},
    {field_consequence, 3},
  [60] =
    {field_alternative, 4, .inherited = true},
    {field_condition, 1},
    {field_consequence, 3},
  [63] =
    {field_condition, 1},
    {field_consequence, 3},
    {field_consequence, 4},
  [66] =
    {field_body, 4},
  [67] =
    {field_alternative, 4},
    {field_body, 3},
    {field_condition, 1},
  [70] =
    {field_body, 3},
    {field_body, 4},
    {field_condition, 1},
  [73] =
    {field_body, 2},
    {field_body, 3},
  [75] =
    {field_body, 3},
    {field_body, 4},
  [77] =
    {field_body, 4},
    {field_name, 1},
    {field_parameters, 2},
  [80] =
    {field_body, 3},
    {field_body, 4},
    {field_name, 1},
  [83] =
    {field_body, 4},
    {field_name, 1},
    {field_superclasses, 2},
  [86] =
    {field_left, 0},
    {field_right, 4},
    {field_type, 2},
  [89] =
    {field_subscript, 1},
  [90] =
    {field_subscript, 0, .inherited = true},
    {field_subscript, 1, .inherited = true},
  [92] =
    {field_name, 4, .inherited = true},
  [93] =
    {field_module_name, 1},
    {field_name, 4, .inherited = true},
  [95] =
    {field_left, 1},
    {field_right, 3},
  [97] =
    {field_alternative, 4, .inherited = true},
    {field_alternative, 5},
    {field_condition, 1},
    {field_consequence, 3},
  [101] =
    {field_alternative, 0, .inherited = true},
    {field_alternative, 1, .inherited = true},
  [103] =
    {field_alternative, 5},
    {field_condition, 1},
    {field_consequence, 3},
    {field_consequence, 4},
  [107] =
    {field_alternative, 5, .inherited = true},
    {field_condition, 1},
    {field_consequence, 3},
    {field_consequence, 4},
  [111] =
    {field_body, 4},
    {field_body, 5},
  [113] =
    {field_body, 5},
    {field_name, 2},
    {field_parameters, 3},
  [116] =
    {field_body, 5},
    {field_left, 1},
    {field_right, 3},
  [119] =
    {field_alternative, 5},
    {field_body, 3},
    {field_body, 4},
    {field_condition, 1},
  [123] =
    {field_body, 4},
    {field_body, 5},
    {field_name, 1},
    {field_parameters, 2},
  [127] =
    {field_body, 4},
    {field_body, 5},
    {field_name, 1},
    {field_superclasses, 2},
  [131] =
    {field_name, 0},
    {field_type, 2},
    {field_value, 4},
  [134] =
    {field_subscript, 2},
    {field_subscript, 3, .inherited = true},
    {field_value, 0},
  [137] =
    {field_left, 2},
    {field_right, 4},
  [139] =
    {field_left, 1},
    {field_right, 3},
    {field_right, 4},
  [142] =
    {field_alternative, 5, .inherited = true},
    {field_alternative, 6},
    {field_condition, 1},
    {field_consequence, 3},
    {field_consequence, 4},
  [147] =
    {field_body, 6},
    {field_left, 2},
    {field_right, 4},
  [150] =
    {field_body, 5},
    {field_body, 6},
    {field_name, 2},
    {field_parameters, 3},
  [154] =
    {field_alternative, 6},
    {field_body, 5},
    {field_left, 1},
    {field_right, 3},
  [158] =
    {field_body, 5},
    {field_body, 6},
    {field_left, 1},
    {field_right, 3},
  [162] =
    {field_body, 6},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [166] =
    {field_left, 2},
    {field_right, 4},
    {field_right, 5},
  [169] =
    {field_alternative, 7},
    {field_body, 6},
    {field_left, 2},
    {field_right, 4},
  [173] =
    {field_body, 6},
    {field_body, 7},
    {field_left, 2},
    {field_right, 4},
  [177] =
    {field_body, 7},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
  [181] =
    {field_alternative, 7},
    {field_body, 5},
    {field_body, 6},
    {field_left, 1},
    {field_right, 3},
  [186] =
    {field_body, 6},
    {field_body, 7},
    {field_name, 1},
    {field_parameters, 2},
    {field_return_type, 4},
  [191] =
    {field_alternative, 8},
    {field_body, 6},
    {field_body, 7},
    {field_left, 2},
    {field_right, 4},
  [196] =
    {field_body, 7},
    {field_body, 8},
    {field_name, 2},
    {field_parameters, 3},
    {field_return_type, 5},
};

static TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [1] = {
    [0] = sym_identifier,
  },
  [4] = {
    [1] = sym_identifier,
  },
  [9] = {
    [0] = sym_list_splat_pattern,
  },
  [27] = {
    [1] = sym_parenthesized_expression,
  },
  [31] = {
    [3] = sym_block,
  },
  [32] = {
    [3] = sym_block,
  },
  [33] = {
    [2] = sym_block,
  },
  [34] = {
    [3] = sym_block,
  },
  [36] = {
    [3] = sym_block,
  },
  [42] = {
    [3] = sym_block,
  },
  [43] = {
    [3] = sym_block,
  },
  [45] = {
    [4] = sym_block,
  },
  [46] = {
    [3] = sym_block,
  },
  [50] = {
    [4] = sym_block,
  },
  [52] = {
    [4] = sym_block,
  },
  [54] = {
    [0] = sym_identifier,
  },
  [60] = {
    [3] = sym_block,
  },
  [65] = {
    [5] = sym_block,
  },
  [66] = {
    [5] = sym_block,
  },
  [68] = {
    [2] = sym_block,
  },
  [76] = {
    [6] = sym_block,
  },
  [78] = {
    [5] = sym_block,
  },
  [80] = {
    [3] = sym_block,
  },
  [81] = {
    [6] = sym_block,
  },
  [83] = {
    [6] = sym_block,
  },
  [85] = {
    [7] = sym_block,
  },
  [90] = {
    [5] = sym_block,
  },
};

static uint16_t ts_non_terminal_alias_map[] = {
  sym__simple_statements, 2,
    sym__simple_statements,
    sym_block,
  sym_parenthesized_list_splat, 2,
    sym_parenthesized_list_splat,
    sym_parenthesized_expression,
  0,
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(39);
      if (lookahead == '!') ADVANCE(14);
      if (lookahead == '#') ADVANCE(110);
      if (lookahead == '%') ADVANCE(60);
      if (lookahead == '&') ADVANCE(63);
      if (lookahead == '(') ADVANCE(42);
      if (lookahead == ')') ADVANCE(43);
      if (lookahead == '*') ADVANCE(45);
      if (lookahead == '+') ADVANCE(56);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '-') ADVANCE(58);
      if (lookahead == '.') ADVANCE(41);
      if (lookahead == '/') ADVANCE(59);
      if (lookahead == '0') ADVANCE(99);
      if (lookahead == ':') ADVANCE(49);
      if (lookahead == ';') ADVANCE(111);
      if (lookahead == '<') ADVANCE(67);
      if (lookahead == '=') ADVANCE(55);
      if (lookahead == '>') ADVANCE(72);
      if (lookahead == '@') ADVANCE(52);
      if (lookahead == '[') ADVANCE(53);
      if (lookahead == '\\') SKIP(35)
      if (lookahead == ']') ADVANCE(54);
      if (lookahead == '^') ADVANCE(64);
      if (lookahead == '{') ADVANCE(88);
      if (lookahead == '|') ADVANCE(62);
      if (lookahead == '}') ADVANCE(89);
      if (lookahead == '~') ADVANCE(66);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) SKIP(0)
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(98);
      if (('A' <= lookahead && lookahead <= '_') ||
          ('a' <= lookahead && lookahead <= 'z') ||
          (913 <= lookahead && lookahead <= 937) ||
          (945 <= lookahead && lookahead <= 969)) ADVANCE(109);
      END_STATE();
    case 1:
      if (lookahead == '\n') SKIP(10)
      END_STATE();
    case 2:
      if (lookahead == '\n') SKIP(10)
      if (lookahead == '\r') SKIP(1)
      END_STATE();
    case 3:
      if (lookahead == '\n') ADVANCE(91);
      END_STATE();
    case 4:
      if (lookahead == '\n') SKIP(4)
      if (lookahead == '#') ADVANCE(95);
      if (lookahead == '\\') ADVANCE(93);
      if (lookahead == '{') ADVANCE(88);
      if (lookahead == '}') ADVANCE(89);
      if (lookahead == '\t' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) ADVANCE(94);
      if (lookahead != 0) ADVANCE(95);
      END_STATE();
    case 5:
      if (lookahead == '\n') SKIP(11)
      END_STATE();
    case 6:
      if (lookahead == '\n') SKIP(11)
      if (lookahead == '\r') SKIP(5)
      END_STATE();
    case 7:
      if (lookahead == '\n') SKIP(9)
      END_STATE();
    case 8:
      if (lookahead == '\n') SKIP(9)
      if (lookahead == '\r') SKIP(7)
      END_STATE();
    case 9:
      if (lookahead == '!') ADVANCE(14);
      if (lookahead == '#') ADVANCE(110);
      if (lookahead == '%') ADVANCE(60);
      if (lookahead == '&') ADVANCE(63);
      if (lookahead == '(') ADVANCE(42);
      if (lookahead == ')') ADVANCE(43);
      if (lookahead == '*') ADVANCE(45);
      if (lookahead == '+') ADVANCE(56);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '-') ADVANCE(57);
      if (lookahead == '.') ADVANCE(40);
      if (lookahead == '/') ADVANCE(59);
      if (lookahead == ':') ADVANCE(49);
      if (lookahead == ';') ADVANCE(111);
      if (lookahead == '<') ADVANCE(67);
      if (lookahead == '=') ADVANCE(55);
      if (lookahead == '>') ADVANCE(72);
      if (lookahead == '@') ADVANCE(52);
      if (lookahead == '[') ADVANCE(53);
      if (lookahead == '\\') SKIP(8)
      if (lookahead == ']') ADVANCE(54);
      if (lookahead == '^') ADVANCE(64);
      if (lookahead == '|') ADVANCE(62);
      if (lookahead == '}') ADVANCE(89);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) SKIP(9)
      if (('A' <= lookahead && lookahead <= '_') ||
          ('a' <= lookahead && lookahead <= 'z') ||
          (913 <= lookahead && lookahead <= 937) ||
          (945 <= lookahead && lookahead <= 969)) ADVANCE(109);
      END_STATE();
    case 10:
      if (lookahead == '!') ADVANCE(14);
      if (lookahead == '#') ADVANCE(110);
      if (lookahead == '%') ADVANCE(60);
      if (lookahead == '&') ADVANCE(63);
      if (lookahead == '(') ADVANCE(42);
      if (lookahead == ')') ADVANCE(43);
      if (lookahead == '*') ADVANCE(45);
      if (lookahead == '+') ADVANCE(56);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '-') ADVANCE(57);
      if (lookahead == '.') ADVANCE(40);
      if (lookahead == '/') ADVANCE(59);
      if (lookahead == ':') ADVANCE(48);
      if (lookahead == ';') ADVANCE(111);
      if (lookahead == '<') ADVANCE(67);
      if (lookahead == '=') ADVANCE(55);
      if (lookahead == '>') ADVANCE(72);
      if (lookahead == '@') ADVANCE(52);
      if (lookahead == '[') ADVANCE(53);
      if (lookahead == '\\') SKIP(2)
      if (lookahead == ']') ADVANCE(54);
      if (lookahead == '^') ADVANCE(64);
      if (lookahead == '|') ADVANCE(62);
      if (lookahead == '}') ADVANCE(89);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) SKIP(10)
      if (('A' <= lookahead && lookahead <= '_') ||
          ('a' <= lookahead && lookahead <= 'z') ||
          (913 <= lookahead && lookahead <= 937) ||
          (945 <= lookahead && lookahead <= 969)) ADVANCE(109);
      END_STATE();
    case 11:
      if (lookahead == '#') ADVANCE(110);
      if (lookahead == '-') ADVANCE(15);
      if (lookahead == ':') ADVANCE(48);
      if (lookahead == '\\') SKIP(6)
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) SKIP(11)
      END_STATE();
    case 12:
      if (lookahead == '#') ADVANCE(110);
      if (lookahead == '\\') ADVANCE(92);
      if (lookahead == '{') ADVANCE(88);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) SKIP(12)
      END_STATE();
    case 13:
      if (lookahead == '.') ADVANCE(87);
      END_STATE();
    case 14:
      if (lookahead == '=') ADVANCE(70);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(96);
      END_STATE();
    case 15:
      if (lookahead == '>') ADVANCE(50);
      END_STATE();
    case 16:
      if (lookahead == '_') ADVANCE(20);
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(101);
      END_STATE();
    case 17:
      if (lookahead == '_') ADVANCE(21);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(102);
      END_STATE();
    case 18:
      if (lookahead == '_') ADVANCE(25);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(103);
      END_STATE();
    case 19:
      if (lookahead == '+' ||
          lookahead == '-') ADVANCE(22);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(106);
      END_STATE();
    case 20:
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(101);
      END_STATE();
    case 21:
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(102);
      END_STATE();
    case 22:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(106);
      END_STATE();
    case 23:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(90);
      END_STATE();
    case 24:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(23);
      END_STATE();
    case 25:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(103);
      END_STATE();
    case 26:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(90);
      END_STATE();
    case 27:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(26);
      END_STATE();
    case 28:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(27);
      END_STATE();
    case 29:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(28);
      END_STATE();
    case 30:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(29);
      END_STATE();
    case 31:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(30);
      END_STATE();
    case 32:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(31);
      END_STATE();
    case 33:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(32);
      END_STATE();
    case 34:
      if (eof) ADVANCE(39);
      if (lookahead == '\n') SKIP(0)
      END_STATE();
    case 35:
      if (eof) ADVANCE(39);
      if (lookahead == '\n') SKIP(0)
      if (lookahead == '\r') SKIP(34)
      END_STATE();
    case 36:
      if (eof) ADVANCE(39);
      if (lookahead == '\n') SKIP(38)
      END_STATE();
    case 37:
      if (eof) ADVANCE(39);
      if (lookahead == '\n') SKIP(38)
      if (lookahead == '\r') SKIP(36)
      END_STATE();
    case 38:
      if (eof) ADVANCE(39);
      if (lookahead == '!') ADVANCE(14);
      if (lookahead == '#') ADVANCE(110);
      if (lookahead == '%') ADVANCE(60);
      if (lookahead == '&') ADVANCE(63);
      if (lookahead == '(') ADVANCE(42);
      if (lookahead == ')') ADVANCE(43);
      if (lookahead == '*') ADVANCE(45);
      if (lookahead == '+') ADVANCE(56);
      if (lookahead == ',') ADVANCE(44);
      if (lookahead == '-') ADVANCE(57);
      if (lookahead == '.') ADVANCE(41);
      if (lookahead == '/') ADVANCE(59);
      if (lookahead == '0') ADVANCE(99);
      if (lookahead == ':') ADVANCE(48);
      if (lookahead == ';') ADVANCE(111);
      if (lookahead == '<') ADVANCE(67);
      if (lookahead == '=') ADVANCE(55);
      if (lookahead == '>') ADVANCE(72);
      if (lookahead == '@') ADVANCE(52);
      if (lookahead == '[') ADVANCE(53);
      if (lookahead == '\\') SKIP(37)
      if (lookahead == ']') ADVANCE(54);
      if (lookahead == '^') ADVANCE(64);
      if (lookahead == '{') ADVANCE(88);
      if (lookahead == '|') ADVANCE(62);
      if (lookahead == '}') ADVANCE(89);
      if (lookahead == '~') ADVANCE(66);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) SKIP(38)
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(98);
      if (('A' <= lookahead && lookahead <= '_') ||
          ('a' <= lookahead && lookahead <= 'z') ||
          (913 <= lookahead && lookahead <= 937) ||
          (945 <= lookahead && lookahead <= 969)) ADVANCE(109);
      END_STATE();
    case 39:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 40:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 41:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(13);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(105);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 45:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '*') ADVANCE(51);
      if (lookahead == '=') ADVANCE(76);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '=') ADVANCE(82);
      END_STATE();
    case 47:
      ACCEPT_TOKEN(anon_sym_COLON_EQ);
      END_STATE();
    case 48:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 49:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == '=') ADVANCE(47);
      END_STATE();
    case 50:
      ACCEPT_TOKEN(anon_sym_DASH_GT);
      END_STATE();
    case 51:
      ACCEPT_TOKEN(anon_sym_STAR_STAR);
      if (lookahead == '=') ADVANCE(81);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(anon_sym_AT);
      if (lookahead == '=') ADVANCE(78);
      END_STATE();
    case 53:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 54:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 55:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(69);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '=') ADVANCE(74);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '=') ADVANCE(75);
      END_STATE();
    case 58:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '=') ADVANCE(75);
      if (lookahead == '>') ADVANCE(50);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '/') ADVANCE(61);
      if (lookahead == '=') ADVANCE(77);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      if (lookahead == '=') ADVANCE(80);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH);
      if (lookahead == '=') ADVANCE(79);
      END_STATE();
    case 62:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '=') ADVANCE(86);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '=') ADVANCE(84);
      END_STATE();
    case 64:
      ACCEPT_TOKEN(anon_sym_CARET);
      if (lookahead == '=') ADVANCE(85);
      END_STATE();
    case 65:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      if (lookahead == '=') ADVANCE(83);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(65);
      if (lookahead == '=') ADVANCE(68);
      if (lookahead == '>') ADVANCE(73);
      END_STATE();
    case 68:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(71);
      if (lookahead == '>') ADVANCE(46);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_LT_GT);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(anon_sym_PLUS_EQ);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_DASH_EQ);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(anon_sym_STAR_EQ);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_AT_EQ);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_SLASH_SLASH_EQ);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(anon_sym_PERCENT_EQ);
      END_STATE();
    case 81:
      ACCEPT_TOKEN(anon_sym_STAR_STAR_EQ);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_GT_GT_EQ);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_LT_LT_EQ);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_AMP_EQ);
      END_STATE();
    case 85:
      ACCEPT_TOKEN(anon_sym_CARET_EQ);
      END_STATE();
    case 86:
      ACCEPT_TOKEN(anon_sym_PIPE_EQ);
      END_STATE();
    case 87:
      ACCEPT_TOKEN(sym_ellipsis);
      END_STATE();
    case 88:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(sym_escape_sequence);
      END_STATE();
    case 91:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (lookahead == '\\') ADVANCE(92);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(sym__not_escape_sequence);
      if (lookahead == '\n') ADVANCE(91);
      if (lookahead == '\r') ADVANCE(3);
      if (lookahead == 'U') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(29);
      if (lookahead == 'x') ADVANCE(27);
      if (lookahead == '"' ||
          lookahead == '\'' ||
          lookahead == '\\' ||
          lookahead == 'a' ||
          lookahead == 'b' ||
          lookahead == 'f' ||
          lookahead == 'n' ||
          lookahead == 'r' ||
          ('t' <= lookahead && lookahead <= 'v')) ADVANCE(90);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(24);
      END_STATE();
    case 93:
      ACCEPT_TOKEN(aux_sym_format_specifier_token1);
      if (lookahead == '\r') ADVANCE(95);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '{' &&
          lookahead != '}') ADVANCE(95);
      END_STATE();
    case 94:
      ACCEPT_TOKEN(aux_sym_format_specifier_token1);
      if (lookahead == '#') ADVANCE(95);
      if (lookahead == '\\') ADVANCE(93);
      if (lookahead == '\t' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) ADVANCE(94);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '{' &&
          lookahead != '}') ADVANCE(95);
      END_STATE();
    case 95:
      ACCEPT_TOKEN(aux_sym_format_specifier_token1);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '{' &&
          lookahead != '}') ADVANCE(95);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(sym_type_conversion);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(sym_integer);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '.') ADVANCE(107);
      if (lookahead == '_') ADVANCE(100);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(19);
      if (lookahead == 'J' ||
          lookahead == 'L' ||
          lookahead == 'j' ||
          lookahead == 'l') ADVANCE(97);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(98);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '.') ADVANCE(107);
      if (lookahead == 'B' ||
          lookahead == 'b') ADVANCE(16);
      if (lookahead == 'O' ||
          lookahead == 'o') ADVANCE(17);
      if (lookahead == 'X' ||
          lookahead == 'x') ADVANCE(18);
      if (lookahead == '_') ADVANCE(100);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(19);
      if (lookahead == 'J' ||
          lookahead == 'L' ||
          lookahead == 'j' ||
          lookahead == 'l') ADVANCE(97);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(98);
      END_STATE();
    case 100:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '.') ADVANCE(107);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(19);
      if (lookahead == 'J' ||
          lookahead == 'L' ||
          lookahead == 'j' ||
          lookahead == 'l') ADVANCE(97);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(98);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '_') ADVANCE(20);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(97);
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(101);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '_') ADVANCE(21);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(97);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(102);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '_') ADVANCE(25);
      if (lookahead == 'L' ||
          lookahead == 'l') ADVANCE(97);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(103);
      END_STATE();
    case 104:
      ACCEPT_TOKEN(sym_float);
      END_STATE();
    case 105:
      ACCEPT_TOKEN(sym_float);
      if (lookahead == '_') ADVANCE(107);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(19);
      if (lookahead == 'J' ||
          lookahead == 'L' ||
          lookahead == 'j' ||
          lookahead == 'l') ADVANCE(104);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(105);
      END_STATE();
    case 106:
      ACCEPT_TOKEN(sym_float);
      if (lookahead == '_') ADVANCE(108);
      if (lookahead == 'J' ||
          lookahead == 'L' ||
          lookahead == 'j' ||
          lookahead == 'l') ADVANCE(104);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(106);
      END_STATE();
    case 107:
      ACCEPT_TOKEN(sym_float);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(19);
      if (lookahead == 'J' ||
          lookahead == 'L' ||
          lookahead == 'j' ||
          lookahead == 'l') ADVANCE(104);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(105);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(sym_float);
      if (lookahead == 'J' ||
          lookahead == 'L' ||
          lookahead == 'j' ||
          lookahead == 'l') ADVANCE(104);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(106);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(sym_identifier);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z') ||
          (913 <= lookahead && lookahead <= 937) ||
          (945 <= lookahead && lookahead <= 969)) ADVANCE(109);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(110);
      END_STATE();
    case 111:
      ACCEPT_TOKEN(sym__semicolon);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (lookahead == 'F') ADVANCE(1);
      if (lookahead == 'N') ADVANCE(2);
      if (lookahead == 'T') ADVANCE(3);
      if (lookahead == '\\') SKIP(4)
      if (lookahead == '_') ADVANCE(5);
      if (lookahead == 'a') ADVANCE(6);
      if (lookahead == 'b') ADVANCE(7);
      if (lookahead == 'c') ADVANCE(8);
      if (lookahead == 'd') ADVANCE(9);
      if (lookahead == 'e') ADVANCE(10);
      if (lookahead == 'f') ADVANCE(11);
      if (lookahead == 'g') ADVANCE(12);
      if (lookahead == 'i') ADVANCE(13);
      if (lookahead == 'l') ADVANCE(14);
      if (lookahead == 'n') ADVANCE(15);
      if (lookahead == 'o') ADVANCE(16);
      if (lookahead == 'p') ADVANCE(17);
      if (lookahead == 'r') ADVANCE(18);
      if (lookahead == 't') ADVANCE(19);
      if (lookahead == 'w') ADVANCE(20);
      if (lookahead == 'y') ADVANCE(21);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\f' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279) SKIP(0)
      END_STATE();
    case 1:
      if (lookahead == 'a') ADVANCE(22);
      END_STATE();
    case 2:
      if (lookahead == 'o') ADVANCE(23);
      END_STATE();
    case 3:
      if (lookahead == 'r') ADVANCE(24);
      END_STATE();
    case 4:
      if (lookahead == '\n') SKIP(0)
      if (lookahead == '\r') SKIP(25)
      END_STATE();
    case 5:
      if (lookahead == '_') ADVANCE(26);
      END_STATE();
    case 6:
      if (lookahead == 'n') ADVANCE(27);
      if (lookahead == 's') ADVANCE(28);
      if (lookahead == 'w') ADVANCE(29);
      END_STATE();
    case 7:
      if (lookahead == 'r') ADVANCE(30);
      END_STATE();
    case 8:
      if (lookahead == 'l') ADVANCE(31);
      if (lookahead == 'o') ADVANCE(32);
      END_STATE();
    case 9:
      if (lookahead == 'e') ADVANCE(33);
      END_STATE();
    case 10:
      if (lookahead == 'l') ADVANCE(34);
      if (lookahead == 'x') ADVANCE(35);
      END_STATE();
    case 11:
      if (lookahead == 'i') ADVANCE(36);
      if (lookahead == 'o') ADVANCE(37);
      if (lookahead == 'r') ADVANCE(38);
      END_STATE();
    case 12:
      if (lookahead == 'l') ADVANCE(39);
      END_STATE();
    case 13:
      if (lookahead == 'f') ADVANCE(40);
      if (lookahead == 'm') ADVANCE(41);
      if (lookahead == 'n') ADVANCE(42);
      if (lookahead == 's') ADVANCE(43);
      END_STATE();
    case 14:
      if (lookahead == 'a') ADVANCE(44);
      END_STATE();
    case 15:
      if (lookahead == 'o') ADVANCE(45);
      END_STATE();
    case 16:
      if (lookahead == 'r') ADVANCE(46);
      END_STATE();
    case 17:
      if (lookahead == 'a') ADVANCE(47);
      if (lookahead == 'r') ADVANCE(48);
      END_STATE();
    case 18:
      if (lookahead == 'a') ADVANCE(49);
      if (lookahead == 'e') ADVANCE(50);
      END_STATE();
    case 19:
      if (lookahead == 'r') ADVANCE(51);
      END_STATE();
    case 20:
      if (lookahead == 'h') ADVANCE(52);
      if (lookahead == 'i') ADVANCE(53);
      END_STATE();
    case 21:
      if (lookahead == 'i') ADVANCE(54);
      END_STATE();
    case 22:
      if (lookahead == 'l') ADVANCE(55);
      END_STATE();
    case 23:
      if (lookahead == 'n') ADVANCE(56);
      END_STATE();
    case 24:
      if (lookahead == 'u') ADVANCE(57);
      END_STATE();
    case 25:
      if (lookahead == '\n') SKIP(0)
      END_STATE();
    case 26:
      if (lookahead == 'f') ADVANCE(58);
      END_STATE();
    case 27:
      if (lookahead == 'd') ADVANCE(59);
      END_STATE();
    case 28:
      ACCEPT_TOKEN(anon_sym_as);
      if (lookahead == 's') ADVANCE(60);
      if (lookahead == 'y') ADVANCE(61);
      END_STATE();
    case 29:
      if (lookahead == 'a') ADVANCE(62);
      END_STATE();
    case 30:
      if (lookahead == 'e') ADVANCE(63);
      END_STATE();
    case 31:
      if (lookahead == 'a') ADVANCE(64);
      END_STATE();
    case 32:
      if (lookahead == 'n') ADVANCE(65);
      END_STATE();
    case 33:
      if (lookahead == 'f') ADVANCE(66);
      if (lookahead == 'l') ADVANCE(67);
      END_STATE();
    case 34:
      if (lookahead == 'i') ADVANCE(68);
      if (lookahead == 's') ADVANCE(69);
      END_STATE();
    case 35:
      if (lookahead == 'c') ADVANCE(70);
      if (lookahead == 'e') ADVANCE(71);
      END_STATE();
    case 36:
      if (lookahead == 'n') ADVANCE(72);
      END_STATE();
    case 37:
      if (lookahead == 'r') ADVANCE(73);
      END_STATE();
    case 38:
      if (lookahead == 'o') ADVANCE(74);
      END_STATE();
    case 39:
      if (lookahead == 'o') ADVANCE(75);
      END_STATE();
    case 40:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 41:
      if (lookahead == 'p') ADVANCE(76);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_in);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_is);
      END_STATE();
    case 44:
      if (lookahead == 'm') ADVANCE(77);
      END_STATE();
    case 45:
      if (lookahead == 'n') ADVANCE(78);
      if (lookahead == 't') ADVANCE(79);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(anon_sym_or);
      END_STATE();
    case 47:
      if (lookahead == 's') ADVANCE(80);
      END_STATE();
    case 48:
      if (lookahead == 'i') ADVANCE(81);
      END_STATE();
    case 49:
      if (lookahead == 'i') ADVANCE(82);
      END_STATE();
    case 50:
      if (lookahead == 't') ADVANCE(83);
      END_STATE();
    case 51:
      if (lookahead == 'y') ADVANCE(84);
      END_STATE();
    case 52:
      if (lookahead == 'i') ADVANCE(85);
      END_STATE();
    case 53:
      if (lookahead == 't') ADVANCE(86);
      END_STATE();
    case 54:
      if (lookahead == 'e') ADVANCE(87);
      END_STATE();
    case 55:
      if (lookahead == 's') ADVANCE(88);
      END_STATE();
    case 56:
      if (lookahead == 'e') ADVANCE(89);
      END_STATE();
    case 57:
      if (lookahead == 'e') ADVANCE(90);
      END_STATE();
    case 58:
      if (lookahead == 'u') ADVANCE(91);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(anon_sym_and);
      END_STATE();
    case 60:
      if (lookahead == 'e') ADVANCE(92);
      END_STATE();
    case 61:
      if (lookahead == 'n') ADVANCE(93);
      END_STATE();
    case 62:
      if (lookahead == 'i') ADVANCE(94);
      END_STATE();
    case 63:
      if (lookahead == 'a') ADVANCE(95);
      END_STATE();
    case 64:
      if (lookahead == 's') ADVANCE(96);
      END_STATE();
    case 65:
      if (lookahead == 't') ADVANCE(97);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_def);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_del);
      END_STATE();
    case 68:
      if (lookahead == 'f') ADVANCE(98);
      END_STATE();
    case 69:
      if (lookahead == 'e') ADVANCE(99);
      END_STATE();
    case 70:
      if (lookahead == 'e') ADVANCE(100);
      END_STATE();
    case 71:
      if (lookahead == 'c') ADVANCE(101);
      END_STATE();
    case 72:
      if (lookahead == 'a') ADVANCE(102);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_for);
      END_STATE();
    case 74:
      if (lookahead == 'm') ADVANCE(103);
      END_STATE();
    case 75:
      if (lookahead == 'b') ADVANCE(104);
      END_STATE();
    case 76:
      if (lookahead == 'o') ADVANCE(105);
      END_STATE();
    case 77:
      if (lookahead == 'b') ADVANCE(106);
      END_STATE();
    case 78:
      if (lookahead == 'l') ADVANCE(107);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_not);
      END_STATE();
    case 80:
      if (lookahead == 's') ADVANCE(108);
      END_STATE();
    case 81:
      if (lookahead == 'n') ADVANCE(109);
      END_STATE();
    case 82:
      if (lookahead == 's') ADVANCE(110);
      END_STATE();
    case 83:
      if (lookahead == 'u') ADVANCE(111);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_try);
      END_STATE();
    case 85:
      if (lookahead == 'l') ADVANCE(112);
      END_STATE();
    case 86:
      if (lookahead == 'h') ADVANCE(113);
      END_STATE();
    case 87:
      if (lookahead == 'l') ADVANCE(114);
      END_STATE();
    case 88:
      if (lookahead == 'e') ADVANCE(115);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(sym_none);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(sym_true);
      END_STATE();
    case 91:
      if (lookahead == 't') ADVANCE(116);
      END_STATE();
    case 92:
      if (lookahead == 'r') ADVANCE(117);
      END_STATE();
    case 93:
      if (lookahead == 'c') ADVANCE(118);
      END_STATE();
    case 94:
      if (lookahead == 't') ADVANCE(119);
      END_STATE();
    case 95:
      if (lookahead == 'k') ADVANCE(120);
      END_STATE();
    case 96:
      if (lookahead == 's') ADVANCE(121);
      END_STATE();
    case 97:
      if (lookahead == 'i') ADVANCE(122);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_elif);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 100:
      if (lookahead == 'p') ADVANCE(123);
      END_STATE();
    case 101:
      ACCEPT_TOKEN(anon_sym_exec);
      END_STATE();
    case 102:
      if (lookahead == 'l') ADVANCE(124);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(anon_sym_from);
      END_STATE();
    case 104:
      if (lookahead == 'a') ADVANCE(125);
      END_STATE();
    case 105:
      if (lookahead == 'r') ADVANCE(126);
      END_STATE();
    case 106:
      if (lookahead == 'd') ADVANCE(127);
      END_STATE();
    case 107:
      if (lookahead == 'o') ADVANCE(128);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_pass);
      END_STATE();
    case 109:
      if (lookahead == 't') ADVANCE(129);
      END_STATE();
    case 110:
      if (lookahead == 'e') ADVANCE(130);
      END_STATE();
    case 111:
      if (lookahead == 'r') ADVANCE(131);
      END_STATE();
    case 112:
      if (lookahead == 'e') ADVANCE(132);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_with);
      END_STATE();
    case 114:
      if (lookahead == 'd') ADVANCE(133);
      END_STATE();
    case 115:
      ACCEPT_TOKEN(sym_false);
      END_STATE();
    case 116:
      if (lookahead == 'u') ADVANCE(134);
      END_STATE();
    case 117:
      if (lookahead == 't') ADVANCE(135);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_async);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_await);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_class);
      END_STATE();
    case 122:
      if (lookahead == 'n') ADVANCE(136);
      END_STATE();
    case 123:
      if (lookahead == 't') ADVANCE(137);
      END_STATE();
    case 124:
      if (lookahead == 'l') ADVANCE(138);
      END_STATE();
    case 125:
      if (lookahead == 'l') ADVANCE(139);
      END_STATE();
    case 126:
      if (lookahead == 't') ADVANCE(140);
      END_STATE();
    case 127:
      if (lookahead == 'a') ADVANCE(141);
      END_STATE();
    case 128:
      if (lookahead == 'c') ADVANCE(142);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_print);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_raise);
      END_STATE();
    case 131:
      if (lookahead == 'n') ADVANCE(143);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(anon_sym_while);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_yield);
      END_STATE();
    case 134:
      if (lookahead == 'r') ADVANCE(144);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(anon_sym_assert);
      END_STATE();
    case 136:
      if (lookahead == 'u') ADVANCE(145);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(anon_sym_except);
      END_STATE();
    case 138:
      if (lookahead == 'y') ADVANCE(146);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(anon_sym_global);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(anon_sym_import);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(anon_sym_lambda);
      END_STATE();
    case 142:
      if (lookahead == 'a') ADVANCE(147);
      END_STATE();
    case 143:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 144:
      if (lookahead == 'e') ADVANCE(148);
      END_STATE();
    case 145:
      if (lookahead == 'e') ADVANCE(149);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(anon_sym_finally);
      END_STATE();
    case 147:
      if (lookahead == 'l') ADVANCE(150);
      END_STATE();
    case 148:
      if (lookahead == '_') ADVANCE(151);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(anon_sym_continue);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(anon_sym_nonlocal);
      END_STATE();
    case 151:
      if (lookahead == '_') ADVANCE(152);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(anon_sym___future__);
      END_STATE();
    default:
      return false;
  }
}

static TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 38, .external_lex_state = 2},
  [2] = {.lex_state = 38, .external_lex_state = 3},
  [3] = {.lex_state = 38, .external_lex_state = 3},
  [4] = {.lex_state = 38, .external_lex_state = 3},
  [5] = {.lex_state = 38, .external_lex_state = 3},
  [6] = {.lex_state = 38, .external_lex_state = 3},
  [7] = {.lex_state = 38, .external_lex_state = 3},
  [8] = {.lex_state = 38, .external_lex_state = 3},
  [9] = {.lex_state = 38, .external_lex_state = 3},
  [10] = {.lex_state = 38, .external_lex_state = 3},
  [11] = {.lex_state = 38, .external_lex_state = 3},
  [12] = {.lex_state = 38, .external_lex_state = 3},
  [13] = {.lex_state = 38, .external_lex_state = 3},
  [14] = {.lex_state = 38, .external_lex_state = 3},
  [15] = {.lex_state = 38, .external_lex_state = 3},
  [16] = {.lex_state = 38, .external_lex_state = 3},
  [17] = {.lex_state = 38, .external_lex_state = 3},
  [18] = {.lex_state = 38, .external_lex_state = 3},
  [19] = {.lex_state = 38, .external_lex_state = 3},
  [20] = {.lex_state = 38, .external_lex_state = 3},
  [21] = {.lex_state = 38, .external_lex_state = 3},
  [22] = {.lex_state = 38, .external_lex_state = 3},
  [23] = {.lex_state = 38, .external_lex_state = 3},
  [24] = {.lex_state = 38, .external_lex_state = 3},
  [25] = {.lex_state = 38, .external_lex_state = 3},
  [26] = {.lex_state = 38, .external_lex_state = 3},
  [27] = {.lex_state = 38, .external_lex_state = 3},
  [28] = {.lex_state = 38, .external_lex_state = 3},
  [29] = {.lex_state = 38, .external_lex_state = 3},
  [30] = {.lex_state = 38, .external_lex_state = 3},
  [31] = {.lex_state = 38, .external_lex_state = 3},
  [32] = {.lex_state = 38, .external_lex_state = 3},
  [33] = {.lex_state = 38, .external_lex_state = 3},
  [34] = {.lex_state = 38, .external_lex_state = 3},
  [35] = {.lex_state = 38, .external_lex_state = 3},
  [36] = {.lex_state = 38, .external_lex_state = 3},
  [37] = {.lex_state = 38, .external_lex_state = 3},
  [38] = {.lex_state = 38, .external_lex_state = 3},
  [39] = {.lex_state = 38, .external_lex_state = 3},
  [40] = {.lex_state = 38, .external_lex_state = 3},
  [41] = {.lex_state = 38, .external_lex_state = 3},
  [42] = {.lex_state = 38, .external_lex_state = 2},
  [43] = {.lex_state = 38, .external_lex_state = 3},
  [44] = {.lex_state = 38, .external_lex_state = 3},
  [45] = {.lex_state = 38, .external_lex_state = 2},
  [46] = {.lex_state = 38, .external_lex_state = 4},
  [47] = {.lex_state = 38, .external_lex_state = 4},
  [48] = {.lex_state = 38, .external_lex_state = 5},
  [49] = {.lex_state = 38, .external_lex_state = 5},
  [50] = {.lex_state = 38, .external_lex_state = 5},
  [51] = {.lex_state = 38, .external_lex_state = 5},
  [52] = {.lex_state = 38, .external_lex_state = 5},
  [53] = {.lex_state = 38, .external_lex_state = 5},
  [54] = {.lex_state = 38, .external_lex_state = 5},
  [55] = {.lex_state = 38, .external_lex_state = 5},
  [56] = {.lex_state = 38, .external_lex_state = 5},
  [57] = {.lex_state = 38, .external_lex_state = 5},
  [58] = {.lex_state = 38, .external_lex_state = 5},
  [59] = {.lex_state = 38, .external_lex_state = 5},
  [60] = {.lex_state = 38, .external_lex_state = 5},
  [61] = {.lex_state = 38, .external_lex_state = 5},
  [62] = {.lex_state = 38, .external_lex_state = 5},
  [63] = {.lex_state = 38, .external_lex_state = 5},
  [64] = {.lex_state = 38, .external_lex_state = 5},
  [65] = {.lex_state = 38, .external_lex_state = 5},
  [66] = {.lex_state = 38, .external_lex_state = 5},
  [67] = {.lex_state = 38, .external_lex_state = 5},
  [68] = {.lex_state = 38, .external_lex_state = 5},
  [69] = {.lex_state = 38, .external_lex_state = 5},
  [70] = {.lex_state = 38, .external_lex_state = 5},
  [71] = {.lex_state = 38, .external_lex_state = 5},
  [72] = {.lex_state = 38, .external_lex_state = 5},
  [73] = {.lex_state = 38, .external_lex_state = 5},
  [74] = {.lex_state = 38, .external_lex_state = 5},
  [75] = {.lex_state = 38, .external_lex_state = 5},
  [76] = {.lex_state = 38, .external_lex_state = 5},
  [77] = {.lex_state = 38, .external_lex_state = 5},
  [78] = {.lex_state = 38, .external_lex_state = 5},
  [79] = {.lex_state = 38, .external_lex_state = 5},
  [80] = {.lex_state = 38, .external_lex_state = 5},
  [81] = {.lex_state = 38, .external_lex_state = 5},
  [82] = {.lex_state = 38, .external_lex_state = 5},
  [83] = {.lex_state = 38, .external_lex_state = 5},
  [84] = {.lex_state = 38, .external_lex_state = 5},
  [85] = {.lex_state = 38, .external_lex_state = 5},
  [86] = {.lex_state = 38, .external_lex_state = 4},
  [87] = {.lex_state = 38, .external_lex_state = 4},
  [88] = {.lex_state = 38, .external_lex_state = 4},
  [89] = {.lex_state = 38, .external_lex_state = 4},
  [90] = {.lex_state = 38, .external_lex_state = 4},
  [91] = {.lex_state = 38, .external_lex_state = 4},
  [92] = {.lex_state = 38, .external_lex_state = 2},
  [93] = {.lex_state = 38, .external_lex_state = 2},
  [94] = {.lex_state = 38, .external_lex_state = 2},
  [95] = {.lex_state = 38, .external_lex_state = 4},
  [96] = {.lex_state = 38, .external_lex_state = 2},
  [97] = {.lex_state = 38, .external_lex_state = 2},
  [98] = {.lex_state = 38, .external_lex_state = 2},
  [99] = {.lex_state = 38, .external_lex_state = 2},
  [100] = {.lex_state = 38, .external_lex_state = 2},
  [101] = {.lex_state = 38, .external_lex_state = 2},
  [102] = {.lex_state = 38, .external_lex_state = 2},
  [103] = {.lex_state = 38, .external_lex_state = 2},
  [104] = {.lex_state = 38, .external_lex_state = 2},
  [105] = {.lex_state = 38, .external_lex_state = 2},
  [106] = {.lex_state = 38, .external_lex_state = 2},
  [107] = {.lex_state = 38, .external_lex_state = 2},
  [108] = {.lex_state = 38, .external_lex_state = 2},
  [109] = {.lex_state = 38, .external_lex_state = 2},
  [110] = {.lex_state = 38, .external_lex_state = 2},
  [111] = {.lex_state = 38, .external_lex_state = 2},
  [112] = {.lex_state = 38, .external_lex_state = 2},
  [113] = {.lex_state = 38, .external_lex_state = 2},
  [114] = {.lex_state = 38, .external_lex_state = 2},
  [115] = {.lex_state = 38, .external_lex_state = 2},
  [116] = {.lex_state = 10},
  [117] = {.lex_state = 10},
  [118] = {.lex_state = 38, .external_lex_state = 2},
  [119] = {.lex_state = 38, .external_lex_state = 2},
  [120] = {.lex_state = 38, .external_lex_state = 2},
  [121] = {.lex_state = 10},
  [122] = {.lex_state = 38, .external_lex_state = 2},
  [123] = {.lex_state = 38, .external_lex_state = 2},
  [124] = {.lex_state = 38, .external_lex_state = 2},
  [125] = {.lex_state = 10},
  [126] = {.lex_state = 38, .external_lex_state = 2},
  [127] = {.lex_state = 38, .external_lex_state = 2},
  [128] = {.lex_state = 38, .external_lex_state = 2},
  [129] = {.lex_state = 38, .external_lex_state = 2},
  [130] = {.lex_state = 38, .external_lex_state = 2},
  [131] = {.lex_state = 38, .external_lex_state = 2},
  [132] = {.lex_state = 38, .external_lex_state = 2},
  [133] = {.lex_state = 38, .external_lex_state = 2},
  [134] = {.lex_state = 38, .external_lex_state = 2},
  [135] = {.lex_state = 38, .external_lex_state = 2},
  [136] = {.lex_state = 38, .external_lex_state = 2},
  [137] = {.lex_state = 38, .external_lex_state = 2},
  [138] = {.lex_state = 38, .external_lex_state = 2},
  [139] = {.lex_state = 38, .external_lex_state = 2},
  [140] = {.lex_state = 38, .external_lex_state = 2},
  [141] = {.lex_state = 38, .external_lex_state = 2},
  [142] = {.lex_state = 38, .external_lex_state = 2},
  [143] = {.lex_state = 38, .external_lex_state = 2},
  [144] = {.lex_state = 38, .external_lex_state = 2},
  [145] = {.lex_state = 38, .external_lex_state = 2},
  [146] = {.lex_state = 38, .external_lex_state = 2},
  [147] = {.lex_state = 38, .external_lex_state = 2},
  [148] = {.lex_state = 38, .external_lex_state = 2},
  [149] = {.lex_state = 38, .external_lex_state = 2},
  [150] = {.lex_state = 38, .external_lex_state = 2},
  [151] = {.lex_state = 38, .external_lex_state = 2},
  [152] = {.lex_state = 38, .external_lex_state = 2},
  [153] = {.lex_state = 38, .external_lex_state = 2},
  [154] = {.lex_state = 10, .external_lex_state = 6},
  [155] = {.lex_state = 38, .external_lex_state = 2},
  [156] = {.lex_state = 38, .external_lex_state = 2},
  [157] = {.lex_state = 10, .external_lex_state = 6},
  [158] = {.lex_state = 38, .external_lex_state = 2},
  [159] = {.lex_state = 38, .external_lex_state = 2},
  [160] = {.lex_state = 38, .external_lex_state = 2},
  [161] = {.lex_state = 38, .external_lex_state = 4},
  [162] = {.lex_state = 38, .external_lex_state = 4},
  [163] = {.lex_state = 38, .external_lex_state = 2},
  [164] = {.lex_state = 38, .external_lex_state = 2},
  [165] = {.lex_state = 38, .external_lex_state = 3},
  [166] = {.lex_state = 10, .external_lex_state = 4},
  [167] = {.lex_state = 38, .external_lex_state = 2},
  [168] = {.lex_state = 38, .external_lex_state = 2},
  [169] = {.lex_state = 38, .external_lex_state = 2},
  [170] = {.lex_state = 38, .external_lex_state = 2},
  [171] = {.lex_state = 38, .external_lex_state = 3},
  [172] = {.lex_state = 10, .external_lex_state = 6},
  [173] = {.lex_state = 38, .external_lex_state = 2},
  [174] = {.lex_state = 10, .external_lex_state = 6},
  [175] = {.lex_state = 9, .external_lex_state = 6},
  [176] = {.lex_state = 38, .external_lex_state = 2},
  [177] = {.lex_state = 38, .external_lex_state = 2},
  [178] = {.lex_state = 38, .external_lex_state = 2},
  [179] = {.lex_state = 38, .external_lex_state = 2},
  [180] = {.lex_state = 38, .external_lex_state = 4},
  [181] = {.lex_state = 38, .external_lex_state = 2},
  [182] = {.lex_state = 10, .external_lex_state = 6},
  [183] = {.lex_state = 38, .external_lex_state = 4},
  [184] = {.lex_state = 38, .external_lex_state = 4},
  [185] = {.lex_state = 10, .external_lex_state = 6},
  [186] = {.lex_state = 38, .external_lex_state = 2},
  [187] = {.lex_state = 38, .external_lex_state = 2},
  [188] = {.lex_state = 38, .external_lex_state = 2},
  [189] = {.lex_state = 38, .external_lex_state = 4},
  [190] = {.lex_state = 38, .external_lex_state = 2},
  [191] = {.lex_state = 38, .external_lex_state = 2},
  [192] = {.lex_state = 38, .external_lex_state = 2},
  [193] = {.lex_state = 38, .external_lex_state = 2},
  [194] = {.lex_state = 38, .external_lex_state = 2},
  [195] = {.lex_state = 38, .external_lex_state = 3},
  [196] = {.lex_state = 38, .external_lex_state = 3},
  [197] = {.lex_state = 38, .external_lex_state = 2},
  [198] = {.lex_state = 38, .external_lex_state = 2},
  [199] = {.lex_state = 38, .external_lex_state = 2},
  [200] = {.lex_state = 38, .external_lex_state = 4},
  [201] = {.lex_state = 10, .external_lex_state = 6},
  [202] = {.lex_state = 38, .external_lex_state = 2},
  [203] = {.lex_state = 38, .external_lex_state = 4},
  [204] = {.lex_state = 38, .external_lex_state = 4},
  [205] = {.lex_state = 38, .external_lex_state = 3},
  [206] = {.lex_state = 38, .external_lex_state = 3},
  [207] = {.lex_state = 38, .external_lex_state = 2},
  [208] = {.lex_state = 10, .external_lex_state = 6},
  [209] = {.lex_state = 38, .external_lex_state = 2},
  [210] = {.lex_state = 38, .external_lex_state = 2},
  [211] = {.lex_state = 38, .external_lex_state = 2},
  [212] = {.lex_state = 38, .external_lex_state = 2},
  [213] = {.lex_state = 38, .external_lex_state = 4},
  [214] = {.lex_state = 38, .external_lex_state = 4},
  [215] = {.lex_state = 38, .external_lex_state = 2},
  [216] = {.lex_state = 10, .external_lex_state = 6},
  [217] = {.lex_state = 38, .external_lex_state = 2},
  [218] = {.lex_state = 38, .external_lex_state = 3},
  [219] = {.lex_state = 38, .external_lex_state = 2},
  [220] = {.lex_state = 38, .external_lex_state = 2},
  [221] = {.lex_state = 10, .external_lex_state = 6},
  [222] = {.lex_state = 38, .external_lex_state = 2},
  [223] = {.lex_state = 38, .external_lex_state = 2},
  [224] = {.lex_state = 38, .external_lex_state = 2},
  [225] = {.lex_state = 38, .external_lex_state = 2},
  [226] = {.lex_state = 38, .external_lex_state = 2},
  [227] = {.lex_state = 38, .external_lex_state = 2},
  [228] = {.lex_state = 38, .external_lex_state = 2},
  [229] = {.lex_state = 38, .external_lex_state = 2},
  [230] = {.lex_state = 38, .external_lex_state = 3},
  [231] = {.lex_state = 38, .external_lex_state = 2},
  [232] = {.lex_state = 38, .external_lex_state = 2},
  [233] = {.lex_state = 38, .external_lex_state = 2},
  [234] = {.lex_state = 38, .external_lex_state = 2},
  [235] = {.lex_state = 38, .external_lex_state = 2},
  [236] = {.lex_state = 38, .external_lex_state = 2},
  [237] = {.lex_state = 38, .external_lex_state = 3},
  [238] = {.lex_state = 38, .external_lex_state = 3},
  [239] = {.lex_state = 38, .external_lex_state = 2},
  [240] = {.lex_state = 38, .external_lex_state = 2},
  [241] = {.lex_state = 38, .external_lex_state = 2},
  [242] = {.lex_state = 38, .external_lex_state = 3},
  [243] = {.lex_state = 38, .external_lex_state = 2},
  [244] = {.lex_state = 38, .external_lex_state = 2},
  [245] = {.lex_state = 38, .external_lex_state = 3},
  [246] = {.lex_state = 38, .external_lex_state = 2},
  [247] = {.lex_state = 38, .external_lex_state = 3},
  [248] = {.lex_state = 38, .external_lex_state = 2},
  [249] = {.lex_state = 38, .external_lex_state = 2},
  [250] = {.lex_state = 38, .external_lex_state = 2},
  [251] = {.lex_state = 38, .external_lex_state = 2},
  [252] = {.lex_state = 38, .external_lex_state = 2},
  [253] = {.lex_state = 38, .external_lex_state = 2},
  [254] = {.lex_state = 38, .external_lex_state = 3},
  [255] = {.lex_state = 38, .external_lex_state = 2},
  [256] = {.lex_state = 38, .external_lex_state = 2},
  [257] = {.lex_state = 38, .external_lex_state = 2},
  [258] = {.lex_state = 38, .external_lex_state = 2},
  [259] = {.lex_state = 38, .external_lex_state = 2},
  [260] = {.lex_state = 38, .external_lex_state = 2},
  [261] = {.lex_state = 38, .external_lex_state = 2},
  [262] = {.lex_state = 38, .external_lex_state = 2},
  [263] = {.lex_state = 38, .external_lex_state = 2},
  [264] = {.lex_state = 38, .external_lex_state = 2},
  [265] = {.lex_state = 38, .external_lex_state = 2},
  [266] = {.lex_state = 38, .external_lex_state = 2},
  [267] = {.lex_state = 38, .external_lex_state = 3},
  [268] = {.lex_state = 38, .external_lex_state = 2},
  [269] = {.lex_state = 38, .external_lex_state = 2},
  [270] = {.lex_state = 38, .external_lex_state = 2},
  [271] = {.lex_state = 38, .external_lex_state = 2},
  [272] = {.lex_state = 38, .external_lex_state = 2},
  [273] = {.lex_state = 38, .external_lex_state = 3},
  [274] = {.lex_state = 38, .external_lex_state = 2},
  [275] = {.lex_state = 38, .external_lex_state = 2},
  [276] = {.lex_state = 38, .external_lex_state = 2},
  [277] = {.lex_state = 38, .external_lex_state = 2},
  [278] = {.lex_state = 38, .external_lex_state = 2},
  [279] = {.lex_state = 38, .external_lex_state = 2},
  [280] = {.lex_state = 38, .external_lex_state = 2},
  [281] = {.lex_state = 38, .external_lex_state = 2},
  [282] = {.lex_state = 38, .external_lex_state = 2},
  [283] = {.lex_state = 38, .external_lex_state = 2},
  [284] = {.lex_state = 38, .external_lex_state = 2},
  [285] = {.lex_state = 38, .external_lex_state = 2},
  [286] = {.lex_state = 38, .external_lex_state = 2},
  [287] = {.lex_state = 38, .external_lex_state = 2},
  [288] = {.lex_state = 38, .external_lex_state = 2},
  [289] = {.lex_state = 38, .external_lex_state = 2},
  [290] = {.lex_state = 38, .external_lex_state = 2},
  [291] = {.lex_state = 38, .external_lex_state = 2},
  [292] = {.lex_state = 38, .external_lex_state = 2},
  [293] = {.lex_state = 38, .external_lex_state = 2},
  [294] = {.lex_state = 38, .external_lex_state = 2},
  [295] = {.lex_state = 38, .external_lex_state = 2},
  [296] = {.lex_state = 38, .external_lex_state = 2},
  [297] = {.lex_state = 38, .external_lex_state = 2},
  [298] = {.lex_state = 38, .external_lex_state = 2},
  [299] = {.lex_state = 38, .external_lex_state = 2},
  [300] = {.lex_state = 38, .external_lex_state = 2},
  [301] = {.lex_state = 38, .external_lex_state = 2},
  [302] = {.lex_state = 38, .external_lex_state = 2},
  [303] = {.lex_state = 38, .external_lex_state = 2},
  [304] = {.lex_state = 38, .external_lex_state = 2},
  [305] = {.lex_state = 38, .external_lex_state = 2},
  [306] = {.lex_state = 38, .external_lex_state = 2},
  [307] = {.lex_state = 38, .external_lex_state = 2},
  [308] = {.lex_state = 38, .external_lex_state = 2},
  [309] = {.lex_state = 38, .external_lex_state = 2},
  [310] = {.lex_state = 38, .external_lex_state = 2},
  [311] = {.lex_state = 38, .external_lex_state = 2},
  [312] = {.lex_state = 38, .external_lex_state = 2},
  [313] = {.lex_state = 38, .external_lex_state = 2},
  [314] = {.lex_state = 38, .external_lex_state = 3},
  [315] = {.lex_state = 38, .external_lex_state = 3},
  [316] = {.lex_state = 38, .external_lex_state = 2},
  [317] = {.lex_state = 38, .external_lex_state = 2},
  [318] = {.lex_state = 38, .external_lex_state = 2},
  [319] = {.lex_state = 38, .external_lex_state = 2},
  [320] = {.lex_state = 38, .external_lex_state = 2},
  [321] = {.lex_state = 38, .external_lex_state = 3},
  [322] = {.lex_state = 38, .external_lex_state = 2},
  [323] = {.lex_state = 38, .external_lex_state = 2},
  [324] = {.lex_state = 38, .external_lex_state = 2},
  [325] = {.lex_state = 38, .external_lex_state = 2},
  [326] = {.lex_state = 38, .external_lex_state = 3},
  [327] = {.lex_state = 38, .external_lex_state = 2},
  [328] = {.lex_state = 38, .external_lex_state = 2},
  [329] = {.lex_state = 38, .external_lex_state = 3},
  [330] = {.lex_state = 38, .external_lex_state = 3},
  [331] = {.lex_state = 38, .external_lex_state = 2},
  [332] = {.lex_state = 38, .external_lex_state = 2},
  [333] = {.lex_state = 38, .external_lex_state = 2},
  [334] = {.lex_state = 38, .external_lex_state = 3},
  [335] = {.lex_state = 38, .external_lex_state = 3},
  [336] = {.lex_state = 38, .external_lex_state = 3},
  [337] = {.lex_state = 38, .external_lex_state = 3},
  [338] = {.lex_state = 38, .external_lex_state = 3},
  [339] = {.lex_state = 38, .external_lex_state = 2},
  [340] = {.lex_state = 38, .external_lex_state = 2},
  [341] = {.lex_state = 38, .external_lex_state = 2},
  [342] = {.lex_state = 38, .external_lex_state = 2},
  [343] = {.lex_state = 38, .external_lex_state = 3},
  [344] = {.lex_state = 38, .external_lex_state = 2},
  [345] = {.lex_state = 38, .external_lex_state = 3},
  [346] = {.lex_state = 38, .external_lex_state = 2},
  [347] = {.lex_state = 38, .external_lex_state = 3},
  [348] = {.lex_state = 10, .external_lex_state = 2},
  [349] = {.lex_state = 38, .external_lex_state = 3},
  [350] = {.lex_state = 10, .external_lex_state = 2},
  [351] = {.lex_state = 10, .external_lex_state = 2},
  [352] = {.lex_state = 38, .external_lex_state = 3},
  [353] = {.lex_state = 38, .external_lex_state = 2},
  [354] = {.lex_state = 38, .external_lex_state = 2},
  [355] = {.lex_state = 38, .external_lex_state = 3},
  [356] = {.lex_state = 38, .external_lex_state = 2},
  [357] = {.lex_state = 38, .external_lex_state = 2},
  [358] = {.lex_state = 38, .external_lex_state = 2},
  [359] = {.lex_state = 38, .external_lex_state = 2},
  [360] = {.lex_state = 38, .external_lex_state = 3},
  [361] = {.lex_state = 38, .external_lex_state = 3},
  [362] = {.lex_state = 38, .external_lex_state = 2},
  [363] = {.lex_state = 38, .external_lex_state = 2},
  [364] = {.lex_state = 38, .external_lex_state = 3},
  [365] = {.lex_state = 38, .external_lex_state = 2},
  [366] = {.lex_state = 38, .external_lex_state = 3},
  [367] = {.lex_state = 38, .external_lex_state = 2},
  [368] = {.lex_state = 38, .external_lex_state = 3},
  [369] = {.lex_state = 38, .external_lex_state = 3},
  [370] = {.lex_state = 38, .external_lex_state = 3},
  [371] = {.lex_state = 38, .external_lex_state = 2},
  [372] = {.lex_state = 38, .external_lex_state = 2},
  [373] = {.lex_state = 38, .external_lex_state = 2},
  [374] = {.lex_state = 38, .external_lex_state = 3},
  [375] = {.lex_state = 38, .external_lex_state = 3},
  [376] = {.lex_state = 38, .external_lex_state = 3},
  [377] = {.lex_state = 38, .external_lex_state = 3},
  [378] = {.lex_state = 38, .external_lex_state = 2},
  [379] = {.lex_state = 38, .external_lex_state = 3},
  [380] = {.lex_state = 38, .external_lex_state = 3},
  [381] = {.lex_state = 38, .external_lex_state = 2},
  [382] = {.lex_state = 38, .external_lex_state = 2},
  [383] = {.lex_state = 38, .external_lex_state = 2},
  [384] = {.lex_state = 38, .external_lex_state = 2},
  [385] = {.lex_state = 38, .external_lex_state = 2},
  [386] = {.lex_state = 38, .external_lex_state = 2},
  [387] = {.lex_state = 38, .external_lex_state = 2},
  [388] = {.lex_state = 38, .external_lex_state = 2},
  [389] = {.lex_state = 38, .external_lex_state = 3},
  [390] = {.lex_state = 38, .external_lex_state = 2},
  [391] = {.lex_state = 38, .external_lex_state = 3},
  [392] = {.lex_state = 38, .external_lex_state = 3},
  [393] = {.lex_state = 38, .external_lex_state = 2},
  [394] = {.lex_state = 38, .external_lex_state = 3},
  [395] = {.lex_state = 38, .external_lex_state = 2},
  [396] = {.lex_state = 38, .external_lex_state = 2},
  [397] = {.lex_state = 38, .external_lex_state = 2},
  [398] = {.lex_state = 38, .external_lex_state = 2},
  [399] = {.lex_state = 38, .external_lex_state = 2},
  [400] = {.lex_state = 38, .external_lex_state = 2},
  [401] = {.lex_state = 38, .external_lex_state = 3},
  [402] = {.lex_state = 38, .external_lex_state = 3},
  [403] = {.lex_state = 38, .external_lex_state = 3},
  [404] = {.lex_state = 38, .external_lex_state = 3},
  [405] = {.lex_state = 38, .external_lex_state = 3},
  [406] = {.lex_state = 38, .external_lex_state = 2},
  [407] = {.lex_state = 38, .external_lex_state = 3},
  [408] = {.lex_state = 38, .external_lex_state = 3},
  [409] = {.lex_state = 38, .external_lex_state = 3},
  [410] = {.lex_state = 38, .external_lex_state = 3},
  [411] = {.lex_state = 38, .external_lex_state = 3},
  [412] = {.lex_state = 38, .external_lex_state = 3},
  [413] = {.lex_state = 38, .external_lex_state = 2},
  [414] = {.lex_state = 38, .external_lex_state = 3},
  [415] = {.lex_state = 38, .external_lex_state = 2},
  [416] = {.lex_state = 38, .external_lex_state = 2},
  [417] = {.lex_state = 38, .external_lex_state = 3},
  [418] = {.lex_state = 38, .external_lex_state = 3},
  [419] = {.lex_state = 38, .external_lex_state = 3},
  [420] = {.lex_state = 38, .external_lex_state = 3},
  [421] = {.lex_state = 38, .external_lex_state = 2},
  [422] = {.lex_state = 38, .external_lex_state = 2},
  [423] = {.lex_state = 38, .external_lex_state = 2},
  [424] = {.lex_state = 38, .external_lex_state = 2},
  [425] = {.lex_state = 38, .external_lex_state = 2},
  [426] = {.lex_state = 38, .external_lex_state = 3},
  [427] = {.lex_state = 38, .external_lex_state = 2},
  [428] = {.lex_state = 38, .external_lex_state = 2},
  [429] = {.lex_state = 38, .external_lex_state = 2},
  [430] = {.lex_state = 38, .external_lex_state = 2},
  [431] = {.lex_state = 10, .external_lex_state = 2},
  [432] = {.lex_state = 10, .external_lex_state = 2},
  [433] = {.lex_state = 38, .external_lex_state = 2},
  [434] = {.lex_state = 38, .external_lex_state = 2},
  [435] = {.lex_state = 38, .external_lex_state = 2},
  [436] = {.lex_state = 38, .external_lex_state = 2},
  [437] = {.lex_state = 10},
  [438] = {.lex_state = 10},
  [439] = {.lex_state = 10},
  [440] = {.lex_state = 10},
  [441] = {.lex_state = 10},
  [442] = {.lex_state = 10},
  [443] = {.lex_state = 10},
  [444] = {.lex_state = 10},
  [445] = {.lex_state = 38, .external_lex_state = 2},
  [446] = {.lex_state = 10},
  [447] = {.lex_state = 10},
  [448] = {.lex_state = 10},
  [449] = {.lex_state = 10},
  [450] = {.lex_state = 10},
  [451] = {.lex_state = 10},
  [452] = {.lex_state = 10},
  [453] = {.lex_state = 10},
  [454] = {.lex_state = 10},
  [455] = {.lex_state = 10},
  [456] = {.lex_state = 10},
  [457] = {.lex_state = 10},
  [458] = {.lex_state = 10},
  [459] = {.lex_state = 10},
  [460] = {.lex_state = 10},
  [461] = {.lex_state = 10},
  [462] = {.lex_state = 10},
  [463] = {.lex_state = 10},
  [464] = {.lex_state = 10},
  [465] = {.lex_state = 10},
  [466] = {.lex_state = 10},
  [467] = {.lex_state = 10},
  [468] = {.lex_state = 10},
  [469] = {.lex_state = 10},
  [470] = {.lex_state = 10},
  [471] = {.lex_state = 10},
  [472] = {.lex_state = 10},
  [473] = {.lex_state = 10},
  [474] = {.lex_state = 10},
  [475] = {.lex_state = 10},
  [476] = {.lex_state = 10},
  [477] = {.lex_state = 10},
  [478] = {.lex_state = 10},
  [479] = {.lex_state = 10},
  [480] = {.lex_state = 10},
  [481] = {.lex_state = 9},
  [482] = {.lex_state = 10},
  [483] = {.lex_state = 10},
  [484] = {.lex_state = 9},
  [485] = {.lex_state = 38, .external_lex_state = 2},
  [486] = {.lex_state = 10},
  [487] = {.lex_state = 10, .external_lex_state = 6},
  [488] = {.lex_state = 10},
  [489] = {.lex_state = 10},
  [490] = {.lex_state = 10},
  [491] = {.lex_state = 10, .external_lex_state = 4},
  [492] = {.lex_state = 10, .external_lex_state = 4},
  [493] = {.lex_state = 10, .external_lex_state = 4},
  [494] = {.lex_state = 10, .external_lex_state = 6},
  [495] = {.lex_state = 10, .external_lex_state = 6},
  [496] = {.lex_state = 10},
  [497] = {.lex_state = 10, .external_lex_state = 6},
  [498] = {.lex_state = 10, .external_lex_state = 6},
  [499] = {.lex_state = 38, .external_lex_state = 2},
  [500] = {.lex_state = 38, .external_lex_state = 2},
  [501] = {.lex_state = 10, .external_lex_state = 2},
  [502] = {.lex_state = 10, .external_lex_state = 6},
  [503] = {.lex_state = 10, .external_lex_state = 6},
  [504] = {.lex_state = 38, .external_lex_state = 2},
  [505] = {.lex_state = 10, .external_lex_state = 2},
  [506] = {.lex_state = 10, .external_lex_state = 6},
  [507] = {.lex_state = 10, .external_lex_state = 6},
  [508] = {.lex_state = 10, .external_lex_state = 2},
  [509] = {.lex_state = 10, .external_lex_state = 6},
  [510] = {.lex_state = 38, .external_lex_state = 2},
  [511] = {.lex_state = 10, .external_lex_state = 6},
  [512] = {.lex_state = 38, .external_lex_state = 2},
  [513] = {.lex_state = 10},
  [514] = {.lex_state = 38, .external_lex_state = 2},
  [515] = {.lex_state = 38, .external_lex_state = 2},
  [516] = {.lex_state = 38, .external_lex_state = 2},
  [517] = {.lex_state = 10, .external_lex_state = 4},
  [518] = {.lex_state = 38, .external_lex_state = 2},
  [519] = {.lex_state = 38, .external_lex_state = 2},
  [520] = {.lex_state = 38, .external_lex_state = 2},
  [521] = {.lex_state = 38, .external_lex_state = 2},
  [522] = {.lex_state = 38, .external_lex_state = 2},
  [523] = {.lex_state = 38, .external_lex_state = 2},
  [524] = {.lex_state = 38, .external_lex_state = 2},
  [525] = {.lex_state = 38, .external_lex_state = 2},
  [526] = {.lex_state = 38, .external_lex_state = 2},
  [527] = {.lex_state = 10},
  [528] = {.lex_state = 10},
  [529] = {.lex_state = 10},
  [530] = {.lex_state = 10},
  [531] = {.lex_state = 10},
  [532] = {.lex_state = 38, .external_lex_state = 2},
  [533] = {.lex_state = 38, .external_lex_state = 2},
  [534] = {.lex_state = 38, .external_lex_state = 2},
  [535] = {.lex_state = 38, .external_lex_state = 2},
  [536] = {.lex_state = 38, .external_lex_state = 2},
  [537] = {.lex_state = 9},
  [538] = {.lex_state = 10},
  [539] = {.lex_state = 38, .external_lex_state = 2},
  [540] = {.lex_state = 9, .external_lex_state = 6},
  [541] = {.lex_state = 38, .external_lex_state = 2},
  [542] = {.lex_state = 38, .external_lex_state = 2},
  [543] = {.lex_state = 38, .external_lex_state = 2},
  [544] = {.lex_state = 38, .external_lex_state = 2},
  [545] = {.lex_state = 38, .external_lex_state = 2},
  [546] = {.lex_state = 38, .external_lex_state = 2},
  [547] = {.lex_state = 38, .external_lex_state = 2},
  [548] = {.lex_state = 38, .external_lex_state = 2},
  [549] = {.lex_state = 38, .external_lex_state = 2},
  [550] = {.lex_state = 10},
  [551] = {.lex_state = 38, .external_lex_state = 2},
  [552] = {.lex_state = 38, .external_lex_state = 2},
  [553] = {.lex_state = 38, .external_lex_state = 2},
  [554] = {.lex_state = 38, .external_lex_state = 2},
  [555] = {.lex_state = 38, .external_lex_state = 2},
  [556] = {.lex_state = 38, .external_lex_state = 2},
  [557] = {.lex_state = 38, .external_lex_state = 2},
  [558] = {.lex_state = 38, .external_lex_state = 2},
  [559] = {.lex_state = 38, .external_lex_state = 2},
  [560] = {.lex_state = 10},
  [561] = {.lex_state = 10},
  [562] = {.lex_state = 10},
  [563] = {.lex_state = 38, .external_lex_state = 2},
  [564] = {.lex_state = 10},
  [565] = {.lex_state = 10},
  [566] = {.lex_state = 10},
  [567] = {.lex_state = 38, .external_lex_state = 2},
  [568] = {.lex_state = 10},
  [569] = {.lex_state = 10},
  [570] = {.lex_state = 38, .external_lex_state = 2},
  [571] = {.lex_state = 38, .external_lex_state = 2},
  [572] = {.lex_state = 10, .external_lex_state = 4},
  [573] = {.lex_state = 9},
  [574] = {.lex_state = 38, .external_lex_state = 2},
  [575] = {.lex_state = 38, .external_lex_state = 2},
  [576] = {.lex_state = 10, .external_lex_state = 6},
  [577] = {.lex_state = 10, .external_lex_state = 6},
  [578] = {.lex_state = 10, .external_lex_state = 6},
  [579] = {.lex_state = 10},
  [580] = {.lex_state = 10, .external_lex_state = 6},
  [581] = {.lex_state = 10, .external_lex_state = 6},
  [582] = {.lex_state = 10},
  [583] = {.lex_state = 10, .external_lex_state = 6},
  [584] = {.lex_state = 10, .external_lex_state = 6},
  [585] = {.lex_state = 10, .external_lex_state = 6},
  [586] = {.lex_state = 10, .external_lex_state = 6},
  [587] = {.lex_state = 10},
  [588] = {.lex_state = 9},
  [589] = {.lex_state = 10, .external_lex_state = 6},
  [590] = {.lex_state = 10, .external_lex_state = 6},
  [591] = {.lex_state = 10, .external_lex_state = 6},
  [592] = {.lex_state = 10},
  [593] = {.lex_state = 10, .external_lex_state = 2},
  [594] = {.lex_state = 10},
  [595] = {.lex_state = 10, .external_lex_state = 6},
  [596] = {.lex_state = 10, .external_lex_state = 6},
  [597] = {.lex_state = 10, .external_lex_state = 6},
  [598] = {.lex_state = 10, .external_lex_state = 6},
  [599] = {.lex_state = 10, .external_lex_state = 6},
  [600] = {.lex_state = 10, .external_lex_state = 2},
  [601] = {.lex_state = 10, .external_lex_state = 6},
  [602] = {.lex_state = 10, .external_lex_state = 6},
  [603] = {.lex_state = 10, .external_lex_state = 6},
  [604] = {.lex_state = 10, .external_lex_state = 6},
  [605] = {.lex_state = 10, .external_lex_state = 6},
  [606] = {.lex_state = 10, .external_lex_state = 6},
  [607] = {.lex_state = 10, .external_lex_state = 6},
  [608] = {.lex_state = 10},
  [609] = {.lex_state = 10},
  [610] = {.lex_state = 10},
  [611] = {.lex_state = 10},
  [612] = {.lex_state = 10},
  [613] = {.lex_state = 9},
  [614] = {.lex_state = 10},
  [615] = {.lex_state = 10},
  [616] = {.lex_state = 10},
  [617] = {.lex_state = 10},
  [618] = {.lex_state = 10},
  [619] = {.lex_state = 10},
  [620] = {.lex_state = 10},
  [621] = {.lex_state = 10},
  [622] = {.lex_state = 10},
  [623] = {.lex_state = 9},
  [624] = {.lex_state = 10},
  [625] = {.lex_state = 10},
  [626] = {.lex_state = 10},
  [627] = {.lex_state = 10},
  [628] = {.lex_state = 10},
  [629] = {.lex_state = 10},
  [630] = {.lex_state = 10},
  [631] = {.lex_state = 10},
  [632] = {.lex_state = 10},
  [633] = {.lex_state = 10},
  [634] = {.lex_state = 10},
  [635] = {.lex_state = 10},
  [636] = {.lex_state = 10},
  [637] = {.lex_state = 38},
  [638] = {.lex_state = 38},
  [639] = {.lex_state = 38},
  [640] = {.lex_state = 38},
  [641] = {.lex_state = 38},
  [642] = {.lex_state = 0, .external_lex_state = 6},
  [643] = {.lex_state = 10, .external_lex_state = 6},
  [644] = {.lex_state = 0, .external_lex_state = 6},
  [645] = {.lex_state = 10},
  [646] = {.lex_state = 38},
  [647] = {.lex_state = 38},
  [648] = {.lex_state = 10},
  [649] = {.lex_state = 38},
  [650] = {.lex_state = 38},
  [651] = {.lex_state = 38},
  [652] = {.lex_state = 10},
  [653] = {.lex_state = 38},
  [654] = {.lex_state = 38},
  [655] = {.lex_state = 38},
  [656] = {.lex_state = 38},
  [657] = {.lex_state = 38},
  [658] = {.lex_state = 38},
  [659] = {.lex_state = 38},
  [660] = {.lex_state = 38},
  [661] = {.lex_state = 38},
  [662] = {.lex_state = 38},
  [663] = {.lex_state = 38},
  [664] = {.lex_state = 38},
  [665] = {.lex_state = 0},
  [666] = {.lex_state = 38},
  [667] = {.lex_state = 38},
  [668] = {.lex_state = 0},
  [669] = {.lex_state = 0},
  [670] = {.lex_state = 0},
  [671] = {.lex_state = 0},
  [672] = {.lex_state = 38},
  [673] = {.lex_state = 38},
  [674] = {.lex_state = 0},
  [675] = {.lex_state = 38},
  [676] = {.lex_state = 38},
  [677] = {.lex_state = 38},
  [678] = {.lex_state = 38},
  [679] = {.lex_state = 38},
  [680] = {.lex_state = 38},
  [681] = {.lex_state = 38},
  [682] = {.lex_state = 38},
  [683] = {.lex_state = 38},
  [684] = {.lex_state = 38},
  [685] = {.lex_state = 38},
  [686] = {.lex_state = 38},
  [687] = {.lex_state = 38},
  [688] = {.lex_state = 0},
  [689] = {.lex_state = 0},
  [690] = {.lex_state = 0},
  [691] = {.lex_state = 0},
  [692] = {.lex_state = 0},
  [693] = {.lex_state = 0},
  [694] = {.lex_state = 0},
  [695] = {.lex_state = 0},
  [696] = {.lex_state = 0},
  [697] = {.lex_state = 0},
  [698] = {.lex_state = 0},
  [699] = {.lex_state = 0},
  [700] = {.lex_state = 0},
  [701] = {.lex_state = 0},
  [702] = {.lex_state = 0},
  [703] = {.lex_state = 0},
  [704] = {.lex_state = 0},
  [705] = {.lex_state = 0, .external_lex_state = 6},
  [706] = {.lex_state = 0, .external_lex_state = 6},
  [707] = {.lex_state = 0},
  [708] = {.lex_state = 0},
  [709] = {.lex_state = 0, .external_lex_state = 6},
  [710] = {.lex_state = 0, .external_lex_state = 6},
  [711] = {.lex_state = 0, .external_lex_state = 6},
  [712] = {.lex_state = 0, .external_lex_state = 6},
  [713] = {.lex_state = 0, .external_lex_state = 6},
  [714] = {.lex_state = 0},
  [715] = {.lex_state = 0},
  [716] = {.lex_state = 0, .external_lex_state = 6},
  [717] = {.lex_state = 0},
  [718] = {.lex_state = 38},
  [719] = {.lex_state = 0, .external_lex_state = 6},
  [720] = {.lex_state = 0},
  [721] = {.lex_state = 0},
  [722] = {.lex_state = 0, .external_lex_state = 6},
  [723] = {.lex_state = 12, .external_lex_state = 7},
  [724] = {.lex_state = 0},
  [725] = {.lex_state = 0, .external_lex_state = 6},
  [726] = {.lex_state = 0},
  [727] = {.lex_state = 38},
  [728] = {.lex_state = 0, .external_lex_state = 6},
  [729] = {.lex_state = 0, .external_lex_state = 6},
  [730] = {.lex_state = 0},
  [731] = {.lex_state = 0, .external_lex_state = 6},
  [732] = {.lex_state = 0, .external_lex_state = 6},
  [733] = {.lex_state = 12, .external_lex_state = 7},
  [734] = {.lex_state = 12, .external_lex_state = 7},
  [735] = {.lex_state = 12, .external_lex_state = 7},
  [736] = {.lex_state = 12, .external_lex_state = 7},
  [737] = {.lex_state = 38},
  [738] = {.lex_state = 38},
  [739] = {.lex_state = 0, .external_lex_state = 6},
  [740] = {.lex_state = 38},
  [741] = {.lex_state = 0},
  [742] = {.lex_state = 0, .external_lex_state = 6},
  [743] = {.lex_state = 0},
  [744] = {.lex_state = 12, .external_lex_state = 7},
  [745] = {.lex_state = 38},
  [746] = {.lex_state = 12, .external_lex_state = 7},
  [747] = {.lex_state = 0},
  [748] = {.lex_state = 0},
  [749] = {.lex_state = 38},
  [750] = {.lex_state = 0},
  [751] = {.lex_state = 0, .external_lex_state = 6},
  [752] = {.lex_state = 0},
  [753] = {.lex_state = 38},
  [754] = {.lex_state = 0},
  [755] = {.lex_state = 38},
  [756] = {.lex_state = 38},
  [757] = {.lex_state = 38},
  [758] = {.lex_state = 12, .external_lex_state = 7},
  [759] = {.lex_state = 38},
  [760] = {.lex_state = 0},
  [761] = {.lex_state = 38},
  [762] = {.lex_state = 38},
  [763] = {.lex_state = 38},
  [764] = {.lex_state = 12, .external_lex_state = 7},
  [765] = {.lex_state = 10},
  [766] = {.lex_state = 38},
  [767] = {.lex_state = 0, .external_lex_state = 6},
  [768] = {.lex_state = 38},
  [769] = {.lex_state = 38},
  [770] = {.lex_state = 10},
  [771] = {.lex_state = 10},
  [772] = {.lex_state = 0, .external_lex_state = 6},
  [773] = {.lex_state = 38},
  [774] = {.lex_state = 38},
  [775] = {.lex_state = 38},
  [776] = {.lex_state = 10},
  [777] = {.lex_state = 38},
  [778] = {.lex_state = 38},
  [779] = {.lex_state = 38},
  [780] = {.lex_state = 38},
  [781] = {.lex_state = 10, .external_lex_state = 6},
  [782] = {.lex_state = 0, .external_lex_state = 6},
  [783] = {.lex_state = 38},
  [784] = {.lex_state = 10, .external_lex_state = 6},
  [785] = {.lex_state = 0},
  [786] = {.lex_state = 10, .external_lex_state = 6},
  [787] = {.lex_state = 38},
  [788] = {.lex_state = 0, .external_lex_state = 6},
  [789] = {.lex_state = 38},
  [790] = {.lex_state = 0},
  [791] = {.lex_state = 0},
  [792] = {.lex_state = 38},
  [793] = {.lex_state = 0, .external_lex_state = 6},
  [794] = {.lex_state = 0},
  [795] = {.lex_state = 12, .external_lex_state = 7},
  [796] = {.lex_state = 0, .external_lex_state = 6},
  [797] = {.lex_state = 0},
  [798] = {.lex_state = 0, .external_lex_state = 6},
  [799] = {.lex_state = 0},
  [800] = {.lex_state = 0, .external_lex_state = 6},
  [801] = {.lex_state = 0},
  [802] = {.lex_state = 4},
  [803] = {.lex_state = 0},
  [804] = {.lex_state = 0},
  [805] = {.lex_state = 0},
  [806] = {.lex_state = 10, .external_lex_state = 6},
  [807] = {.lex_state = 4},
  [808] = {.lex_state = 12, .external_lex_state = 7},
  [809] = {.lex_state = 0},
  [810] = {.lex_state = 10},
  [811] = {.lex_state = 0},
  [812] = {.lex_state = 0},
  [813] = {.lex_state = 12, .external_lex_state = 7},
  [814] = {.lex_state = 0},
  [815] = {.lex_state = 0, .external_lex_state = 6},
  [816] = {.lex_state = 0, .external_lex_state = 6},
  [817] = {.lex_state = 4},
  [818] = {.lex_state = 0, .external_lex_state = 6},
  [819] = {.lex_state = 0},
  [820] = {.lex_state = 0, .external_lex_state = 6},
  [821] = {.lex_state = 0},
  [822] = {.lex_state = 0},
  [823] = {.lex_state = 0},
  [824] = {.lex_state = 0},
  [825] = {.lex_state = 0},
  [826] = {.lex_state = 0, .external_lex_state = 6},
  [827] = {.lex_state = 0},
  [828] = {.lex_state = 38},
  [829] = {.lex_state = 38},
  [830] = {.lex_state = 38},
  [831] = {.lex_state = 0, .external_lex_state = 6},
  [832] = {.lex_state = 0},
  [833] = {.lex_state = 0},
  [834] = {.lex_state = 10},
  [835] = {.lex_state = 0, .external_lex_state = 6},
  [836] = {.lex_state = 0, .external_lex_state = 6},
  [837] = {.lex_state = 0, .external_lex_state = 6},
  [838] = {.lex_state = 0, .external_lex_state = 6},
  [839] = {.lex_state = 0},
  [840] = {.lex_state = 0},
  [841] = {.lex_state = 0, .external_lex_state = 6},
  [842] = {.lex_state = 0, .external_lex_state = 6},
  [843] = {.lex_state = 0, .external_lex_state = 6},
  [844] = {.lex_state = 10},
  [845] = {.lex_state = 0, .external_lex_state = 6},
  [846] = {.lex_state = 0},
  [847] = {.lex_state = 0, .external_lex_state = 6},
  [848] = {.lex_state = 38},
  [849] = {.lex_state = 38},
  [850] = {.lex_state = 0},
  [851] = {.lex_state = 38},
  [852] = {.lex_state = 0},
  [853] = {.lex_state = 0},
  [854] = {.lex_state = 38},
  [855] = {.lex_state = 0, .external_lex_state = 6},
  [856] = {.lex_state = 38},
  [857] = {.lex_state = 38},
  [858] = {.lex_state = 0},
  [859] = {.lex_state = 38},
  [860] = {.lex_state = 0, .external_lex_state = 6},
  [861] = {.lex_state = 38},
  [862] = {.lex_state = 0, .external_lex_state = 6},
  [863] = {.lex_state = 0, .external_lex_state = 6},
  [864] = {.lex_state = 0, .external_lex_state = 6},
  [865] = {.lex_state = 38},
  [866] = {.lex_state = 0},
  [867] = {.lex_state = 0},
  [868] = {.lex_state = 0},
  [869] = {.lex_state = 0},
  [870] = {.lex_state = 0},
  [871] = {.lex_state = 0},
  [872] = {.lex_state = 0, .external_lex_state = 6},
  [873] = {.lex_state = 38},
  [874] = {.lex_state = 0},
  [875] = {.lex_state = 0},
  [876] = {.lex_state = 0},
  [877] = {.lex_state = 0},
  [878] = {.lex_state = 0, .external_lex_state = 6},
  [879] = {.lex_state = 0},
  [880] = {.lex_state = 0},
  [881] = {.lex_state = 0, .external_lex_state = 6},
  [882] = {.lex_state = 0},
  [883] = {.lex_state = 0, .external_lex_state = 6},
  [884] = {.lex_state = 0},
  [885] = {.lex_state = 0},
  [886] = {.lex_state = 4},
  [887] = {.lex_state = 0},
  [888] = {.lex_state = 0},
  [889] = {.lex_state = 0},
  [890] = {.lex_state = 0},
  [891] = {.lex_state = 0},
  [892] = {.lex_state = 38},
  [893] = {.lex_state = 0},
  [894] = {.lex_state = 0},
  [895] = {.lex_state = 0, .external_lex_state = 6},
  [896] = {.lex_state = 0},
  [897] = {.lex_state = 38},
  [898] = {.lex_state = 0},
  [899] = {.lex_state = 0},
  [900] = {.lex_state = 0},
  [901] = {.lex_state = 0},
  [902] = {.lex_state = 0},
  [903] = {.lex_state = 0},
  [904] = {.lex_state = 0},
  [905] = {.lex_state = 0},
  [906] = {.lex_state = 0},
  [907] = {.lex_state = 0},
  [908] = {.lex_state = 0},
  [909] = {.lex_state = 0, .external_lex_state = 6},
  [910] = {.lex_state = 0},
  [911] = {.lex_state = 38},
  [912] = {.lex_state = 0},
  [913] = {.lex_state = 0},
  [914] = {.lex_state = 0},
  [915] = {.lex_state = 0},
  [916] = {.lex_state = 0},
  [917] = {.lex_state = 0},
  [918] = {.lex_state = 0, .external_lex_state = 6},
  [919] = {.lex_state = 0},
  [920] = {.lex_state = 0},
  [921] = {.lex_state = 38},
  [922] = {.lex_state = 0},
  [923] = {.lex_state = 0},
  [924] = {.lex_state = 0, .external_lex_state = 6},
  [925] = {.lex_state = 0, .external_lex_state = 6},
  [926] = {.lex_state = 0},
  [927] = {.lex_state = 38},
  [928] = {.lex_state = 38},
  [929] = {.lex_state = 0},
  [930] = {.lex_state = 38},
  [931] = {.lex_state = 38},
  [932] = {.lex_state = 0},
  [933] = {.lex_state = 0},
  [934] = {.lex_state = 0, .external_lex_state = 6},
  [935] = {.lex_state = 0},
  [936] = {.lex_state = 0},
  [937] = {.lex_state = 0},
  [938] = {.lex_state = 38},
  [939] = {.lex_state = 0},
  [940] = {.lex_state = 38},
  [941] = {.lex_state = 0},
  [942] = {.lex_state = 0, .external_lex_state = 6},
  [943] = {.lex_state = 0, .external_lex_state = 6},
  [944] = {.lex_state = 38},
  [945] = {.lex_state = 0, .external_lex_state = 6},
  [946] = {.lex_state = 0, .external_lex_state = 6},
  [947] = {.lex_state = 0},
  [948] = {.lex_state = 38},
  [949] = {.lex_state = 38},
  [950] = {.lex_state = 0},
  [951] = {.lex_state = 38},
  [952] = {.lex_state = 38},
  [953] = {.lex_state = 0, .external_lex_state = 6},
  [954] = {.lex_state = 11},
  [955] = {.lex_state = 0, .external_lex_state = 6},
  [956] = {.lex_state = 0},
  [957] = {.lex_state = 0},
  [958] = {.lex_state = 0, .external_lex_state = 6},
  [959] = {.lex_state = 0, .external_lex_state = 6},
  [960] = {.lex_state = 38},
  [961] = {.lex_state = 0, .external_lex_state = 6},
  [962] = {.lex_state = 0, .external_lex_state = 6},
  [963] = {.lex_state = 0, .external_lex_state = 6},
  [964] = {.lex_state = 0},
  [965] = {.lex_state = 0, .external_lex_state = 6},
  [966] = {.lex_state = 0},
  [967] = {.lex_state = 0, .external_lex_state = 6},
  [968] = {.lex_state = 0},
  [969] = {.lex_state = 0, .external_lex_state = 6},
  [970] = {.lex_state = 0},
  [971] = {.lex_state = 11},
  [972] = {.lex_state = 0},
  [973] = {.lex_state = 11},
  [974] = {.lex_state = 0},
  [975] = {.lex_state = 0, .external_lex_state = 6},
  [976] = {.lex_state = 0},
  [977] = {.lex_state = 11},
  [978] = {.lex_state = 0, .external_lex_state = 6},
  [979] = {.lex_state = 11},
  [980] = {.lex_state = 0, .external_lex_state = 6},
  [981] = {.lex_state = 0, .external_lex_state = 6},
  [982] = {.lex_state = 0},
  [983] = {.lex_state = 0},
  [984] = {.lex_state = 0},
  [985] = {.lex_state = 0},
  [986] = {.lex_state = 0, .external_lex_state = 6},
  [987] = {.lex_state = 0, .external_lex_state = 6},
  [988] = {.lex_state = 0, .external_lex_state = 6},
  [989] = {.lex_state = 11},
  [990] = {.lex_state = 0, .external_lex_state = 6},
  [991] = {.lex_state = 0},
  [992] = {.lex_state = 0},
  [993] = {.lex_state = 0},
  [994] = {.lex_state = 0},
  [995] = {.lex_state = 0},
  [996] = {.lex_state = 0},
  [997] = {.lex_state = 0},
  [998] = {.lex_state = 0},
  [999] = {.lex_state = 0},
  [1000] = {.lex_state = 0},
  [1001] = {.lex_state = 0},
  [1002] = {.lex_state = 0},
  [1003] = {.lex_state = 0},
  [1004] = {.lex_state = 0},
  [1005] = {.lex_state = 0},
  [1006] = {.lex_state = 0},
  [1007] = {.lex_state = 0},
  [1008] = {.lex_state = 0},
  [1009] = {.lex_state = 0},
  [1010] = {.lex_state = 0},
  [1011] = {.lex_state = 0},
  [1012] = {.lex_state = 0},
  [1013] = {.lex_state = 0},
  [1014] = {.lex_state = 0},
  [1015] = {.lex_state = 0},
  [1016] = {.lex_state = 38},
  [1017] = {.lex_state = 0},
  [1018] = {.lex_state = 0},
  [1019] = {.lex_state = 0},
  [1020] = {.lex_state = 0},
  [1021] = {.lex_state = 0},
  [1022] = {.lex_state = 0},
  [1023] = {.lex_state = 38},
  [1024] = {.lex_state = 38},
  [1025] = {.lex_state = 0},
  [1026] = {.lex_state = 38},
  [1027] = {.lex_state = 0},
  [1028] = {.lex_state = 38},
  [1029] = {.lex_state = 0},
  [1030] = {.lex_state = 38},
  [1031] = {.lex_state = 0},
  [1032] = {.lex_state = 0},
  [1033] = {.lex_state = 38},
  [1034] = {.lex_state = 0},
  [1035] = {.lex_state = 0},
  [1036] = {.lex_state = 38},
  [1037] = {.lex_state = 0},
  [1038] = {.lex_state = 0},
  [1039] = {.lex_state = 0},
  [1040] = {.lex_state = 0},
  [1041] = {.lex_state = 0},
  [1042] = {.lex_state = 38},
  [1043] = {.lex_state = 0},
  [1044] = {.lex_state = 0},
  [1045] = {.lex_state = 38},
  [1046] = {.lex_state = 0},
  [1047] = {.lex_state = 38},
  [1048] = {.lex_state = 38},
  [1049] = {.lex_state = 38},
  [1050] = {.lex_state = 38},
  [1051] = {.lex_state = 38},
  [1052] = {.lex_state = 38},
  [1053] = {.lex_state = 0},
  [1054] = {.lex_state = 0},
  [1055] = {.lex_state = 38},
  [1056] = {.lex_state = 0},
  [1057] = {.lex_state = 38},
  [1058] = {.lex_state = 38},
  [1059] = {.lex_state = 38},
  [1060] = {.lex_state = 0},
  [1061] = {.lex_state = 38},
  [1062] = {.lex_state = 38},
  [1063] = {.lex_state = 0},
  [1064] = {.lex_state = 0},
  [1065] = {.lex_state = 0},
  [1066] = {.lex_state = 38},
  [1067] = {.lex_state = 0},
  [1068] = {.lex_state = 0},
  [1069] = {.lex_state = 0},
  [1070] = {.lex_state = 38},
  [1071] = {.lex_state = 0},
  [1072] = {.lex_state = 0},
  [1073] = {.lex_state = 0},
  [1074] = {.lex_state = 0},
  [1075] = {.lex_state = 38},
  [1076] = {.lex_state = 0},
  [1077] = {.lex_state = 38},
  [1078] = {.lex_state = 38},
  [1079] = {.lex_state = 0},
  [1080] = {.lex_state = 0},
  [1081] = {.lex_state = 0},
  [1082] = {.lex_state = 0},
  [1083] = {.lex_state = 0},
  [1084] = {.lex_state = 38},
  [1085] = {.lex_state = 0},
  [1086] = {.lex_state = 0},
  [1087] = {.lex_state = 0},
};

enum {
  ts_external_token__newline = 0,
  ts_external_token__indent = 1,
  ts_external_token__dedent = 2,
  ts_external_token__string_start = 3,
  ts_external_token__string_content = 4,
  ts_external_token__string_end = 5,
};

static TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__newline] = sym__newline,
  [ts_external_token__indent] = sym__indent,
  [ts_external_token__dedent] = sym__dedent,
  [ts_external_token__string_start] = sym__string_start,
  [ts_external_token__string_content] = sym__string_content,
  [ts_external_token__string_end] = sym__string_end,
};

static bool ts_external_scanner_states[8][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__newline] = true,
    [ts_external_token__indent] = true,
    [ts_external_token__dedent] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__string_content] = true,
    [ts_external_token__string_end] = true,
  },
  [2] = {
    [ts_external_token__string_start] = true,
  },
  [3] = {
    [ts_external_token__dedent] = true,
    [ts_external_token__string_start] = true,
  },
  [4] = {
    [ts_external_token__newline] = true,
    [ts_external_token__string_start] = true,
  },
  [5] = {
    [ts_external_token__newline] = true,
    [ts_external_token__indent] = true,
    [ts_external_token__string_start] = true,
  },
  [6] = {
    [ts_external_token__newline] = true,
  },
  [7] = {
    [ts_external_token__string_content] = true,
    [ts_external_token__string_end] = true,
  },
};

static uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [anon_sym_import] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_from] = ACTIONS(1),
    [anon_sym___future__] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_as] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_print] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_assert] = ACTIONS(1),
    [anon_sym_COLON_EQ] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_del] = ACTIONS(1),
    [anon_sym_raise] = ACTIONS(1),
    [anon_sym_pass] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_continue] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_elif] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_async] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_in] = ACTIONS(1),
    [anon_sym_while] = ACTIONS(1),
    [anon_sym_try] = ACTIONS(1),
    [anon_sym_except] = ACTIONS(1),
    [anon_sym_finally] = ACTIONS(1),
    [anon_sym_with] = ACTIONS(1),
    [anon_sym_def] = ACTIONS(1),
    [anon_sym_DASH_GT] = ACTIONS(1),
    [anon_sym_STAR_STAR] = ACTIONS(1),
    [anon_sym_global] = ACTIONS(1),
    [anon_sym_nonlocal] = ACTIONS(1),
    [anon_sym_exec] = ACTIONS(1),
    [anon_sym_class] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_not] = ACTIONS(1),
    [anon_sym_and] = ACTIONS(1),
    [anon_sym_or] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_SLASH_SLASH] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_LT_GT] = ACTIONS(1),
    [anon_sym_is] = ACTIONS(1),
    [anon_sym_lambda] = ACTIONS(1),
    [anon_sym_PLUS_EQ] = ACTIONS(1),
    [anon_sym_DASH_EQ] = ACTIONS(1),
    [anon_sym_STAR_EQ] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [anon_sym_AT_EQ] = ACTIONS(1),
    [anon_sym_SLASH_SLASH_EQ] = ACTIONS(1),
    [anon_sym_PERCENT_EQ] = ACTIONS(1),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_LT_LT_EQ] = ACTIONS(1),
    [anon_sym_AMP_EQ] = ACTIONS(1),
    [anon_sym_CARET_EQ] = ACTIONS(1),
    [anon_sym_PIPE_EQ] = ACTIONS(1),
    [anon_sym_yield] = ACTIONS(1),
    [sym_ellipsis] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [sym_type_conversion] = ACTIONS(1),
    [sym_integer] = ACTIONS(1),
    [sym_float] = ACTIONS(1),
    [anon_sym_await] = ACTIONS(1),
    [sym_true] = ACTIONS(1),
    [sym_false] = ACTIONS(1),
    [sym_none] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
    [sym__semicolon] = ACTIONS(1),
    [sym__newline] = ACTIONS(1),
    [sym__indent] = ACTIONS(1),
    [sym__dedent] = ACTIONS(1),
    [sym__string_start] = ACTIONS(1),
    [sym__string_content] = ACTIONS(1),
    [sym__string_end] = ACTIONS(1),
  },
  [1] = {
    [sym_module] = STATE(1065),
    [sym__statement] = STATE(45),
    [sym__simple_statements] = STATE(45),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_if_statement] = STATE(45),
    [sym_for_statement] = STATE(45),
    [sym_while_statement] = STATE(45),
    [sym_try_statement] = STATE(45),
    [sym_with_statement] = STATE(45),
    [sym_function_definition] = STATE(45),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_class_definition] = STATE(45),
    [sym_decorated_definition] = STATE(45),
    [sym_decorator] = STATE(720),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(45),
    [aux_sym_decorated_definition_repeat1] = STATE(720),
    [ts_builtin_sym_end] = ACTIONS(5),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(33),
    [anon_sym_async] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_try] = ACTIONS(41),
    [anon_sym_with] = ACTIONS(43),
    [anon_sym_def] = ACTIONS(45),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(75),
  },
  [2] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(336),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [3] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(281),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [4] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(386),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [5] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(309),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [6] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(333),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [7] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(358),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [8] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(410),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [9] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(331),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [10] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(353),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [11] = {
    [sym__statement] = STATE(43),
    [sym__simple_statements] = STATE(43),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(43),
    [sym_for_statement] = STATE(43),
    [sym_while_statement] = STATE(43),
    [sym_try_statement] = STATE(43),
    [sym_with_statement] = STATE(43),
    [sym_function_definition] = STATE(43),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(43),
    [sym_decorated_definition] = STATE(43),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(797),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(43),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(97),
    [sym__string_start] = ACTIONS(75),
  },
  [12] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(344),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [13] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(378),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [14] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(415),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [15] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(190),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [16] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(346),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [17] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(420),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [18] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(397),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [19] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(423),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [20] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(196),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [21] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(389),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [22] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(335),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [23] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(303),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [24] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(337),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [25] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(400),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [26] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(352),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [27] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(408),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [28] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(379),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [29] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(361),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [30] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(368),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [31] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(267),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [32] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(321),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [33] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(369),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [34] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(363),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [35] = {
    [sym__statement] = STATE(43),
    [sym__simple_statements] = STATE(43),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(43),
    [sym_for_statement] = STATE(43),
    [sym_while_statement] = STATE(43),
    [sym_try_statement] = STATE(43),
    [sym_with_statement] = STATE(43),
    [sym_function_definition] = STATE(43),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(43),
    [sym_decorated_definition] = STATE(43),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(794),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(43),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(97),
    [sym__string_start] = ACTIONS(75),
  },
  [36] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(418),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [37] = {
    [sym__statement] = STATE(44),
    [sym__simple_statements] = STATE(44),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_try_statement] = STATE(44),
    [sym_with_statement] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(44),
    [sym_decorated_definition] = STATE(44),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(385),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(44),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(95),
    [sym__string_start] = ACTIONS(75),
  },
  [38] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(338),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [39] = {
    [sym__statement] = STATE(41),
    [sym__simple_statements] = STATE(41),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(41),
    [sym_for_statement] = STATE(41),
    [sym_while_statement] = STATE(41),
    [sym_try_statement] = STATE(41),
    [sym_with_statement] = STATE(41),
    [sym_function_definition] = STATE(41),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(41),
    [sym_decorated_definition] = STATE(41),
    [sym_decorator] = STATE(717),
    [sym_block] = STATE(314),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(41),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(93),
    [sym__string_start] = ACTIONS(75),
  },
  [40] = {
    [sym__statement] = STATE(40),
    [sym__simple_statements] = STATE(40),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(40),
    [sym_for_statement] = STATE(40),
    [sym_while_statement] = STATE(40),
    [sym_try_statement] = STATE(40),
    [sym_with_statement] = STATE(40),
    [sym_function_definition] = STATE(40),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(40),
    [sym_decorated_definition] = STATE(40),
    [sym_decorator] = STATE(717),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(40),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(99),
    [anon_sym_import] = ACTIONS(102),
    [anon_sym_from] = ACTIONS(105),
    [anon_sym_LPAREN] = ACTIONS(108),
    [anon_sym_STAR] = ACTIONS(111),
    [anon_sym_print] = ACTIONS(114),
    [anon_sym_assert] = ACTIONS(117),
    [anon_sym_return] = ACTIONS(120),
    [anon_sym_del] = ACTIONS(123),
    [anon_sym_raise] = ACTIONS(126),
    [anon_sym_pass] = ACTIONS(129),
    [anon_sym_break] = ACTIONS(132),
    [anon_sym_continue] = ACTIONS(135),
    [anon_sym_if] = ACTIONS(138),
    [anon_sym_async] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(144),
    [anon_sym_while] = ACTIONS(147),
    [anon_sym_try] = ACTIONS(150),
    [anon_sym_with] = ACTIONS(153),
    [anon_sym_def] = ACTIONS(156),
    [anon_sym_global] = ACTIONS(159),
    [anon_sym_nonlocal] = ACTIONS(162),
    [anon_sym_exec] = ACTIONS(165),
    [anon_sym_class] = ACTIONS(168),
    [anon_sym_AT] = ACTIONS(171),
    [anon_sym_LBRACK] = ACTIONS(174),
    [anon_sym_not] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(180),
    [anon_sym_DASH] = ACTIONS(180),
    [anon_sym_TILDE] = ACTIONS(180),
    [anon_sym_lambda] = ACTIONS(183),
    [anon_sym_yield] = ACTIONS(186),
    [sym_ellipsis] = ACTIONS(189),
    [anon_sym_LBRACE] = ACTIONS(192),
    [sym_integer] = ACTIONS(195),
    [sym_float] = ACTIONS(189),
    [anon_sym_await] = ACTIONS(198),
    [sym_true] = ACTIONS(195),
    [sym_false] = ACTIONS(195),
    [sym_none] = ACTIONS(195),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(201),
    [sym__string_start] = ACTIONS(203),
  },
  [41] = {
    [sym__statement] = STATE(40),
    [sym__simple_statements] = STATE(40),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(40),
    [sym_for_statement] = STATE(40),
    [sym_while_statement] = STATE(40),
    [sym_try_statement] = STATE(40),
    [sym_with_statement] = STATE(40),
    [sym_function_definition] = STATE(40),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(40),
    [sym_decorated_definition] = STATE(40),
    [sym_decorator] = STATE(717),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(40),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(206),
    [sym__string_start] = ACTIONS(75),
  },
  [42] = {
    [sym__statement] = STATE(42),
    [sym__simple_statements] = STATE(42),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_if_statement] = STATE(42),
    [sym_for_statement] = STATE(42),
    [sym_while_statement] = STATE(42),
    [sym_try_statement] = STATE(42),
    [sym_with_statement] = STATE(42),
    [sym_function_definition] = STATE(42),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_class_definition] = STATE(42),
    [sym_decorated_definition] = STATE(42),
    [sym_decorator] = STATE(720),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(42),
    [aux_sym_decorated_definition_repeat1] = STATE(720),
    [ts_builtin_sym_end] = ACTIONS(201),
    [sym_identifier] = ACTIONS(99),
    [anon_sym_import] = ACTIONS(102),
    [anon_sym_from] = ACTIONS(105),
    [anon_sym_LPAREN] = ACTIONS(108),
    [anon_sym_STAR] = ACTIONS(111),
    [anon_sym_print] = ACTIONS(114),
    [anon_sym_assert] = ACTIONS(117),
    [anon_sym_return] = ACTIONS(120),
    [anon_sym_del] = ACTIONS(123),
    [anon_sym_raise] = ACTIONS(126),
    [anon_sym_pass] = ACTIONS(129),
    [anon_sym_break] = ACTIONS(132),
    [anon_sym_continue] = ACTIONS(135),
    [anon_sym_if] = ACTIONS(208),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_for] = ACTIONS(214),
    [anon_sym_while] = ACTIONS(217),
    [anon_sym_try] = ACTIONS(220),
    [anon_sym_with] = ACTIONS(223),
    [anon_sym_def] = ACTIONS(226),
    [anon_sym_global] = ACTIONS(159),
    [anon_sym_nonlocal] = ACTIONS(162),
    [anon_sym_exec] = ACTIONS(165),
    [anon_sym_class] = ACTIONS(229),
    [anon_sym_AT] = ACTIONS(171),
    [anon_sym_LBRACK] = ACTIONS(174),
    [anon_sym_not] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(180),
    [anon_sym_DASH] = ACTIONS(180),
    [anon_sym_TILDE] = ACTIONS(180),
    [anon_sym_lambda] = ACTIONS(183),
    [anon_sym_yield] = ACTIONS(186),
    [sym_ellipsis] = ACTIONS(189),
    [anon_sym_LBRACE] = ACTIONS(192),
    [sym_integer] = ACTIONS(195),
    [sym_float] = ACTIONS(189),
    [anon_sym_await] = ACTIONS(198),
    [sym_true] = ACTIONS(195),
    [sym_false] = ACTIONS(195),
    [sym_none] = ACTIONS(195),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(203),
  },
  [43] = {
    [sym__statement] = STATE(40),
    [sym__simple_statements] = STATE(40),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(40),
    [sym_for_statement] = STATE(40),
    [sym_while_statement] = STATE(40),
    [sym_try_statement] = STATE(40),
    [sym_with_statement] = STATE(40),
    [sym_function_definition] = STATE(40),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(40),
    [sym_decorated_definition] = STATE(40),
    [sym_decorator] = STATE(717),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(40),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(232),
    [sym__string_start] = ACTIONS(75),
  },
  [44] = {
    [sym__statement] = STATE(40),
    [sym__simple_statements] = STATE(40),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_if_statement] = STATE(40),
    [sym_for_statement] = STATE(40),
    [sym_while_statement] = STATE(40),
    [sym_try_statement] = STATE(40),
    [sym_with_statement] = STATE(40),
    [sym_function_definition] = STATE(40),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_class_definition] = STATE(40),
    [sym_decorated_definition] = STATE(40),
    [sym_decorator] = STATE(717),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(40),
    [aux_sym_decorated_definition_repeat1] = STATE(717),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(77),
    [anon_sym_async] = ACTIONS(79),
    [anon_sym_for] = ACTIONS(81),
    [anon_sym_while] = ACTIONS(83),
    [anon_sym_try] = ACTIONS(85),
    [anon_sym_with] = ACTIONS(87),
    [anon_sym_def] = ACTIONS(89),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(91),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__dedent] = ACTIONS(234),
    [sym__string_start] = ACTIONS(75),
  },
  [45] = {
    [sym__statement] = STATE(42),
    [sym__simple_statements] = STATE(42),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_if_statement] = STATE(42),
    [sym_for_statement] = STATE(42),
    [sym_while_statement] = STATE(42),
    [sym_try_statement] = STATE(42),
    [sym_with_statement] = STATE(42),
    [sym_function_definition] = STATE(42),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_class_definition] = STATE(42),
    [sym_decorated_definition] = STATE(42),
    [sym_decorator] = STATE(720),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [aux_sym_module_repeat1] = STATE(42),
    [aux_sym_decorated_definition_repeat1] = STATE(720),
    [ts_builtin_sym_end] = ACTIONS(236),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_if] = ACTIONS(33),
    [anon_sym_async] = ACTIONS(35),
    [anon_sym_for] = ACTIONS(37),
    [anon_sym_while] = ACTIONS(39),
    [anon_sym_try] = ACTIONS(41),
    [anon_sym_with] = ACTIONS(43),
    [anon_sym_def] = ACTIONS(45),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_AT] = ACTIONS(55),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(75),
  },
  [46] = {
    [sym_chevron] = STATE(855),
    [sym_named_expression] = STATE(709),
    [sym_expression] = STATE(742),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_attribute] = STATE(605),
    [sym_subscript] = STATE(605),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(238),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(242),
    [anon_sym_COMMA] = ACTIONS(244),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(247),
    [anon_sym_GT_GT] = ACTIONS(249),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_COLON] = ACTIONS(251),
    [anon_sym_async] = ACTIONS(247),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(240),
    [anon_sym_exec] = ACTIONS(247),
    [anon_sym_AT] = ACTIONS(240),
    [anon_sym_LBRACK] = ACTIONS(242),
    [anon_sym_EQ] = ACTIONS(253),
    [anon_sym_not] = ACTIONS(240),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(240),
    [anon_sym_DASH] = ACTIONS(240),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(240),
    [anon_sym_SLASH_SLASH] = ACTIONS(240),
    [anon_sym_PIPE] = ACTIONS(240),
    [anon_sym_AMP] = ACTIONS(240),
    [anon_sym_CARET] = ACTIONS(240),
    [anon_sym_LT_LT] = ACTIONS(240),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_PLUS_EQ] = ACTIONS(251),
    [anon_sym_DASH_EQ] = ACTIONS(251),
    [anon_sym_STAR_EQ] = ACTIONS(251),
    [anon_sym_SLASH_EQ] = ACTIONS(251),
    [anon_sym_AT_EQ] = ACTIONS(251),
    [anon_sym_SLASH_SLASH_EQ] = ACTIONS(251),
    [anon_sym_PERCENT_EQ] = ACTIONS(251),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(251),
    [anon_sym_GT_GT_EQ] = ACTIONS(251),
    [anon_sym_LT_LT_EQ] = ACTIONS(251),
    [anon_sym_AMP_EQ] = ACTIONS(251),
    [anon_sym_CARET_EQ] = ACTIONS(251),
    [anon_sym_PIPE_EQ] = ACTIONS(251),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(255),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__semicolon] = ACTIONS(242),
    [sym__newline] = ACTIONS(242),
    [sym__string_start] = ACTIONS(75),
  },
  [47] = {
    [sym_named_expression] = STATE(709),
    [sym_expression] = STATE(710),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_attribute] = STATE(605),
    [sym_subscript] = STATE(605),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(238),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(257),
    [anon_sym_COMMA] = ACTIONS(244),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(247),
    [anon_sym_GT_GT] = ACTIONS(240),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_COLON] = ACTIONS(251),
    [anon_sym_async] = ACTIONS(247),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(240),
    [anon_sym_exec] = ACTIONS(247),
    [anon_sym_AT] = ACTIONS(240),
    [anon_sym_LBRACK] = ACTIONS(259),
    [anon_sym_EQ] = ACTIONS(253),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(261),
    [anon_sym_DASH] = ACTIONS(261),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(240),
    [anon_sym_SLASH_SLASH] = ACTIONS(240),
    [anon_sym_PIPE] = ACTIONS(240),
    [anon_sym_AMP] = ACTIONS(240),
    [anon_sym_CARET] = ACTIONS(240),
    [anon_sym_LT_LT] = ACTIONS(240),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_PLUS_EQ] = ACTIONS(251),
    [anon_sym_DASH_EQ] = ACTIONS(251),
    [anon_sym_STAR_EQ] = ACTIONS(251),
    [anon_sym_SLASH_EQ] = ACTIONS(251),
    [anon_sym_AT_EQ] = ACTIONS(251),
    [anon_sym_SLASH_SLASH_EQ] = ACTIONS(251),
    [anon_sym_PERCENT_EQ] = ACTIONS(251),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(251),
    [anon_sym_GT_GT_EQ] = ACTIONS(251),
    [anon_sym_LT_LT_EQ] = ACTIONS(251),
    [anon_sym_AMP_EQ] = ACTIONS(251),
    [anon_sym_CARET_EQ] = ACTIONS(251),
    [anon_sym_PIPE_EQ] = ACTIONS(251),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(255),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__semicolon] = ACTIONS(242),
    [sym__newline] = ACTIONS(242),
    [sym__string_start] = ACTIONS(75),
  },
  [48] = {
    [sym__simple_statements] = STATE(421),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(265),
    [sym__indent] = ACTIONS(267),
    [sym__string_start] = ACTIONS(75),
  },
  [49] = {
    [sym__simple_statements] = STATE(329),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(269),
    [sym__indent] = ACTIONS(271),
    [sym__string_start] = ACTIONS(75),
  },
  [50] = {
    [sym__simple_statements] = STATE(409),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(273),
    [sym__indent] = ACTIONS(275),
    [sym__string_start] = ACTIONS(75),
  },
  [51] = {
    [sym__simple_statements] = STATE(328),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(277),
    [sym__indent] = ACTIONS(279),
    [sym__string_start] = ACTIONS(75),
  },
  [52] = {
    [sym__simple_statements] = STATE(279),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(281),
    [sym__indent] = ACTIONS(283),
    [sym__string_start] = ACTIONS(75),
  },
  [53] = {
    [sym__simple_statements] = STATE(377),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(285),
    [sym__indent] = ACTIONS(287),
    [sym__string_start] = ACTIONS(75),
  },
  [54] = {
    [sym__simple_statements] = STATE(371),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(289),
    [sym__indent] = ACTIONS(291),
    [sym__string_start] = ACTIONS(75),
  },
  [55] = {
    [sym__simple_statements] = STATE(398),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(293),
    [sym__indent] = ACTIONS(295),
    [sym__string_start] = ACTIONS(75),
  },
  [56] = {
    [sym__simple_statements] = STATE(260),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(297),
    [sym__indent] = ACTIONS(299),
    [sym__string_start] = ACTIONS(75),
  },
  [57] = {
    [sym__simple_statements] = STATE(362),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(301),
    [sym__indent] = ACTIONS(303),
    [sym__string_start] = ACTIONS(75),
  },
  [58] = {
    [sym__simple_statements] = STATE(339),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(305),
    [sym__indent] = ACTIONS(307),
    [sym__string_start] = ACTIONS(75),
  },
  [59] = {
    [sym__simple_statements] = STATE(342),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(309),
    [sym__indent] = ACTIONS(311),
    [sym__string_start] = ACTIONS(75),
  },
  [60] = {
    [sym__simple_statements] = STATE(341),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(313),
    [sym__indent] = ACTIONS(315),
    [sym__string_start] = ACTIONS(75),
  },
  [61] = {
    [sym__simple_statements] = STATE(209),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(317),
    [sym__indent] = ACTIONS(319),
    [sym__string_start] = ACTIONS(75),
  },
  [62] = {
    [sym__simple_statements] = STATE(295),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(321),
    [sym__indent] = ACTIONS(323),
    [sym__string_start] = ACTIONS(75),
  },
  [63] = {
    [sym__simple_statements] = STATE(427),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(325),
    [sym__indent] = ACTIONS(327),
    [sym__string_start] = ACTIONS(75),
  },
  [64] = {
    [sym__simple_statements] = STATE(413),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(329),
    [sym__indent] = ACTIONS(331),
    [sym__string_start] = ACTIONS(75),
  },
  [65] = {
    [sym__simple_statements] = STATE(812),
    [sym_import_statement] = STATE(878),
    [sym_future_import_statement] = STATE(878),
    [sym_import_from_statement] = STATE(878),
    [sym_print_statement] = STATE(878),
    [sym_assert_statement] = STATE(878),
    [sym_expression_statement] = STATE(878),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(878),
    [sym_delete_statement] = STATE(878),
    [sym_raise_statement] = STATE(878),
    [sym_pass_statement] = STATE(878),
    [sym_break_statement] = STATE(878),
    [sym_continue_statement] = STATE(878),
    [sym_global_statement] = STATE(878),
    [sym_nonlocal_statement] = STATE(878),
    [sym_exec_statement] = STATE(878),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(333),
    [sym__indent] = ACTIONS(335),
    [sym__string_start] = ACTIONS(75),
  },
  [66] = {
    [sym__simple_statements] = STATE(381),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(337),
    [sym__indent] = ACTIONS(339),
    [sym__string_start] = ACTIONS(75),
  },
  [67] = {
    [sym__simple_statements] = STATE(354),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(341),
    [sym__indent] = ACTIONS(343),
    [sym__string_start] = ACTIONS(75),
  },
  [68] = {
    [sym__simple_statements] = STATE(205),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(345),
    [sym__indent] = ACTIONS(347),
    [sym__string_start] = ACTIONS(75),
  },
  [69] = {
    [sym__simple_statements] = STATE(345),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(349),
    [sym__indent] = ACTIONS(351),
    [sym__string_start] = ACTIONS(75),
  },
  [70] = {
    [sym__simple_statements] = STATE(403),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(353),
    [sym__indent] = ACTIONS(355),
    [sym__string_start] = ACTIONS(75),
  },
  [71] = {
    [sym__simple_statements] = STATE(404),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(357),
    [sym__indent] = ACTIONS(359),
    [sym__string_start] = ACTIONS(75),
  },
  [72] = {
    [sym__simple_statements] = STATE(326),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(361),
    [sym__indent] = ACTIONS(363),
    [sym__string_start] = ACTIONS(75),
  },
  [73] = {
    [sym__simple_statements] = STATE(254),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(365),
    [sym__indent] = ACTIONS(367),
    [sym__string_start] = ACTIONS(75),
  },
  [74] = {
    [sym__simple_statements] = STATE(366),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(369),
    [sym__indent] = ACTIONS(371),
    [sym__string_start] = ACTIONS(75),
  },
  [75] = {
    [sym__simple_statements] = STATE(393),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(373),
    [sym__indent] = ACTIONS(375),
    [sym__string_start] = ACTIONS(75),
  },
  [76] = {
    [sym__simple_statements] = STATE(349),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(377),
    [sym__indent] = ACTIONS(379),
    [sym__string_start] = ACTIONS(75),
  },
  [77] = {
    [sym__simple_statements] = STATE(384),
    [sym_import_statement] = STATE(883),
    [sym_future_import_statement] = STATE(883),
    [sym_import_from_statement] = STATE(883),
    [sym_print_statement] = STATE(883),
    [sym_assert_statement] = STATE(883),
    [sym_expression_statement] = STATE(883),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(883),
    [sym_delete_statement] = STATE(883),
    [sym_raise_statement] = STATE(883),
    [sym_pass_statement] = STATE(883),
    [sym_break_statement] = STATE(883),
    [sym_continue_statement] = STATE(883),
    [sym_global_statement] = STATE(883),
    [sym_nonlocal_statement] = STATE(883),
    [sym_exec_statement] = STATE(883),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(381),
    [sym__indent] = ACTIONS(383),
    [sym__string_start] = ACTIONS(75),
  },
  [78] = {
    [sym__simple_statements] = STATE(375),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(385),
    [sym__indent] = ACTIONS(387),
    [sym__string_start] = ACTIONS(75),
  },
  [79] = {
    [sym__simple_statements] = STATE(315),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(389),
    [sym__indent] = ACTIONS(391),
    [sym__string_start] = ACTIONS(75),
  },
  [80] = {
    [sym__simple_statements] = STATE(809),
    [sym_import_statement] = STATE(878),
    [sym_future_import_statement] = STATE(878),
    [sym_import_from_statement] = STATE(878),
    [sym_print_statement] = STATE(878),
    [sym_assert_statement] = STATE(878),
    [sym_expression_statement] = STATE(878),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(878),
    [sym_delete_statement] = STATE(878),
    [sym_raise_statement] = STATE(878),
    [sym_pass_statement] = STATE(878),
    [sym_break_statement] = STATE(878),
    [sym_continue_statement] = STATE(878),
    [sym_global_statement] = STATE(878),
    [sym_nonlocal_statement] = STATE(878),
    [sym_exec_statement] = STATE(878),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(393),
    [sym__indent] = ACTIONS(395),
    [sym__string_start] = ACTIONS(75),
  },
  [81] = {
    [sym__simple_statements] = STATE(407),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(397),
    [sym__indent] = ACTIONS(399),
    [sym__string_start] = ACTIONS(75),
  },
  [82] = {
    [sym__simple_statements] = STATE(273),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(401),
    [sym__indent] = ACTIONS(403),
    [sym__string_start] = ACTIONS(75),
  },
  [83] = {
    [sym__simple_statements] = STATE(405),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(405),
    [sym__indent] = ACTIONS(407),
    [sym__string_start] = ACTIONS(75),
  },
  [84] = {
    [sym__simple_statements] = STATE(334),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(409),
    [sym__indent] = ACTIONS(411),
    [sym__string_start] = ACTIONS(75),
  },
  [85] = {
    [sym__simple_statements] = STATE(392),
    [sym_import_statement] = STATE(953),
    [sym_future_import_statement] = STATE(953),
    [sym_import_from_statement] = STATE(953),
    [sym_print_statement] = STATE(953),
    [sym_assert_statement] = STATE(953),
    [sym_expression_statement] = STATE(953),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(953),
    [sym_delete_statement] = STATE(953),
    [sym_raise_statement] = STATE(953),
    [sym_pass_statement] = STATE(953),
    [sym_break_statement] = STATE(953),
    [sym_continue_statement] = STATE(953),
    [sym_global_statement] = STATE(953),
    [sym_nonlocal_statement] = STATE(953),
    [sym_exec_statement] = STATE(953),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(413),
    [sym__indent] = ACTIONS(415),
    [sym__string_start] = ACTIONS(75),
  },
  [86] = {
    [sym_import_statement] = STATE(955),
    [sym_future_import_statement] = STATE(955),
    [sym_import_from_statement] = STATE(955),
    [sym_print_statement] = STATE(955),
    [sym_assert_statement] = STATE(955),
    [sym_expression_statement] = STATE(955),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(955),
    [sym_delete_statement] = STATE(955),
    [sym_raise_statement] = STATE(955),
    [sym_pass_statement] = STATE(955),
    [sym_break_statement] = STATE(955),
    [sym_continue_statement] = STATE(955),
    [sym_global_statement] = STATE(955),
    [sym_nonlocal_statement] = STATE(955),
    [sym_exec_statement] = STATE(955),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(417),
    [sym__string_start] = ACTIONS(75),
  },
  [87] = {
    [sym_import_statement] = STATE(955),
    [sym_future_import_statement] = STATE(955),
    [sym_import_from_statement] = STATE(955),
    [sym_print_statement] = STATE(955),
    [sym_assert_statement] = STATE(955),
    [sym_expression_statement] = STATE(955),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(955),
    [sym_delete_statement] = STATE(955),
    [sym_raise_statement] = STATE(955),
    [sym_pass_statement] = STATE(955),
    [sym_break_statement] = STATE(955),
    [sym_continue_statement] = STATE(955),
    [sym_global_statement] = STATE(955),
    [sym_nonlocal_statement] = STATE(955),
    [sym_exec_statement] = STATE(955),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(419),
    [sym__string_start] = ACTIONS(75),
  },
  [88] = {
    [sym_import_statement] = STATE(955),
    [sym_future_import_statement] = STATE(955),
    [sym_import_from_statement] = STATE(955),
    [sym_print_statement] = STATE(955),
    [sym_assert_statement] = STATE(955),
    [sym_expression_statement] = STATE(955),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(955),
    [sym_delete_statement] = STATE(955),
    [sym_raise_statement] = STATE(955),
    [sym_pass_statement] = STATE(955),
    [sym_break_statement] = STATE(955),
    [sym_continue_statement] = STATE(955),
    [sym_global_statement] = STATE(955),
    [sym_nonlocal_statement] = STATE(955),
    [sym_exec_statement] = STATE(955),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(421),
    [sym__string_start] = ACTIONS(75),
  },
  [89] = {
    [sym_import_statement] = STATE(955),
    [sym_future_import_statement] = STATE(955),
    [sym_import_from_statement] = STATE(955),
    [sym_print_statement] = STATE(955),
    [sym_assert_statement] = STATE(955),
    [sym_expression_statement] = STATE(955),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(955),
    [sym_delete_statement] = STATE(955),
    [sym_raise_statement] = STATE(955),
    [sym_pass_statement] = STATE(955),
    [sym_break_statement] = STATE(955),
    [sym_continue_statement] = STATE(955),
    [sym_global_statement] = STATE(955),
    [sym_nonlocal_statement] = STATE(955),
    [sym_exec_statement] = STATE(955),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(423),
    [sym__string_start] = ACTIONS(75),
  },
  [90] = {
    [sym_import_statement] = STATE(955),
    [sym_future_import_statement] = STATE(955),
    [sym_import_from_statement] = STATE(955),
    [sym_print_statement] = STATE(955),
    [sym_assert_statement] = STATE(955),
    [sym_expression_statement] = STATE(955),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(955),
    [sym_delete_statement] = STATE(955),
    [sym_raise_statement] = STATE(955),
    [sym_pass_statement] = STATE(955),
    [sym_break_statement] = STATE(955),
    [sym_continue_statement] = STATE(955),
    [sym_global_statement] = STATE(955),
    [sym_nonlocal_statement] = STATE(955),
    [sym_exec_statement] = STATE(955),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(425),
    [sym__string_start] = ACTIONS(75),
  },
  [91] = {
    [sym_import_statement] = STATE(955),
    [sym_future_import_statement] = STATE(955),
    [sym_import_from_statement] = STATE(955),
    [sym_print_statement] = STATE(955),
    [sym_assert_statement] = STATE(955),
    [sym_expression_statement] = STATE(955),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(955),
    [sym_delete_statement] = STATE(955),
    [sym_raise_statement] = STATE(955),
    [sym_pass_statement] = STATE(955),
    [sym_break_statement] = STATE(955),
    [sym_continue_statement] = STATE(955),
    [sym_global_statement] = STATE(955),
    [sym_nonlocal_statement] = STATE(955),
    [sym_exec_statement] = STATE(955),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__newline] = ACTIONS(427),
    [sym__string_start] = ACTIONS(75),
  },
  [92] = {
    [sym_import_statement] = STATE(955),
    [sym_future_import_statement] = STATE(955),
    [sym_import_from_statement] = STATE(955),
    [sym_print_statement] = STATE(955),
    [sym_assert_statement] = STATE(955),
    [sym_expression_statement] = STATE(955),
    [sym_named_expression] = STATE(709),
    [sym_return_statement] = STATE(955),
    [sym_delete_statement] = STATE(955),
    [sym_raise_statement] = STATE(955),
    [sym_pass_statement] = STATE(955),
    [sym_break_statement] = STATE(955),
    [sym_continue_statement] = STATE(955),
    [sym_global_statement] = STATE(955),
    [sym_nonlocal_statement] = STATE(955),
    [sym_exec_statement] = STATE(955),
    [sym_pattern] = STATE(655),
    [sym_tuple_pattern] = STATE(646),
    [sym_list_pattern] = STATE(646),
    [sym_list_splat_pattern] = STATE(646),
    [sym_expression] = STATE(725),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_assignment] = STATE(981),
    [sym_augmented_assignment] = STATE(981),
    [sym_pattern_list] = STATE(656),
    [sym_yield] = STATE(981),
    [sym_attribute] = STATE(208),
    [sym_subscript] = STATE(208),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(7),
    [anon_sym_import] = ACTIONS(9),
    [anon_sym_from] = ACTIONS(11),
    [anon_sym_LPAREN] = ACTIONS(13),
    [anon_sym_STAR] = ACTIONS(15),
    [anon_sym_print] = ACTIONS(17),
    [anon_sym_assert] = ACTIONS(19),
    [anon_sym_return] = ACTIONS(21),
    [anon_sym_del] = ACTIONS(23),
    [anon_sym_raise] = ACTIONS(25),
    [anon_sym_pass] = ACTIONS(27),
    [anon_sym_break] = ACTIONS(29),
    [anon_sym_continue] = ACTIONS(31),
    [anon_sym_async] = ACTIONS(263),
    [anon_sym_global] = ACTIONS(47),
    [anon_sym_nonlocal] = ACTIONS(49),
    [anon_sym_exec] = ACTIONS(51),
    [anon_sym_LBRACK] = ACTIONS(57),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_lambda] = ACTIONS(63),
    [anon_sym_yield] = ACTIONS(65),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(73),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(75),
  },
  [93] = {
    [sym_named_expression] = STATE(664),
    [sym_expression] = STATE(663),
    [sym_primary_expression] = STATE(446),
    [sym_not_operator] = STATE(664),
    [sym_boolean_operator] = STATE(664),
    [sym_binary_operator] = STATE(443),
    [sym_unary_operator] = STATE(443),
    [sym_comparison_operator] = STATE(664),
    [sym_lambda] = STATE(664),
    [sym_attribute] = STATE(443),
    [sym_subscript] = STATE(443),
    [sym_call] = STATE(443),
    [sym_list] = STATE(443),
    [sym_set] = STATE(443),
    [sym_tuple] = STATE(443),
    [sym_dictionary] = STATE(443),
    [sym_list_comprehension] = STATE(443),
    [sym_dictionary_comprehension] = STATE(443),
    [sym_set_comprehension] = STATE(443),
    [sym_generator_expression] = STATE(443),
    [sym_parenthesized_expression] = STATE(443),
    [sym_conditional_expression] = STATE(664),
    [sym_concatenated_string] = STATE(443),
    [sym_string] = STATE(348),
    [sym_await] = STATE(664),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(431),
    [anon_sym_RPAREN] = ACTIONS(242),
    [anon_sym_COMMA] = ACTIONS(242),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(433),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_COLON] = ACTIONS(242),
    [anon_sym_else] = ACTIONS(240),
    [anon_sym_async] = ACTIONS(433),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(433),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(435),
    [anon_sym_RBRACK] = ACTIONS(242),
    [anon_sym_EQ] = ACTIONS(240),
    [anon_sym_not] = ACTIONS(437),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(439),
    [anon_sym_DASH] = ACTIONS(439),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(439),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(441),
    [sym_ellipsis] = ACTIONS(443),
    [anon_sym_LBRACE] = ACTIONS(445),
    [anon_sym_RBRACE] = ACTIONS(242),
    [sym_type_conversion] = ACTIONS(242),
    [sym_integer] = ACTIONS(447),
    [sym_float] = ACTIONS(443),
    [anon_sym_await] = ACTIONS(449),
    [sym_true] = ACTIONS(447),
    [sym_false] = ACTIONS(447),
    [sym_none] = ACTIONS(447),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(451),
  },
  [94] = {
    [sym_named_expression] = STATE(664),
    [sym_expression] = STATE(663),
    [sym_primary_expression] = STATE(448),
    [sym_not_operator] = STATE(664),
    [sym_boolean_operator] = STATE(664),
    [sym_binary_operator] = STATE(443),
    [sym_unary_operator] = STATE(443),
    [sym_comparison_operator] = STATE(664),
    [sym_lambda] = STATE(664),
    [sym_attribute] = STATE(443),
    [sym_subscript] = STATE(443),
    [sym_call] = STATE(443),
    [sym_list] = STATE(443),
    [sym_set] = STATE(443),
    [sym_tuple] = STATE(443),
    [sym_dictionary] = STATE(443),
    [sym_list_comprehension] = STATE(443),
    [sym_dictionary_comprehension] = STATE(443),
    [sym_set_comprehension] = STATE(443),
    [sym_generator_expression] = STATE(443),
    [sym_parenthesized_expression] = STATE(443),
    [sym_conditional_expression] = STATE(664),
    [sym_concatenated_string] = STATE(443),
    [sym_string] = STATE(348),
    [sym_await] = STATE(664),
    [sym_identifier] = ACTIONS(453),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(455),
    [anon_sym_RPAREN] = ACTIONS(242),
    [anon_sym_COMMA] = ACTIONS(242),
    [anon_sym_as] = ACTIONS(240),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(457),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_COLON] = ACTIONS(242),
    [anon_sym_async] = ACTIONS(457),
    [anon_sym_for] = ACTIONS(240),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(457),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(459),
    [anon_sym_RBRACK] = ACTIONS(242),
    [anon_sym_not] = ACTIONS(461),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(463),
    [anon_sym_DASH] = ACTIONS(463),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(463),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(465),
    [sym_ellipsis] = ACTIONS(443),
    [anon_sym_LBRACE] = ACTIONS(445),
    [anon_sym_RBRACE] = ACTIONS(242),
    [sym_integer] = ACTIONS(447),
    [sym_float] = ACTIONS(443),
    [anon_sym_await] = ACTIONS(467),
    [sym_true] = ACTIONS(447),
    [sym_false] = ACTIONS(447),
    [sym_none] = ACTIONS(447),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(451),
  },
  [95] = {
    [sym_named_expression] = STATE(709),
    [sym_expression] = STATE(710),
    [sym_primary_expression] = STATE(487),
    [sym_not_operator] = STATE(709),
    [sym_boolean_operator] = STATE(709),
    [sym_binary_operator] = STATE(605),
    [sym_unary_operator] = STATE(605),
    [sym_comparison_operator] = STATE(709),
    [sym_lambda] = STATE(709),
    [sym_attribute] = STATE(605),
    [sym_subscript] = STATE(605),
    [sym_call] = STATE(605),
    [sym_list] = STATE(605),
    [sym_set] = STATE(605),
    [sym_tuple] = STATE(605),
    [sym_dictionary] = STATE(605),
    [sym_list_comprehension] = STATE(605),
    [sym_dictionary_comprehension] = STATE(605),
    [sym_set_comprehension] = STATE(605),
    [sym_generator_expression] = STATE(605),
    [sym_parenthesized_expression] = STATE(605),
    [sym_conditional_expression] = STATE(709),
    [sym_concatenated_string] = STATE(605),
    [sym_string] = STATE(492),
    [sym_await] = STATE(709),
    [sym_identifier] = ACTIONS(238),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_from] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(257),
    [anon_sym_COMMA] = ACTIONS(242),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(247),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_async] = ACTIONS(247),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(247),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(259),
    [anon_sym_EQ] = ACTIONS(240),
    [anon_sym_not] = ACTIONS(59),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(61),
    [anon_sym_DASH] = ACTIONS(61),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(61),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(63),
    [sym_ellipsis] = ACTIONS(67),
    [anon_sym_LBRACE] = ACTIONS(69),
    [sym_integer] = ACTIONS(71),
    [sym_float] = ACTIONS(67),
    [anon_sym_await] = ACTIONS(255),
    [sym_true] = ACTIONS(71),
    [sym_false] = ACTIONS(71),
    [sym_none] = ACTIONS(71),
    [sym_comment] = ACTIONS(3),
    [sym__semicolon] = ACTIONS(242),
    [sym__newline] = ACTIONS(242),
    [sym__string_start] = ACTIONS(75),
  },
  [96] = {
    [sym_named_expression] = STATE(664),
    [sym_expression] = STATE(663),
    [sym_primary_expression] = STATE(448),
    [sym_not_operator] = STATE(664),
    [sym_boolean_operator] = STATE(664),
    [sym_binary_operator] = STATE(443),
    [sym_unary_operator] = STATE(443),
    [sym_comparison_operator] = STATE(664),
    [sym_lambda] = STATE(664),
    [sym_attribute] = STATE(443),
    [sym_subscript] = STATE(443),
    [sym_call] = STATE(443),
    [sym_list] = STATE(443),
    [sym_set] = STATE(443),
    [sym_tuple] = STATE(443),
    [sym_dictionary] = STATE(443),
    [sym_list_comprehension] = STATE(443),
    [sym_dictionary_comprehension] = STATE(443),
    [sym_set_comprehension] = STATE(443),
    [sym_generator_expression] = STATE(443),
    [sym_parenthesized_expression] = STATE(443),
    [sym_conditional_expression] = STATE(664),
    [sym_concatenated_string] = STATE(443),
    [sym_string] = STATE(348),
    [sym_await] = STATE(664),
    [sym_identifier] = ACTIONS(453),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(455),
    [anon_sym_RPAREN] = ACTIONS(244),
    [anon_sym_COMMA] = ACTIONS(244),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(457),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_async] = ACTIONS(457),
    [anon_sym_for] = ACTIONS(240),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(457),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(459),
    [anon_sym_RBRACK] = ACTIONS(244),
    [anon_sym_not] = ACTIONS(461),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(463),
    [anon_sym_DASH] = ACTIONS(463),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(463),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(465),
    [sym_ellipsis] = ACTIONS(443),
    [anon_sym_LBRACE] = ACTIONS(445),
    [sym_integer] = ACTIONS(447),
    [sym_float] = ACTIONS(443),
    [anon_sym_await] = ACTIONS(467),
    [sym_true] = ACTIONS(447),
    [sym_false] = ACTIONS(447),
    [sym_none] = ACTIONS(447),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(451),
  },
  [97] = {
    [sym_named_expression] = STATE(749),
    [sym_expression] = STATE(757),
    [sym_primary_expression] = STATE(496),
    [sym_not_operator] = STATE(749),
    [sym_boolean_operator] = STATE(749),
    [sym_binary_operator] = STATE(628),
    [sym_unary_operator] = STATE(628),
    [sym_comparison_operator] = STATE(749),
    [sym_lambda] = STATE(749),
    [sym_attribute] = STATE(628),
    [sym_subscript] = STATE(628),
    [sym_call] = STATE(628),
    [sym_list] = STATE(628),
    [sym_set] = STATE(628),
    [sym_tuple] = STATE(628),
    [sym_dictionary] = STATE(628),
    [sym_list_comprehension] = STATE(628),
    [sym_dictionary_comprehension] = STATE(628),
    [sym_set_comprehension] = STATE(628),
    [sym_generator_expression] = STATE(628),
    [sym_parenthesized_expression] = STATE(628),
    [sym_conditional_expression] = STATE(749),
    [sym_concatenated_string] = STATE(628),
    [sym_string] = STATE(501),
    [sym_await] = STATE(749),
    [sym_identifier] = ACTIONS(469),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(471),
    [anon_sym_RPAREN] = ACTIONS(242),
    [anon_sym_COMMA] = ACTIONS(242),
    [anon_sym_as] = ACTIONS(240),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(433),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_COLON] = ACTIONS(242),
    [anon_sym_async] = ACTIONS(433),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(433),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(473),
    [anon_sym_not] = ACTIONS(475),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(477),
    [anon_sym_DASH] = ACTIONS(477),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(477),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(479),
    [sym_ellipsis] = ACTIONS(481),
    [anon_sym_LBRACE] = ACTIONS(483),
    [sym_integer] = ACTIONS(485),
    [sym_float] = ACTIONS(481),
    [anon_sym_await] = ACTIONS(487),
    [sym_true] = ACTIONS(485),
    [sym_false] = ACTIONS(485),
    [sym_none] = ACTIONS(485),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(489),
  },
  [98] = {
    [sym_named_expression] = STATE(664),
    [sym_expression] = STATE(663),
    [sym_primary_expression] = STATE(448),
    [sym_not_operator] = STATE(664),
    [sym_boolean_operator] = STATE(664),
    [sym_binary_operator] = STATE(443),
    [sym_unary_operator] = STATE(443),
    [sym_comparison_operator] = STATE(664),
    [sym_lambda] = STATE(664),
    [sym_attribute] = STATE(443),
    [sym_subscript] = STATE(443),
    [sym_call] = STATE(443),
    [sym_list] = STATE(443),
    [sym_set] = STATE(443),
    [sym_tuple] = STATE(443),
    [sym_dictionary] = STATE(443),
    [sym_list_comprehension] = STATE(443),
    [sym_dictionary_comprehension] = STATE(443),
    [sym_set_comprehension] = STATE(443),
    [sym_generator_expression] = STATE(443),
    [sym_parenthesized_expression] = STATE(443),
    [sym_conditional_expression] = STATE(664),
    [sym_concatenated_string] = STATE(443),
    [sym_string] = STATE(348),
    [sym_await] = STATE(664),
    [sym_identifier] = ACTIONS(453),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(455),
    [anon_sym_RPAREN] = ACTIONS(242),
    [anon_sym_COMMA] = ACTIONS(242),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(457),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_async] = ACTIONS(457),
    [anon_sym_for] = ACTIONS(240),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(457),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(459),
    [anon_sym_EQ] = ACTIONS(491),
    [anon_sym_not] = ACTIONS(461),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(463),
    [anon_sym_DASH] = ACTIONS(463),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(463),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(465),
    [sym_ellipsis] = ACTIONS(443),
    [anon_sym_LBRACE] = ACTIONS(445),
    [sym_integer] = ACTIONS(447),
    [sym_float] = ACTIONS(443),
    [anon_sym_await] = ACTIONS(467),
    [sym_true] = ACTIONS(447),
    [sym_false] = ACTIONS(447),
    [sym_none] = ACTIONS(447),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(451),
  },
  [99] = {
    [sym_named_expression] = STATE(664),
    [sym_expression] = STATE(663),
    [sym_primary_expression] = STATE(446),
    [sym_not_operator] = STATE(664),
    [sym_boolean_operator] = STATE(664),
    [sym_binary_operator] = STATE(443),
    [sym_unary_operator] = STATE(443),
    [sym_comparison_operator] = STATE(664),
    [sym_lambda] = STATE(664),
    [sym_attribute] = STATE(443),
    [sym_subscript] = STATE(443),
    [sym_call] = STATE(443),
    [sym_list] = STATE(443),
    [sym_set] = STATE(443),
    [sym_tuple] = STATE(443),
    [sym_dictionary] = STATE(443),
    [sym_list_comprehension] = STATE(443),
    [sym_dictionary_comprehension] = STATE(443),
    [sym_set_comprehension] = STATE(443),
    [sym_generator_expression] = STATE(443),
    [sym_parenthesized_expression] = STATE(443),
    [sym_conditional_expression] = STATE(664),
    [sym_concatenated_string] = STATE(443),
    [sym_string] = STATE(348),
    [sym_await] = STATE(664),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(431),
    [anon_sym_RPAREN] = ACTIONS(242),
    [anon_sym_COMMA] = ACTIONS(242),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(433),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_async] = ACTIONS(433),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(433),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(435),
    [anon_sym_EQ] = ACTIONS(491),
    [anon_sym_not] = ACTIONS(437),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(439),
    [anon_sym_DASH] = ACTIONS(439),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(439),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(441),
    [sym_ellipsis] = ACTIONS(443),
    [anon_sym_LBRACE] = ACTIONS(445),
    [sym_integer] = ACTIONS(447),
    [sym_float] = ACTIONS(443),
    [anon_sym_await] = ACTIONS(449),
    [sym_true] = ACTIONS(447),
    [sym_false] = ACTIONS(447),
    [sym_none] = ACTIONS(447),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(451),
  },
  [100] = {
    [sym_named_expression] = STATE(664),
    [sym_expression] = STATE(663),
    [sym_primary_expression] = STATE(446),
    [sym_not_operator] = STATE(664),
    [sym_boolean_operator] = STATE(664),
    [sym_binary_operator] = STATE(443),
    [sym_unary_operator] = STATE(443),
    [sym_comparison_operator] = STATE(664),
    [sym_lambda] = STATE(664),
    [sym_attribute] = STATE(443),
    [sym_subscript] = STATE(443),
    [sym_call] = STATE(443),
    [sym_list] = STATE(443),
    [sym_set] = STATE(443),
    [sym_tuple] = STATE(443),
    [sym_dictionary] = STATE(443),
    [sym_list_comprehension] = STATE(443),
    [sym_dictionary_comprehension] = STATE(443),
    [sym_set_comprehension] = STATE(443),
    [sym_generator_expression] = STATE(443),
    [sym_parenthesized_expression] = STATE(443),
    [sym_conditional_expression] = STATE(664),
    [sym_concatenated_string] = STATE(443),
    [sym_string] = STATE(348),
    [sym_await] = STATE(664),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_DOT] = ACTIONS(240),
    [anon_sym_LPAREN] = ACTIONS(431),
    [anon_sym_RPAREN] = ACTIONS(493),
    [anon_sym_COMMA] = ACTIONS(493),
    [anon_sym_STAR] = ACTIONS(240),
    [anon_sym_print] = ACTIONS(433),
    [anon_sym_GT_GT] = ACTIONS(242),
    [anon_sym_if] = ACTIONS(240),
    [anon_sym_async] = ACTIONS(433),
    [anon_sym_in] = ACTIONS(240),
    [anon_sym_STAR_STAR] = ACTIONS(242),
    [anon_sym_exec] = ACTIONS(433),
    [anon_sym_AT] = ACTIONS(242),
    [anon_sym_LBRACK] = ACTIONS(435),
    [anon_sym_RBRACK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(437),
    [anon_sym_and] = ACTIONS(240),
    [anon_sym_or] = ACTIONS(240),
    [anon_sym_PLUS] = ACTIONS(439),
    [anon_sym_DASH] = ACTIONS(439),
    [anon_sym_SLASH] = ACTIONS(240),
    [anon_sym_PERCENT] = ACTIONS(242),
    [anon_sym_SLASH_SLASH] = ACTIONS(242),
    [anon_sym_PIPE] = ACTIONS(242),
    [anon_sym_AMP] = ACTIONS(242),
    [anon_sym_CARET] = ACTIONS(242),
    [anon_sym_LT_LT] = ACTIONS(242),
    [anon_sym_TILDE] = ACTIONS(439),
    [anon_sym_LT] = ACTIONS(240),
    [anon_sym_LT_EQ] = ACTIONS(242),
    [anon_sym_EQ_EQ] = ACTIONS(242),
    [anon_sym_BANG_EQ] = ACTIONS(242),
    [anon_sym_GT_EQ] = ACTIONS(242),
    [anon_sym_GT] = ACTIONS(240),
    [anon_sym_LT_GT] = ACTIONS(242),
    [anon_sym_is] = ACTIONS(240),
    [anon_sym_lambda] = ACTIONS(441),
    [sym_ellipsis] = ACTIONS(443),
    [anon_sym_LBRACE] = ACTIONS(445),
    [sym_integer] = ACTIONS(447),
    [sym_float] = ACTIONS(443),
    [anon_sym_await] = ACTIONS(449),
    [sym_true] = ACTIONS(447),
    [sym_false] = ACTIONS(447),
    [sym_none] = ACTIONS(447),
    [sym_comment] = ACTIONS(3),
    [sym__string_start] = ACTIONS(451),
  },
};

static uint16_t ts_small_parse_table[] = {
  [0] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(496), 1,
      sym_identifier,
    ACTIONS(498), 1,
      anon_sym_LPAREN,
    ACTIONS(500), 1,
      anon_sym_STAR,
    ACTIONS(504), 1,
      anon_sym_LBRACK,
    ACTIONS(506), 1,
      anon_sym_RBRACK,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(510), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(690), 1,
      sym_expression,
    STATE(824), 1,
      sym_pattern,
    STATE(1071), 1,
      sym__patterns,
    STATE(1076), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(579), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(502), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [113] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(496), 1,
      sym_identifier,
    ACTIONS(498), 1,
      anon_sym_LPAREN,
    ACTIONS(500), 1,
      anon_sym_STAR,
    ACTIONS(504), 1,
      anon_sym_LBRACK,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(510), 1,
      anon_sym_await,
    ACTIONS(512), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(692), 1,
      sym_expression,
    STATE(824), 1,
      sym_pattern,
    STATE(1064), 1,
      sym__collection_elements,
    STATE(1071), 1,
      sym__patterns,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(579), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(502), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [226] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(7), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_LPAREN,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(57), 1,
      anon_sym_LBRACK,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(65), 1,
      anon_sym_yield,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(73), 1,
      anon_sym_await,
    ACTIONS(75), 1,
      sym__string_start,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(655), 1,
      sym_pattern,
    STATE(656), 1,
      sym_pattern_list,
    STATE(729), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    STATE(208), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(263), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(990), 5,
      sym_expression_list,
      sym_assignment,
      sym_augmented_assignment,
      sym__right_hand_side,
      sym_yield,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [335] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(496), 1,
      sym_identifier,
    ACTIONS(498), 1,
      anon_sym_LPAREN,
    ACTIONS(500), 1,
      anon_sym_STAR,
    ACTIONS(504), 1,
      anon_sym_LBRACK,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(510), 1,
      anon_sym_await,
    ACTIONS(514), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(694), 1,
      sym_expression,
    STATE(824), 1,
      sym_pattern,
    STATE(914), 1,
      sym_yield,
    STATE(1000), 1,
      sym__collection_elements,
    STATE(1014), 1,
      sym__patterns,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(579), 2,
      sym_attribute,
      sym_subscript,
    STATE(814), 2,
      sym_list_splat,
      sym_parenthesized_list_splat,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(502), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [450] = 29,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(496), 1,
      sym_identifier,
    ACTIONS(498), 1,
      anon_sym_LPAREN,
    ACTIONS(500), 1,
      anon_sym_STAR,
    ACTIONS(504), 1,
      anon_sym_LBRACK,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(510), 1,
      anon_sym_await,
    ACTIONS(516), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(691), 1,
      sym_expression,
    STATE(824), 1,
      sym_pattern,
    STATE(882), 1,
      sym_yield,
    STATE(885), 1,
      sym_parenthesized_list_splat,
    STATE(889), 1,
      sym_list_splat,
    STATE(1014), 1,
      sym__patterns,
    STATE(1067), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(579), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(502), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [567] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(496), 1,
      sym_identifier,
    ACTIONS(498), 1,
      anon_sym_LPAREN,
    ACTIONS(500), 1,
      anon_sym_STAR,
    ACTIONS(504), 1,
      anon_sym_LBRACK,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(510), 1,
      anon_sym_await,
    ACTIONS(518), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(692), 1,
      sym_expression,
    STATE(824), 1,
      sym_pattern,
    STATE(1064), 1,
      sym__collection_elements,
    STATE(1071), 1,
      sym__patterns,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(579), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(502), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [680] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(7), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_LPAREN,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(57), 1,
      anon_sym_LBRACK,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(65), 1,
      anon_sym_yield,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(73), 1,
      anon_sym_await,
    ACTIONS(75), 1,
      sym__string_start,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(655), 1,
      sym_pattern,
    STATE(656), 1,
      sym_pattern_list,
    STATE(729), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    STATE(208), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(263), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(986), 5,
      sym_expression_list,
      sym_assignment,
      sym_augmented_assignment,
      sym__right_hand_side,
      sym_yield,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [789] = 28,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(496), 1,
      sym_identifier,
    ACTIONS(498), 1,
      anon_sym_LPAREN,
    ACTIONS(500), 1,
      anon_sym_STAR,
    ACTIONS(504), 1,
      anon_sym_LBRACK,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(510), 1,
      anon_sym_await,
    ACTIONS(520), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(691), 1,
      sym_expression,
    STATE(824), 1,
      sym_pattern,
    STATE(882), 1,
      sym_yield,
    STATE(1014), 1,
      sym__patterns,
    STATE(1067), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(579), 2,
      sym_attribute,
      sym_subscript,
    STATE(814), 2,
      sym_list_splat,
      sym_parenthesized_list_splat,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(502), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [904] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(7), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_LPAREN,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(57), 1,
      anon_sym_LBRACK,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(65), 1,
      anon_sym_yield,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(73), 1,
      anon_sym_await,
    ACTIONS(75), 1,
      sym__string_start,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(655), 1,
      sym_pattern,
    STATE(656), 1,
      sym_pattern_list,
    STATE(729), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    STATE(208), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(263), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(987), 5,
      sym_expression_list,
      sym_assignment,
      sym_augmented_assignment,
      sym__right_hand_side,
      sym_yield,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [1013] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(524), 1,
      anon_sym_COMMA,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(530), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(673), 1,
      sym_expression,
    STATE(730), 1,
      sym_pair,
    STATE(891), 1,
      sym_dictionary_splat,
    STATE(1054), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [1125] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(532), 1,
      anon_sym_COMMA,
    ACTIONS(534), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(682), 1,
      sym_expression,
    STATE(748), 1,
      sym_pair,
    STATE(926), 1,
      sym_dictionary_splat,
    STATE(1027), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [1237] = 27,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(536), 1,
      anon_sym_COMMA,
    ACTIONS(538), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(678), 1,
      sym_expression,
    STATE(754), 1,
      sym_pair,
    STATE(906), 1,
      sym_dictionary_splat,
    STATE(1025), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [1349] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(439), 1,
      anon_sym_TILDE,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(544), 1,
      anon_sym_STAR,
    ACTIONS(550), 1,
      anon_sym_in,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(647), 1,
      sym_pattern,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(554), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
    ACTIONS(548), 15,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [1447] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(439), 1,
      anon_sym_TILDE,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(544), 1,
      anon_sym_STAR,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    ACTIONS(558), 1,
      anon_sym_in,
    STATE(348), 1,
      sym_string,
    STATE(647), 1,
      sym_pattern,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(554), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
    ACTIONS(556), 15,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [1545] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(724), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(562), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(867), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [1644] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(568), 17,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(566), 36,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [1705] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(572), 17,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(570), 36,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [1766] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(574), 1,
      sym_identifier,
    ACTIONS(576), 1,
      anon_sym_RPAREN,
    ACTIONS(578), 1,
      anon_sym_COMMA,
    ACTIONS(582), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(693), 1,
      sym_expression,
    STATE(935), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(932), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [1869] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(724), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(584), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(867), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [1968] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(574), 1,
      sym_identifier,
    ACTIONS(582), 1,
      anon_sym_await,
    ACTIONS(586), 1,
      anon_sym_RPAREN,
    ACTIONS(588), 1,
      anon_sym_COMMA,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(696), 1,
      sym_expression,
    STATE(920), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(919), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2071] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(592), 17,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(590), 36,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [2132] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(574), 1,
      sym_identifier,
    ACTIONS(582), 1,
      anon_sym_await,
    ACTIONS(594), 1,
      anon_sym_RPAREN,
    ACTIONS(596), 1,
      anon_sym_COMMA,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(695), 1,
      sym_expression,
    STATE(907), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(912), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2235] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(598), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(670), 1,
      sym_expression,
    STATE(884), 1,
      sym_with_item,
    STATE(929), 1,
      sym_yield,
    STATE(1038), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(814), 2,
      sym_list_splat,
      sym_parenthesized_list_splat,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2340] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(586), 1,
      anon_sym_RPAREN,
    ACTIONS(588), 1,
      anon_sym_COMMA,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(790), 1,
      sym_expression,
    STATE(920), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(919), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2443] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(606), 17,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(604), 36,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [2504] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(700), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 2,
      anon_sym_print,
      anon_sym_exec,
    STATE(747), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(608), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    ACTIONS(610), 3,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2600] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(614), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(689), 1,
      sym_expression,
    STATE(1034), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2700] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(700), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 2,
      anon_sym_print,
      anon_sym_exec,
    STATE(747), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(616), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    ACTIONS(618), 3,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2796] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(620), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2896] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(622), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [2996] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(624), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3096] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(626), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3196] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(628), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(691), 1,
      sym_expression,
    STATE(882), 1,
      sym_yield,
    STATE(885), 1,
      sym_parenthesized_list_splat,
    STATE(889), 1,
      sym_list_splat,
    STATE(1067), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3300] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(598), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(688), 1,
      sym_expression,
    STATE(929), 1,
      sym_yield,
    STATE(1038), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(814), 2,
      sym_list_splat,
      sym_parenthesized_list_splat,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3402] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(598), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(691), 1,
      sym_expression,
    STATE(882), 1,
      sym_yield,
    STATE(1067), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(814), 2,
      sym_list_splat,
      sym_parenthesized_list_splat,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3504] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(630), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3604] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(632), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3704] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(634), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3804] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(636), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [3904] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(598), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(691), 1,
      sym_expression,
    STATE(882), 1,
      sym_yield,
    STATE(885), 1,
      sym_parenthesized_list_splat,
    STATE(889), 1,
      sym_list_splat,
    STATE(1067), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4008] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(638), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4108] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(640), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(690), 1,
      sym_expression,
    STATE(1076), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4208] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(642), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4308] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(700), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 2,
      anon_sym_print,
      anon_sym_exec,
    STATE(747), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(644), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    ACTIONS(646), 3,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4404] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(648), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4504] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(628), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(691), 1,
      sym_expression,
    STATE(882), 1,
      sym_yield,
    STATE(1067), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(814), 2,
      sym_list_splat,
      sym_parenthesized_list_splat,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4606] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(650), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(692), 1,
      sym_expression,
    STATE(1064), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4706] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(652), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(694), 1,
      sym_expression,
    STATE(914), 1,
      sym_yield,
    STATE(1000), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(814), 2,
      sym_list_splat,
      sym_parenthesized_list_splat,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4808] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(522), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    ACTIONS(614), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(692), 1,
      sym_expression,
    STATE(1064), 1,
      sym__collection_elements,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(814), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [4908] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    ACTIONS(654), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5008] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(700), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 2,
      anon_sym_print,
      anon_sym_exec,
    STATE(747), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(656), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    ACTIONS(658), 3,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5104] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(660), 1,
      anon_sym_from,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(708), 1,
      sym_expression,
    STATE(869), 1,
      sym_expression_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(662), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5199] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(526), 1,
      anon_sym_STAR,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      sym_identifier,
    ACTIONS(602), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(823), 1,
      sym_expression,
    STATE(983), 1,
      sym_parenthesized_list_splat,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(580), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    STATE(984), 3,
      sym_list_splat,
      sym_dictionary_splat,
      sym_keyword_argument,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5296] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(244), 1,
      anon_sym_COMMA,
    ACTIONS(253), 1,
      anon_sym_EQ,
    ACTIONS(664), 1,
      anon_sym_for,
    ACTIONS(666), 1,
      anon_sym_with,
    ACTIONS(668), 1,
      anon_sym_def,
    ACTIONS(251), 14,
      anon_sym_COLON,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(240), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [5366] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(718), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(670), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5456] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(718), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(672), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5546] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(244), 1,
      anon_sym_COMMA,
    ACTIONS(253), 1,
      anon_sym_EQ,
    ACTIONS(674), 1,
      anon_sym_for,
    ACTIONS(676), 1,
      anon_sym_with,
    ACTIONS(678), 1,
      anon_sym_def,
    ACTIONS(251), 14,
      anon_sym_COLON,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(240), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [5616] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(508), 1,
      anon_sym_yield,
    ACTIONS(560), 1,
      anon_sym_LPAREN,
    ACTIONS(564), 1,
      anon_sym_STAR,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(724), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(867), 3,
      sym_list_splat,
      sym_parenthesized_list_splat,
      sym_yield,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5710] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(680), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(856), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(991), 2,
      sym_dictionary_splat,
      sym_pair,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5803] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(682), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(856), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(991), 2,
      sym_dictionary_splat,
      sym_pair,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5896] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    ACTIONS(684), 1,
      anon_sym_from,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(719), 1,
      sym_expression,
    STATE(942), 1,
      sym_expression_list,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(686), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [5989] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    ACTIONS(688), 1,
      anon_sym_from,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(751), 1,
      sym_expression,
    STATE(959), 1,
      sym_expression_list,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(662), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [6082] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(690), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(856), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(991), 2,
      sym_dictionary_splat,
      sym_pair,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [6175] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    ACTIONS(698), 1,
      anon_sym_except,
    ACTIONS(700), 1,
      anon_sym_finally,
    STATE(327), 1,
      sym_else_clause,
    STATE(365), 1,
      sym_finally_clause,
    STATE(197), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
    ACTIONS(692), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(694), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [6244] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    ACTIONS(708), 1,
      anon_sym_except,
    ACTIONS(710), 1,
      anon_sym_finally,
    STATE(330), 1,
      sym_else_clause,
    STATE(417), 1,
      sym_finally_clause,
    STATE(218), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
    ACTIONS(704), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(702), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [6313] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(244), 1,
      anon_sym_COMMA,
    ACTIONS(253), 1,
      anon_sym_EQ,
    ACTIONS(712), 1,
      sym__string_start,
    STATE(946), 1,
      sym_string,
    ACTIONS(251), 14,
      anon_sym_COLON,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(240), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [6380] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(714), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(856), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(991), 2,
      sym_dictionary_splat,
      sym_pair,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [6473] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    ACTIONS(698), 1,
      anon_sym_except,
    ACTIONS(700), 1,
      anon_sym_finally,
    STATE(340), 1,
      sym_else_clause,
    STATE(359), 1,
      sym_finally_clause,
    STATE(197), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
    ACTIONS(704), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(702), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [6542] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(716), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(856), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(991), 2,
      sym_dictionary_splat,
      sym_pair,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [6635] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    ACTIONS(718), 1,
      anon_sym_RBRACE,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(856), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(991), 2,
      sym_dictionary_splat,
      sym_pair,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [6728] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    ACTIONS(708), 1,
      anon_sym_except,
    ACTIONS(710), 1,
      anon_sym_finally,
    STATE(347), 1,
      sym_else_clause,
    STATE(370), 1,
      sym_finally_clause,
    STATE(218), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
    ACTIONS(692), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(694), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [6797] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(606), 16,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(604), 32,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym__semicolon,
  [6853] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    ACTIONS(722), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(768), 1,
      sym_expression,
    STATE(982), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [6945] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(592), 16,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(590), 32,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym__semicolon,
  [7001] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(726), 1,
      anon_sym_COMMA,
    ACTIONS(731), 1,
      anon_sym_COLON_EQ,
    ACTIONS(733), 2,
      anon_sym_COLON,
      anon_sym_EQ,
    ACTIONS(735), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(729), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [7065] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    ACTIONS(737), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(768), 1,
      sym_expression,
    STATE(982), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7157] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(528), 1,
      anon_sym_STAR_STAR,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(856), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(991), 2,
      sym_dictionary_splat,
      sym_pair,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7247] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    ACTIONS(739), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(768), 1,
      sym_expression,
    STATE(982), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7339] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(743), 1,
      anon_sym_COLON,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(787), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(741), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7429] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(731), 1,
      sym_expression,
    STATE(980), 1,
      sym_expression_list,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(745), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7519] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    ACTIONS(747), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(768), 1,
      sym_expression,
    STATE(982), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7611] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(568), 16,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(566), 32,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym__semicolon,
  [7667] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    ACTIONS(749), 1,
      anon_sym_from,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(767), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(670), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7757] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    ACTIONS(751), 1,
      anon_sym_from,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(767), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(672), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7847] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(572), 16,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_EQ,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(570), 32,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym__semicolon,
  [7903] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(755), 1,
      anon_sym_COLON,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(780), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(753), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [7993] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    ACTIONS(757), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(768), 1,
      sym_expression,
    STATE(982), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8085] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    ACTIONS(759), 1,
      anon_sym_RBRACK,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(768), 1,
      sym_expression,
    STATE(982), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8177] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(767), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(761), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8264] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    ACTIONS(767), 1,
      anon_sym_elif,
    STATE(202), 1,
      aux_sym_if_statement_repeat1,
    STATE(332), 1,
      sym_elif_clause,
    STATE(388), 1,
      sym_else_clause,
    ACTIONS(763), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(765), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [8329] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(768), 1,
      sym_expression,
    STATE(982), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8418] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(700), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(721), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8505] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(819), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(753), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8592] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    ACTIONS(769), 1,
      anon_sym_LPAREN,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(740), 1,
      sym_expression,
    STATE(873), 1,
      sym_with_item,
    STATE(1058), 1,
      sym_with_clause,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8681] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    ACTIONS(775), 1,
      anon_sym_elif,
    STATE(230), 1,
      aux_sym_if_statement_repeat1,
    STATE(343), 1,
      sym_elif_clause,
    STATE(394), 1,
      sym_else_clause,
    ACTIONS(773), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(771), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [8746] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    ACTIONS(775), 1,
      anon_sym_elif,
    STATE(206), 1,
      aux_sym_if_statement_repeat1,
    STATE(343), 1,
      sym_elif_clause,
    STATE(355), 1,
      sym_else_clause,
    ACTIONS(763), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(765), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [8811] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(781), 1,
      anon_sym_except,
    STATE(197), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
    ACTIONS(777), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(779), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [8870] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(700), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(747), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [8957] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    ACTIONS(769), 1,
      anon_sym_LPAREN,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(740), 1,
      sym_expression,
    STATE(873), 1,
      sym_with_item,
    STATE(1084), 1,
      sym_with_clause,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [9046] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(767), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(784), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [9133] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(788), 1,
      anon_sym_COMMA,
    ACTIONS(795), 1,
      anon_sym_EQ,
    ACTIONS(793), 14,
      anon_sym_COLON,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(791), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [9194] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    ACTIONS(767), 1,
      anon_sym_elif,
    STATE(241), 1,
      aux_sym_if_statement_repeat1,
    STATE(332), 1,
      sym_elif_clause,
    STATE(387), 1,
      sym_else_clause,
    ACTIONS(797), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(799), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [9259] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(782), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(801), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [9346] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(782), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(803), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [9433] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    ACTIONS(775), 1,
      anon_sym_elif,
    STATE(195), 1,
      aux_sym_if_statement_repeat1,
    STATE(343), 1,
      sym_elif_clause,
    STATE(411), 1,
      sym_else_clause,
    ACTIONS(807), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(805), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [9498] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    ACTIONS(775), 1,
      anon_sym_elif,
    STATE(230), 1,
      aux_sym_if_statement_repeat1,
    STATE(343), 1,
      sym_elif_clause,
    STATE(364), 1,
      sym_else_clause,
    ACTIONS(797), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(799), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [9563] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(803), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(809), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [9650] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(726), 1,
      anon_sym_COMMA,
    ACTIONS(733), 1,
      anon_sym_EQ,
    ACTIONS(735), 14,
      anon_sym_COLON,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(729), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [9711] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    ACTIONS(767), 1,
      anon_sym_elif,
    STATE(217), 1,
      aux_sym_if_statement_repeat1,
    STATE(332), 1,
      sym_elif_clause,
    STATE(422), 1,
      sym_else_clause,
    ACTIONS(807), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(805), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [9776] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    ACTIONS(769), 1,
      anon_sym_LPAREN,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(740), 1,
      sym_expression,
    STATE(873), 1,
      sym_with_item,
    STATE(1036), 1,
      sym_with_clause,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [9865] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(801), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(811), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [9952] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(762), 1,
      sym_expression,
    STATE(923), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10041] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(782), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(813), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10128] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(782), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(815), 2,
      sym__newline,
      sym__semicolon,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10215] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(759), 1,
      sym_expression,
    STATE(936), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10304] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(819), 1,
      anon_sym_COMMA,
    ACTIONS(826), 1,
      anon_sym_EQ,
    ACTIONS(824), 14,
      anon_sym_COLON,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(822), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(817), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [10365] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    ACTIONS(767), 1,
      anon_sym_elif,
    STATE(241), 1,
      aux_sym_if_statement_repeat1,
    STATE(332), 1,
      sym_elif_clause,
    STATE(390), 1,
      sym_else_clause,
    ACTIONS(773), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(771), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [10430] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(828), 1,
      anon_sym_except,
    STATE(218), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
    ACTIONS(777), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(779), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [10489] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    ACTIONS(769), 1,
      anon_sym_LPAREN,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(740), 1,
      sym_expression,
    STATE(873), 1,
      sym_with_item,
    STATE(1042), 1,
      sym_with_clause,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10578] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(698), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(743), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10665] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(244), 1,
      anon_sym_COMMA,
    ACTIONS(253), 1,
      anon_sym_EQ,
    ACTIONS(251), 14,
      anon_sym_COLON,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(240), 15,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 16,
      sym__newline,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_if,
      anon_sym_in,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [10726] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(699), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(752), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10813] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(467), 1,
      anon_sym_await,
    ACTIONS(612), 1,
      anon_sym_lambda,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(700), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(704), 2,
      sym__expression_within_for_in_clause,
      sym_lambda_within_for_in_clause,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10900] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(720), 1,
      anon_sym_COLON,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(763), 1,
      sym_expression,
    STATE(904), 1,
      sym_slice,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [10989] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(783), 1,
      sym_expression,
    STATE(1024), 1,
      sym_expression_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11075] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(831), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(833), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [11129] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(740), 1,
      sym_expression,
    STATE(951), 1,
      sym_with_item,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11215] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(835), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(837), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [11269] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(737), 1,
      sym_expression,
    STATE(1057), 1,
      sym_type,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11355] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(843), 1,
      anon_sym_elif,
    STATE(230), 1,
      aux_sym_if_statement_repeat1,
    STATE(343), 1,
      sym_elif_clause,
    ACTIONS(841), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(839), 31,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [11415] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    ACTIONS(846), 1,
      anon_sym_COLON,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(777), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11501] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    ACTIONS(848), 1,
      anon_sym_COLON,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(769), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11587] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(779), 1,
      sym_expression,
    STATE(1047), 1,
      sym_expression_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11673] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(792), 1,
      sym_expression,
    STATE(1070), 1,
      sym_expression_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11759] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(850), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(852), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [11813] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(778), 1,
      sym_expression,
    STATE(1052), 1,
      sym_expression_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [11899] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(850), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(852), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [11953] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(856), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(854), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [12007] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(737), 1,
      sym_expression,
    STATE(1030), 1,
      sym_type,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [12093] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(856), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(854), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [12147] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(858), 1,
      anon_sym_elif,
    STATE(241), 1,
      aux_sym_if_statement_repeat1,
    STATE(332), 1,
      sym_elif_clause,
    ACTIONS(841), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(839), 31,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [12207] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(831), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(833), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [12261] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(737), 1,
      sym_expression,
    STATE(859), 1,
      sym_type,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [12347] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(737), 1,
      sym_expression,
    STATE(1059), 1,
      sym_type,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [12433] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(861), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [12487] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(737), 1,
      sym_expression,
    STATE(1055), 1,
      sym_type,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [12573] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(835), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(837), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [12627] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(737), 1,
      sym_expression,
    STATE(927), 1,
      sym_type,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [12713] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(788), 1,
      sym_expression,
    STATE(925), 1,
      sym_type,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [12799] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(861), 34,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [12853] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(732), 1,
      sym_expression,
    STATE(978), 1,
      sym_expression_list,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [12939] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(760), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13022] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(705), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13105] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(867), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(865), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [13158] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(750), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13241] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(739), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13324] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(854), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13407] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(677), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13490] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(851), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13573] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(867), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(865), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [13626] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(848), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13709] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(805), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13792] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(865), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13875] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(857), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [13958] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(712), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14041] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(680), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14124] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(871), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(869), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [14177] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(829), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14260] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(722), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14343] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(789), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14426] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(766), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14509] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(828), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14592] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(875), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(873), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [14645] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(827), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14728] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(675), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14811] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(852), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14894] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(850), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [14977] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(738), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15060] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(875), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(873), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [15113] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(756), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15196] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(871), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(869), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [15249] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(745), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15332] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(676), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15415] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(684), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15498] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(727), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15581] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(679), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15664] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(753), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15747] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(877), 1,
      sym_identifier,
    ACTIONS(881), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(760), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(635), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(879), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15832] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(661), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15915] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(728), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [15998] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(775), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16081] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(672), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16164] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(755), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16247] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(686), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16330] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(883), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(885), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [16383] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(741), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16466] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(772), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16549] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(661), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16632] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(681), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16715] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(718), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16798] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(683), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16881] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(687), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [16964] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(889), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [17017] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(711), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17100] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(821), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17183] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(685), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17266] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(767), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17349] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(798), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17432] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(891), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(893), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [17485] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(822), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17568] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(800), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17651] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(469), 1,
      sym_identifier,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(475), 1,
      anon_sym_not,
    ACTIONS(479), 1,
      anon_sym_lambda,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(487), 1,
      anon_sym_await,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(496), 1,
      sym_primary_expression,
    STATE(501), 1,
      sym_string,
    STATE(761), 1,
      sym_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(485), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(749), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17734] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(706), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [17817] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(887), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(889), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [17870] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(883), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(885), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [17923] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(849), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18006] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(782), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18089] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(453), 1,
      sym_identifier,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(461), 1,
      anon_sym_not,
    ACTIONS(465), 1,
      anon_sym_lambda,
    ACTIONS(467), 1,
      anon_sym_await,
    STATE(348), 1,
      sym_string,
    STATE(448), 1,
      sym_primary_expression,
    STATE(707), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(457), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18172] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(716), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18255] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(825), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18338] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(891), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(893), 33,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_except,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [18391] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(815), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18474] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(785), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18557] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(429), 1,
      sym_identifier,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(437), 1,
      anon_sym_not,
    ACTIONS(441), 1,
      anon_sym_lambda,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(449), 1,
      anon_sym_await,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(446), 1,
      sym_primary_expression,
    STATE(832), 1,
      sym_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(433), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(664), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18640] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(59), 1,
      anon_sym_not,
    ACTIONS(63), 1,
      anon_sym_lambda,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(238), 1,
      sym_identifier,
    ACTIONS(255), 1,
      anon_sym_await,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(487), 1,
      sym_primary_expression,
    STATE(492), 1,
      sym_string,
    STATE(713), 1,
      sym_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 3,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
    ACTIONS(71), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    STATE(709), 7,
      sym_named_expression,
      sym_not_operator,
      sym_boolean_operator,
      sym_comparison_operator,
      sym_lambda,
      sym_conditional_expression,
      sym_await,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [18723] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    STATE(360), 1,
      sym_else_clause,
    ACTIONS(897), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(895), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [18779] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(700), 1,
      anon_sym_finally,
    STATE(399), 1,
      sym_finally_clause,
    ACTIONS(899), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(901), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [18835] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    STATE(406), 1,
      sym_else_clause,
    ACTIONS(897), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(895), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [18891] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    STATE(401), 1,
      sym_else_clause,
    ACTIONS(905), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(903), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [18947] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(710), 1,
      anon_sym_finally,
    STATE(374), 1,
      sym_finally_clause,
    ACTIONS(909), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(907), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19003] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(911), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(913), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19055] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(915), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(917), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19107] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    STATE(372), 1,
      sym_else_clause,
    ACTIONS(919), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(921), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19163] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(925), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(923), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19215] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    STATE(376), 1,
      sym_else_clause,
    ACTIONS(929), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(927), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19271] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    STATE(419), 1,
      sym_else_clause,
    ACTIONS(919), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(921), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19327] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(911), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(913), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19379] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    STATE(412), 1,
      sym_else_clause,
    ACTIONS(933), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(931), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19435] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(925), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(923), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19487] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(700), 1,
      anon_sym_finally,
    STATE(367), 1,
      sym_finally_clause,
    ACTIONS(909), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(907), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19543] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    STATE(425), 1,
      sym_else_clause,
    ACTIONS(905), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(903), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19599] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    STATE(396), 1,
      sym_else_clause,
    ACTIONS(935), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(937), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19655] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(915), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(917), 32,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_elif,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19707] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    STATE(395), 1,
      sym_else_clause,
    ACTIONS(933), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(931), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19763] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(706), 1,
      anon_sym_else,
    STATE(426), 1,
      sym_else_clause,
    ACTIONS(935), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(937), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19819] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(696), 1,
      anon_sym_else,
    STATE(383), 1,
      sym_else_clause,
    ACTIONS(929), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(927), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19875] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(710), 1,
      anon_sym_finally,
    STATE(391), 1,
      sym_finally_clause,
    ACTIONS(899), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(901), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [19931] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(351), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(729), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [19986] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(941), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(939), 31,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20037] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(947), 1,
      sym__string_start,
    STATE(350), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(945), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(943), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [20092] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(350), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(952), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(950), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [20147] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(956), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(954), 31,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20198] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(956), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(954), 31,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20249] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(941), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(939), 31,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_finally,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20300] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(960), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(958), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20350] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(647), 1,
      sym_pattern,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(962), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [20430] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(704), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(702), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20480] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(964), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(966), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20530] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(909), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(907), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20580] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(970), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(968), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20630] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(974), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(972), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20680] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(976), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(978), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20730] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(980), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(982), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20780] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(986), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(984), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20830] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(901), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20880] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(990), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(988), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20930] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(992), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(994), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [20980] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(964), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(966), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21030] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(980), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(982), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21080] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(899), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(901), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21130] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(996), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(998), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21180] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1000), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1002), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21230] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(647), 1,
      sym_pattern,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(1004), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [21310] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(992), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(994), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21360] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1008), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1006), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21410] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1012), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1010), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21460] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1016), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1014), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21510] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1018), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1020), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21560] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1024), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1022), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21610] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1028), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1026), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21660] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1008), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1006), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21710] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1028), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1026), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21760] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1012), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1010), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21810] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1016), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1014), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21860] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1024), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1022), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21910] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1030), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1032), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [21960] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(986), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(984), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22010] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(960), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(958), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22060] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1036), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1034), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22110] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1038), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1040), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22160] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1044), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1042), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22210] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1046), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22260] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1048), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1046), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22310] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1038), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1040), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22360] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1052), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22410] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1054), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1056), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22460] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1058), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1060), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22510] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1064), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22560] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1044), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1042), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22610] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1036), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1034), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22660] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1068), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1066), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22710] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(704), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(702), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22760] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(976), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(978), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22810] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(996), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(998), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22860] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1062), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1064), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22910] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(970), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(968), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [22960] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1072), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1070), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23010] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1076), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1074), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23060] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1080), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1078), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23110] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1058), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1060), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23160] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1084), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1082), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23210] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1050), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1052), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23260] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1080), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1078), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23310] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(692), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(694), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23360] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(974), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(972), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23410] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(692), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(694), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23460] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(909), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(907), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23510] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1030), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1032), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23560] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1000), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1002), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23610] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1018), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1020), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23660] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1072), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1070), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23710] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1084), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1082), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23760] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1076), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1074), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23810] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    ACTIONS(1086), 1,
      anon_sym_RPAREN,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(824), 1,
      sym_pattern,
    STATE(1014), 1,
      sym__patterns,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [23892] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1068), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1066), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23942] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1054), 12,
      sym__dedent,
      sym__string_start,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(1056), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [23992] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(990), 12,
      sym__string_start,
      ts_builtin_sym_end,
      anon_sym_LPAREN,
      anon_sym_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
      sym_ellipsis,
      anon_sym_LBRACE,
      sym_float,
    ACTIONS(988), 30,
      anon_sym_import,
      anon_sym_from,
      anon_sym_print,
      anon_sym_assert,
      anon_sym_return,
      anon_sym_del,
      anon_sym_raise,
      anon_sym_pass,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_while,
      anon_sym_try,
      anon_sym_with,
      anon_sym_def,
      anon_sym_global,
      anon_sym_nonlocal,
      anon_sym_exec,
      anon_sym_class,
      anon_sym_not,
      anon_sym_lambda,
      anon_sym_yield,
      sym_integer,
      sym_identifier,
      anon_sym_await,
      sym_true,
      sym_false,
      sym_none,
  [24042] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(902), 1,
      sym_pattern,
    STATE(1018), 1,
      sym_pattern_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [24121] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(876), 1,
      sym_pattern,
    STATE(1087), 1,
      sym_pattern_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [24200] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(877), 1,
      sym_pattern,
    STATE(1085), 1,
      sym_pattern_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [24279] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1090), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1088), 35,
      sym__string_start,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24328] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1094), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1092), 35,
      sym__string_start,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24377] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(947), 1,
      sym_pattern,
    STATE(1056), 1,
      sym_pattern_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [24456] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(875), 1,
      sym_pattern,
    STATE(1063), 1,
      sym_pattern_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [24535] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(879), 1,
      sym_pattern,
    STATE(1082), 1,
      sym_pattern_list,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [24614] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    STATE(931), 1,
      sym_pattern,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [24690] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1098), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1096), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24738] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1102), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1100), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24786] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1106), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1104), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24834] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1110), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1108), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24882] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1114), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1112), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24930] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1118), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1116), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [24978] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(729), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25026] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1122), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1120), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25074] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(15), 1,
      anon_sym_STAR,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(540), 1,
      sym_identifier,
    ACTIONS(542), 1,
      anon_sym_LPAREN,
    ACTIONS(552), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(647), 1,
      sym_pattern,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(527), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(646), 3,
      sym_tuple_pattern,
      sym_list_pattern,
      sym_list_splat_pattern,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(546), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [25150] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1142), 1,
      anon_sym_EQ,
    ACTIONS(1144), 1,
      anon_sym_not,
    ACTIONS(1148), 1,
      anon_sym_PIPE,
    ACTIONS(1150), 1,
      anon_sym_AMP,
    ACTIONS(1152), 1,
      anon_sym_CARET,
    ACTIONS(1156), 1,
      anon_sym_is,
    STATE(640), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1132), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1154), 2,
      anon_sym_LT,
      anon_sym_GT,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1134), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1128), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25232] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1160), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1158), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25280] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1142), 1,
      anon_sym_as,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1172), 1,
      anon_sym_not,
    ACTIONS(1176), 1,
      anon_sym_PIPE,
    ACTIONS(1178), 1,
      anon_sym_AMP,
    ACTIONS(1180), 1,
      anon_sym_CARET,
    ACTIONS(1184), 1,
      anon_sym_is,
    STATE(638), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1164), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1174), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1182), 2,
      anon_sym_LT,
      anon_sym_GT,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1166), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1128), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
  [25362] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1188), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1186), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25410] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1192), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1190), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25458] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1196), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1194), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25506] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1200), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1198), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25554] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1204), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1202), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25602] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1208), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1206), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25650] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1212), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1210), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25698] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1216), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1214), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25746] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1220), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1218), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25794] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1224), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1222), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25842] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1228), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1226), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25890] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1232), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1230), 34,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [25938] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1148), 1,
      anon_sym_PIPE,
    ACTIONS(1150), 1,
      anon_sym_AMP,
    ACTIONS(1152), 1,
      anon_sym_CARET,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1132), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1236), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1234), 18,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26009] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1164), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1174), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_as,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 21,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [26074] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1176), 1,
      anon_sym_PIPE,
    ACTIONS(1178), 1,
      anon_sym_AMP,
    ACTIONS(1180), 1,
      anon_sym_CARET,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1164), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1174), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1244), 3,
      anon_sym_as,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1242), 18,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [26145] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1248), 5,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1246), 28,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [26202] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1150), 1,
      anon_sym_AMP,
    ACTIONS(1152), 1,
      anon_sym_CARET,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1132), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26271] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1152), 1,
      anon_sym_CARET,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1132), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 20,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26338] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1132), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 21,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26403] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1174), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_as,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 23,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [26466] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 25,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26527] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 28,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26584] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1148), 1,
      anon_sym_PIPE,
    ACTIONS(1150), 1,
      anon_sym_AMP,
    ACTIONS(1152), 1,
      anon_sym_CARET,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1132), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1244), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1242), 18,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26655] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 23,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26718] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 5,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 28,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [26775] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1248), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1246), 28,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26832] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 28,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [26889] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 5,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 28,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [26946] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1176), 1,
      anon_sym_PIPE,
    ACTIONS(1178), 1,
      anon_sym_AMP,
    ACTIONS(1180), 1,
      anon_sym_CARET,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1164), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1174), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1236), 3,
      anon_sym_as,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1234), 18,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27017] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_as,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 25,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27078] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1180), 1,
      anon_sym_CARET,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1164), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1174), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_as,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 20,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27145] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1168), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1178), 1,
      anon_sym_AMP,
    ACTIONS(1180), 1,
      anon_sym_CARET,
    ACTIONS(1162), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1164), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1174), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1170), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1240), 3,
      anon_sym_as,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27214] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 1,
      anon_sym_COLON_EQ,
    ACTIONS(729), 6,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_COLON,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 31,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27262] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(822), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(817), 33,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [27308] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(791), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 33,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [27354] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1252), 1,
      anon_sym_COLON_EQ,
    ACTIONS(729), 6,
      anon_sym_STAR,
      anon_sym_COLON,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 31,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_else,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [27402] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(1254), 1,
      sym_identifier,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(538), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(1256), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(1258), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [27472] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(240), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 33,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
      sym_type_conversion,
  [27518] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1142), 1,
      anon_sym_EQ,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1276), 1,
      anon_sym_not,
    ACTIONS(1280), 1,
      anon_sym_PIPE,
    ACTIONS(1282), 1,
      anon_sym_AMP,
    ACTIONS(1284), 1,
      anon_sym_CARET,
    ACTIONS(1288), 1,
      anon_sym_is,
    STATE(644), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1266), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1286), 2,
      anon_sym_LT,
      anon_sym_GT,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1268), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1128), 7,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_and,
      anon_sym_or,
      sym__semicolon,
  [27597] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(240), 5,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 32,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27642] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(822), 5,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(817), 32,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27687] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(791), 5,
      anon_sym_as,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 32,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      anon_sym_RBRACE,
  [27732] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(75), 1,
      sym__string_start,
    STATE(493), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(952), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(950), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [27781] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(75), 1,
      sym__string_start,
    STATE(491), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(729), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [27830] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1290), 1,
      sym__string_start,
    STATE(493), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(945), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(943), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [27879] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1282), 1,
      anon_sym_AMP,
    ACTIONS(1284), 1,
      anon_sym_CARET,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1266), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 16,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [27945] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 22,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28003] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1309), 1,
      anon_sym_not,
    ACTIONS(1313), 1,
      anon_sym_PIPE,
    ACTIONS(1315), 1,
      anon_sym_AMP,
    ACTIONS(1317), 1,
      anon_sym_CARET,
    ACTIONS(1321), 1,
      anon_sym_is,
    STATE(653), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1311), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1319), 2,
      anon_sym_LT,
      anon_sym_GT,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1301), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1128), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_and,
      anon_sym_or,
  [28079] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1248), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1246), 25,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28133] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1266), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 18,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28195] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    ACTIONS(1323), 1,
      anon_sym_not,
    STATE(501), 1,
      sym_string,
    STATE(561), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [28259] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    ACTIONS(1325), 1,
      anon_sym_not,
    STATE(492), 1,
      sym_string,
    STATE(511), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [28323] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(508), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [28371] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1280), 1,
      anon_sym_PIPE,
    ACTIONS(1282), 1,
      anon_sym_AMP,
    ACTIONS(1284), 1,
      anon_sym_CARET,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1266), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1236), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1234), 15,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28439] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1284), 1,
      anon_sym_CARET,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1266), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 17,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28503] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    ACTIONS(1327), 1,
      anon_sym_not,
    STATE(348), 1,
      sym_string,
    STATE(463), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [28567] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1329), 1,
      sym__string_start,
    STATE(505), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(945), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(943), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [28615] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 25,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28669] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 25,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28723] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(505), 2,
      sym_string,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(952), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(950), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [28771] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 20,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28831] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(1332), 1,
      anon_sym_not,
    STATE(348), 1,
      sym_string,
    STATE(471), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [28895] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1280), 1,
      anon_sym_PIPE,
    ACTIONS(1282), 1,
      anon_sym_AMP,
    ACTIONS(1284), 1,
      anon_sym_CARET,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1266), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1244), 3,
      anon_sym_EQ,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1242), 15,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [28963] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(506), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29024] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 25,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [29077] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(477), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29138] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(643), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29199] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(497), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29260] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1094), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1092), 30,
      sym__newline,
      sym__string_start,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [29303] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(531), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29364] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(462), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29425] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(479), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29486] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(480), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29547] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(478), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29608] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(473), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29669] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(463), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29730] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(468), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29791] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(476), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [29852] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(724), 3,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
    ACTIONS(729), 13,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
    ACTIONS(735), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [29897] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(242), 3,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
    ACTIONS(240), 13,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
    ACTIONS(1334), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [29942] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(817), 3,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
    ACTIONS(822), 13,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
    ACTIONS(824), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [29987] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(786), 3,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
    ACTIONS(791), 13,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
    ACTIONS(793), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30032] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1248), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1246), 25,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [30085] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(502), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30146] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(475), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30207] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(472), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30268] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(471), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30329] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(470), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30390] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1336), 1,
      anon_sym_EQ,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [30437] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(724), 3,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
    ACTIONS(729), 13,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
    ACTIONS(1338), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30482] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(469), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30543] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(731), 1,
      anon_sym_COLON_EQ,
    ACTIONS(729), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [30588] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(465), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30649] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(466), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30710] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(467), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30771] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(509), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30832] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(461), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30893] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    STATE(348), 1,
      sym_string,
    STATE(474), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [30954] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(1340), 1,
      sym_identifier,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(648), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(1342), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31019] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(431), 1,
      anon_sym_LPAREN,
    ACTIONS(435), 1,
      anon_sym_LBRACK,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(1254), 1,
      sym_identifier,
    STATE(348), 1,
      sym_string,
    STATE(652), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    STATE(538), 2,
      sym_attribute,
      sym_subscript,
    ACTIONS(439), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(447), 4,
      sym_integer,
      sym_true,
      sym_false,
      sym_none,
    ACTIONS(1258), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    STATE(443), 13,
      sym_binary_operator,
      sym_unary_operator,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31084] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(513), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31145] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(242), 3,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
    ACTIONS(240), 13,
      anon_sym_STAR,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
    ACTIONS(251), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31190] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(560), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31251] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(511), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31312] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(561), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31373] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(562), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31434] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(564), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31495] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(565), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31556] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(566), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31617] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(568), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31678] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(507), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31739] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1311), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 20,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [31798] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1313), 1,
      anon_sym_PIPE,
    ACTIONS(1315), 1,
      anon_sym_AMP,
    ACTIONS(1317), 1,
      anon_sym_CARET,
    ACTIONS(1244), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1311), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1242), 15,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [31865] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1240), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1238), 25,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_AT,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [31918] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(471), 1,
      anon_sym_LPAREN,
    ACTIONS(473), 1,
      anon_sym_LBRACK,
    ACTIONS(483), 1,
      anon_sym_LBRACE,
    ACTIONS(489), 1,
      sym__string_start,
    STATE(501), 1,
      sym_string,
    STATE(569), 1,
      sym_primary_expression,
    ACTIONS(481), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(477), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(433), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(485), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(628), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [31979] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 22,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32036] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1315), 1,
      anon_sym_AMP,
    ACTIONS(1317), 1,
      anon_sym_CARET,
    ACTIONS(1240), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1311), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 16,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32101] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1317), 1,
      anon_sym_CARET,
    ACTIONS(1240), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1311), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 17,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32164] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(495), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [32225] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1240), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1311), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1238), 18,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32286] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1293), 1,
      anon_sym_DOT,
    ACTIONS(1295), 1,
      anon_sym_LPAREN,
    ACTIONS(1303), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1307), 1,
      anon_sym_LBRACK,
    ACTIONS(1313), 1,
      anon_sym_PIPE,
    ACTIONS(1315), 1,
      anon_sym_AMP,
    ACTIONS(1317), 1,
      anon_sym_CARET,
    ACTIONS(1236), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1297), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1299), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1311), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(611), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1305), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
    ACTIONS(1234), 15,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32353] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(494), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [32414] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(445), 1,
      anon_sym_LBRACE,
    ACTIONS(451), 1,
      sym__string_start,
    ACTIONS(455), 1,
      anon_sym_LPAREN,
    ACTIONS(459), 1,
      anon_sym_LBRACK,
    STATE(348), 1,
      sym_string,
    STATE(464), 1,
      sym_primary_expression,
    ACTIONS(443), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(463), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(457), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(447), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(443), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [32475] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1090), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1088), 30,
      sym__newline,
      sym__string_start,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [32518] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 1,
      anon_sym_COLON_EQ,
    ACTIONS(726), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 27,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32565] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(498), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [32626] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(69), 1,
      anon_sym_LBRACE,
    ACTIONS(75), 1,
      sym__string_start,
    ACTIONS(257), 1,
      anon_sym_LPAREN,
    ACTIONS(259), 1,
      anon_sym_LBRACK,
    STATE(492), 1,
      sym_string,
    STATE(503), 1,
      sym_primary_expression,
    ACTIONS(67), 2,
      sym_ellipsis,
      sym_float,
    ACTIONS(61), 3,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(247), 4,
      anon_sym_print,
      anon_sym_async,
      anon_sym_exec,
      anon_sym_await,
    ACTIONS(71), 5,
      sym_integer,
      sym_identifier,
      sym_true,
      sym_false,
      sym_none,
    STATE(605), 15,
      sym_binary_operator,
      sym_unary_operator,
      sym_attribute,
      sym_subscript,
      sym_call,
      sym_list,
      sym_set,
      sym_tuple,
      sym_dictionary,
      sym_list_comprehension,
      sym_dictionary_comprehension,
      sym_set_comprehension,
      sym_generator_expression,
      sym_parenthesized_expression,
      sym_concatenated_string,
  [32687] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1192), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1190), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [32729] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1102), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1100), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [32771] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1122), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1120), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [32813] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(726), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 27,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32857] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1110), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1108), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [32899] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(822), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(817), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [32941] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(819), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(822), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(817), 27,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [32985] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(791), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33027] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1114), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1112), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33069] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1160), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1158), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33111] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1228), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1226), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33153] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(788), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(791), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(786), 27,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [33197] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1344), 1,
      anon_sym_COLON_EQ,
    ACTIONS(729), 5,
      anon_sym_STAR,
      anon_sym_COLON,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 28,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [33241] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1200), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1198), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33283] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1118), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1116), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33325] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(240), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33367] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(491), 1,
      anon_sym_EQ,
    ACTIONS(240), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [33411] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1094), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1092), 30,
      sym__string_start,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [33453] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(244), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(240), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 27,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [33497] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1188), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1186), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33539] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1216), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1214), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33581] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1212), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1210), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33623] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1208), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1206), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33665] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1232), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1230), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33707] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1090), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1088), 30,
      sym__string_start,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [33749] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1220), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1218), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33791] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1098), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1096), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33833] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1204), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1202), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33875] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1196), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1194), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33917] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(729), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [33959] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1224), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1222), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [34001] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1106), 5,
      anon_sym_STAR,
      anon_sym_EQ,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1104), 29,
      sym__newline,
      anon_sym_DOT,
      anon_sym_from,
      anon_sym_LPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
      sym__semicolon,
  [34043] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1216), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1214), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34084] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1160), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1158), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34125] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1200), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1198), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34166] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1192), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1190), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34207] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1196), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1194), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34248] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1252), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1336), 1,
      anon_sym_EQ,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 27,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34293] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(568), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(566), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34334] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(572), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(570), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34375] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1110), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1108), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34416] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1106), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1104), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34457] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1228), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1226), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34498] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(606), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(604), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34539] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(592), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(590), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34580] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1188), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1186), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34621] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1224), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1222), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34662] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1252), 1,
      anon_sym_COLON_EQ,
    ACTIONS(1346), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 25,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34707] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1204), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1202), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34748] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1220), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1218), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34789] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1212), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1210), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34830] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1208), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1206), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34871] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34912] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1232), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1230), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34953] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1114), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1112), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [34994] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1102), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1100), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [35035] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1098), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1096), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [35076] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1122), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1120), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [35117] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1118), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1116), 29,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [35158] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1346), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(729), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(724), 25,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [35200] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(493), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(240), 4,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(242), 25,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_if,
      anon_sym_in,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_not,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
      anon_sym_is,
  [35242] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1244), 1,
      anon_sym_as,
    ACTIONS(1352), 1,
      anon_sym_not,
    ACTIONS(1358), 1,
      anon_sym_is,
    STATE(637), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1355), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1349), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1242), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
  [35282] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1172), 1,
      anon_sym_not,
    ACTIONS(1184), 1,
      anon_sym_is,
    ACTIONS(1363), 1,
      anon_sym_as,
    STATE(637), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1182), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1166), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1361), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
  [35322] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1244), 1,
      anon_sym_EQ,
    ACTIONS(1368), 1,
      anon_sym_not,
    ACTIONS(1374), 1,
      anon_sym_is,
    STATE(639), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1371), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1365), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1242), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
      sym_type_conversion,
  [35362] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1144), 1,
      anon_sym_not,
    ACTIONS(1156), 1,
      anon_sym_is,
    ACTIONS(1363), 1,
      anon_sym_EQ,
    STATE(639), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1154), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1134), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1361), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
      sym_type_conversion,
  [35402] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1379), 1,
      anon_sym_COMMA,
    STATE(641), 1,
      aux_sym__patterns_repeat1,
    ACTIONS(1377), 18,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35432] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1244), 1,
      anon_sym_EQ,
    ACTIONS(1385), 1,
      anon_sym_not,
    ACTIONS(1391), 1,
      anon_sym_is,
    STATE(642), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1388), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1382), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1242), 7,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_and,
      anon_sym_or,
      sym__semicolon,
  [35469] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1260), 1,
      anon_sym_DOT,
    ACTIONS(1262), 1,
      anon_sym_LPAREN,
    ACTIONS(1270), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1274), 1,
      anon_sym_LBRACK,
    ACTIONS(1280), 1,
      anon_sym_PIPE,
    ACTIONS(1282), 1,
      anon_sym_AMP,
    ACTIONS(1284), 1,
      anon_sym_CARET,
    ACTIONS(1394), 1,
      sym__newline,
    ACTIONS(1264), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1266), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1278), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(576), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1272), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
  [35518] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1276), 1,
      anon_sym_not,
    ACTIONS(1288), 1,
      anon_sym_is,
    ACTIONS(1363), 1,
      anon_sym_EQ,
    STATE(642), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1286), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1268), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1361), 7,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_and,
      anon_sym_or,
      sym__semicolon,
  [35555] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(240), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1396), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
    ACTIONS(242), 14,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
  [35584] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(735), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35609] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1377), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35634] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(729), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1398), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
    ACTIONS(724), 14,
      anon_sym_DOT,
      anon_sym_LPAREN,
      anon_sym_GT_GT,
      anon_sym_STAR_STAR,
      anon_sym_AT,
      anon_sym_LBRACK,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_CARET,
      anon_sym_LT_LT,
  [35663] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1400), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35688] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1402), 19,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35713] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1404), 1,
      anon_sym_COMMA,
    STATE(641), 1,
      aux_sym__patterns_repeat1,
    ACTIONS(548), 16,
      anon_sym_COLON,
      anon_sym_in,
      anon_sym_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35741] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1124), 1,
      anon_sym_DOT,
    ACTIONS(1126), 1,
      anon_sym_LPAREN,
    ACTIONS(1136), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1140), 1,
      anon_sym_LBRACK,
    ACTIONS(1148), 1,
      anon_sym_PIPE,
    ACTIONS(1150), 1,
      anon_sym_AMP,
    ACTIONS(1152), 1,
      anon_sym_CARET,
    ACTIONS(1130), 2,
      anon_sym_STAR,
      anon_sym_SLASH,
    ACTIONS(1132), 2,
      anon_sym_GT_GT,
      anon_sym_LT_LT,
    ACTIONS(1146), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    STATE(450), 2,
      sym_argument_list,
      sym_generator_expression,
    ACTIONS(1138), 3,
      anon_sym_AT,
      anon_sym_PERCENT,
      anon_sym_SLASH_SLASH,
  [35787] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1309), 1,
      anon_sym_not,
    ACTIONS(1321), 1,
      anon_sym_is,
    STATE(654), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1319), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1301), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1361), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_and,
      anon_sym_or,
  [35821] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1409), 1,
      anon_sym_not,
    ACTIONS(1415), 1,
      anon_sym_is,
    STATE(654), 1,
      aux_sym_comparison_operator_repeat1,
    ACTIONS(1412), 2,
      anon_sym_LT,
      anon_sym_GT,
    ACTIONS(1406), 6,
      anon_sym_in,
      anon_sym_LT_EQ,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_GT,
    ACTIONS(1242), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_and,
      anon_sym_or,
  [35855] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1418), 1,
      anon_sym_COMMA,
    ACTIONS(1420), 1,
      anon_sym_COLON,
    ACTIONS(1422), 1,
      anon_sym_EQ,
    STATE(651), 1,
      aux_sym__patterns_repeat1,
    ACTIONS(1424), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35886] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1420), 1,
      anon_sym_COLON,
    ACTIONS(1422), 1,
      anon_sym_EQ,
    ACTIONS(1424), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_AT_EQ,
      anon_sym_SLASH_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [35911] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1432), 1,
      anon_sym_COLON,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    STATE(949), 1,
      sym_parameter,
    STATE(1033), 1,
      sym_lambda_parameters,
    STATE(1078), 1,
      sym__parameters,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [35949] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1436), 1,
      anon_sym_COLON,
    STATE(949), 1,
      sym_parameter,
    STATE(1028), 1,
      sym_lambda_parameters,
    STATE(1078), 1,
      sym__parameters,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [35987] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1438), 1,
      anon_sym_COLON,
    STATE(949), 1,
      sym_parameter,
    STATE(1062), 1,
      sym_lambda_parameters,
    STATE(1078), 1,
      sym__parameters,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36025] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1440), 1,
      anon_sym_COLON,
    STATE(949), 1,
      sym_parameter,
    STATE(1061), 1,
      sym_lambda_parameters,
    STATE(1078), 1,
      sym__parameters,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36063] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1444), 1,
      anon_sym_as,
    ACTIONS(1442), 13,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36085] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1446), 1,
      anon_sym_COLON,
    STATE(949), 1,
      sym_parameter,
    STATE(1077), 1,
      sym_lambda_parameters,
    STATE(1078), 1,
      sym__parameters,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36123] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1450), 1,
      anon_sym_as,
    ACTIONS(1448), 13,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36145] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1142), 1,
      anon_sym_as,
    ACTIONS(1128), 13,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_and,
      anon_sym_or,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36167] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1452), 1,
      sym_identifier,
    ACTIONS(1454), 1,
      anon_sym_RPAREN,
    STATE(888), 1,
      sym_parameter,
    STATE(1044), 1,
      sym__parameters,
    STATE(938), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36202] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1456), 1,
      anon_sym_COLON,
    STATE(930), 1,
      sym_parameter,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36234] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1458), 1,
      anon_sym_COLON,
    STATE(930), 1,
      sym_parameter,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36266] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1452), 1,
      sym_identifier,
    ACTIONS(1458), 1,
      anon_sym_RPAREN,
    STATE(930), 1,
      sym_parameter,
    STATE(938), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36298] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1452), 1,
      sym_identifier,
    ACTIONS(1456), 1,
      anon_sym_RPAREN,
    STATE(930), 1,
      sym_parameter,
    STATE(938), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36330] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1460), 1,
      anon_sym_RPAREN,
    ACTIONS(1462), 1,
      anon_sym_COMMA,
    ACTIONS(1465), 1,
      anon_sym_as,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(998), 1,
      sym__comprehension_clauses,
  [36367] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1426), 1,
      sym_identifier,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    STATE(930), 1,
      sym_parameter,
    STATE(960), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36396] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1442), 10,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_or,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36415] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1481), 1,
      anon_sym_COLON,
    ACTIONS(1483), 1,
      anon_sym_RBRACE,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(1008), 1,
      sym__comprehension_clauses,
  [36452] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1428), 1,
      anon_sym_LPAREN,
    ACTIONS(1430), 1,
      anon_sym_STAR,
    ACTIONS(1434), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1452), 1,
      sym_identifier,
    STATE(930), 1,
      sym_parameter,
    STATE(938), 2,
      sym_list_splat_pattern,
      sym_dictionary_splat_pattern,
    STATE(948), 4,
      sym_tuple_pattern,
      sym_default_parameter,
      sym_typed_default_parameter,
      sym_typed_parameter,
  [36481] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1485), 8,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36504] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1493), 1,
      anon_sym_as,
    ACTIONS(1491), 8,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [36527] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1497), 1,
      anon_sym_as,
    ACTIONS(1495), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [36552] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1481), 1,
      anon_sym_COLON,
    ACTIONS(1483), 1,
      anon_sym_RBRACE,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(1001), 1,
      sym__comprehension_clauses,
  [36589] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1499), 1,
      anon_sym_as,
    ACTIONS(1485), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [36614] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1495), 8,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36637] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1503), 1,
      anon_sym_as,
    ACTIONS(1501), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [36662] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1481), 1,
      anon_sym_COLON,
    ACTIONS(1483), 1,
      anon_sym_RBRACE,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(1002), 1,
      sym__comprehension_clauses,
  [36699] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1505), 9,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36720] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1444), 1,
      anon_sym_as,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1442), 9,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_or,
      anon_sym_RBRACE,
  [36741] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1491), 9,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36762] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1507), 1,
      anon_sym_as,
    ACTIONS(1505), 8,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [36785] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1501), 8,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_else,
      anon_sym_RBRACK,
      anon_sym_EQ,
      anon_sym_RBRACE,
      sym_type_conversion,
  [36808] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1460), 1,
      anon_sym_RPAREN,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(998), 1,
      sym__comprehension_clauses,
  [36842] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1483), 1,
      anon_sym_RBRACK,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(999), 1,
      sym__comprehension_clauses,
  [36876] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1483), 1,
      anon_sym_RBRACK,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(1015), 1,
      sym__comprehension_clauses,
  [36910] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1509), 1,
      anon_sym_RPAREN,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(1019), 1,
      sym__comprehension_clauses,
  [36944] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1483), 1,
      anon_sym_RBRACK,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(1010), 1,
      sym__comprehension_clauses,
  [36978] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1511), 1,
      anon_sym_RPAREN,
    ACTIONS(1513), 1,
      anon_sym_COMMA,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(905), 1,
      aux_sym_argument_list_repeat1,
    STATE(1046), 1,
      sym__comprehension_clauses,
  [37012] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1515), 1,
      anon_sym_RPAREN,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    STATE(1046), 1,
      sym__comprehension_clauses,
  [37046] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1517), 1,
      anon_sym_RPAREN,
    ACTIONS(1519), 1,
      anon_sym_COMMA,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(903), 1,
      aux_sym_argument_list_repeat1,
    STATE(998), 1,
      sym__comprehension_clauses,
  [37080] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1521), 1,
      anon_sym_RPAREN,
    ACTIONS(1523), 1,
      anon_sym_COMMA,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(937), 1,
      aux_sym_argument_list_repeat1,
    STATE(1019), 1,
      sym__comprehension_clauses,
  [37114] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1527), 1,
      anon_sym_if,
    ACTIONS(1525), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(701), 3,
      sym_for_in_clause,
      sym_if_clause,
      aux_sym__comprehension_clauses_repeat1,
  [37137] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1529), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37156] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1529), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37175] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1529), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37194] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1533), 1,
      anon_sym_if,
    ACTIONS(1536), 1,
      anon_sym_async,
    ACTIONS(1539), 1,
      anon_sym_for,
    ACTIONS(1531), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(701), 3,
      sym_for_in_clause,
      sym_if_clause,
      aux_sym__comprehension_clauses_repeat1,
  [37217] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1527), 1,
      anon_sym_if,
    ACTIONS(1542), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
    STATE(697), 3,
      sym_for_in_clause,
      sym_if_clause,
      aux_sym__comprehension_clauses_repeat1,
  [37240] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1546), 1,
      anon_sym_COMMA,
    STATE(714), 1,
      aux_sym_for_in_clause_repeat1,
    ACTIONS(1544), 6,
      anon_sym_RPAREN,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37258] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1550), 1,
      anon_sym_COMMA,
    STATE(703), 1,
      aux_sym_for_in_clause_repeat1,
    ACTIONS(1548), 6,
      anon_sym_RPAREN,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37276] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1442), 8,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_EQ,
      anon_sym_and,
      anon_sym_or,
      sym__semicolon,
  [37290] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1442), 7,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_EQ,
      anon_sym_or,
      sym__semicolon,
  [37306] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1554), 6,
      anon_sym_RPAREN,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37324] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1558), 1,
      anon_sym_COMMA,
    STATE(773), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1556), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37348] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1128), 8,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_EQ,
      anon_sym_and,
      anon_sym_or,
      sym__semicolon,
  [37362] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1448), 8,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_EQ,
      anon_sym_and,
      anon_sym_or,
      sym__semicolon,
  [37376] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1485), 5,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_EQ,
      sym__semicolon,
  [37396] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1491), 6,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_EQ,
      sym__semicolon,
  [37414] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1505), 6,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_EQ,
      sym__semicolon,
  [37432] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1566), 1,
      anon_sym_COMMA,
    STATE(714), 1,
      aux_sym_for_in_clause_repeat1,
    ACTIONS(1564), 6,
      anon_sym_RPAREN,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37450] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1571), 1,
      anon_sym_COMMA,
    STATE(714), 1,
      aux_sym_for_in_clause_repeat1,
    ACTIONS(1569), 6,
      anon_sym_RPAREN,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37468] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1501), 5,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_EQ,
      sym__semicolon,
  [37488] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(55), 1,
      anon_sym_AT,
    ACTIONS(1573), 1,
      anon_sym_async,
    ACTIONS(1575), 1,
      anon_sym_def,
    ACTIONS(1577), 1,
      anon_sym_class,
    STATE(380), 2,
      sym_function_definition,
      sym_class_definition,
    STATE(791), 2,
      sym_decorator,
      aux_sym_decorated_definition_repeat1,
  [37512] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1579), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37532] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1581), 1,
      anon_sym_from,
    ACTIONS(1583), 1,
      anon_sym_COMMA,
    STATE(796), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1585), 2,
      sym__newline,
      sym__semicolon,
  [37558] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(55), 1,
      anon_sym_AT,
    ACTIONS(1587), 1,
      anon_sym_async,
    ACTIONS(1589), 1,
      anon_sym_def,
    ACTIONS(1591), 1,
      anon_sym_class,
    STATE(382), 2,
      sym_function_definition,
      sym_class_definition,
    STATE(791), 2,
      sym_decorator,
      aux_sym_decorated_definition_repeat1,
  [37582] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1595), 1,
      anon_sym_COMMA,
    STATE(715), 1,
      aux_sym_for_in_clause_repeat1,
    ACTIONS(1593), 6,
      anon_sym_RPAREN,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37600] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1495), 5,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      anon_sym_EQ,
      sym__semicolon,
  [37620] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1603), 1,
      sym__string_content,
    ACTIONS(1605), 1,
      sym__string_end,
    ACTIONS(1599), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(735), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [37641] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1607), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [37660] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1609), 1,
      anon_sym_COMMA,
    STATE(835), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1611), 2,
      sym__newline,
      sym__semicolon,
  [37683] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1613), 1,
      sym_identifier,
    ACTIONS(1615), 1,
      anon_sym_LPAREN,
    ACTIONS(1617), 1,
      anon_sym_STAR,
    STATE(820), 1,
      sym_dotted_name,
    STATE(847), 1,
      sym_aliased_import,
    STATE(967), 2,
      sym__import_list,
      sym_wildcard_import,
  [37706] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1442), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_and,
      anon_sym_or,
  [37719] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1619), 1,
      anon_sym_COMMA,
    STATE(826), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1621), 2,
      sym__newline,
      sym__semicolon,
  [37742] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1583), 1,
      anon_sym_COMMA,
    STATE(796), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1623), 2,
      sym__newline,
      sym__semicolon,
  [37765] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1625), 1,
      anon_sym_COMMA,
    ACTIONS(1627), 1,
      anon_sym_RBRACE,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(910), 1,
      aux_sym_dictionary_repeat1,
    STATE(1007), 1,
      sym__comprehension_clauses,
  [37790] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1583), 1,
      anon_sym_COMMA,
    STATE(796), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1629), 2,
      sym__newline,
      sym__semicolon,
  [37813] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1583), 1,
      anon_sym_COMMA,
    STATE(796), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1631), 2,
      sym__newline,
      sym__semicolon,
  [37836] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1635), 1,
      sym__string_content,
    ACTIONS(1637), 1,
      sym__string_end,
    ACTIONS(1633), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(746), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [37857] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1603), 1,
      sym__string_content,
    ACTIONS(1639), 1,
      sym__string_end,
    ACTIONS(1599), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(735), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [37878] = 6,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1641), 1,
      anon_sym_LBRACE,
    ACTIONS(1647), 1,
      sym__string_content,
    ACTIONS(1650), 1,
      sym__string_end,
    ACTIONS(1644), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(735), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [37899] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1603), 1,
      sym__string_content,
    ACTIONS(1652), 1,
      sym__string_end,
    ACTIONS(1599), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(735), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [37920] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1654), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
      anon_sym_EQ,
  [37939] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1656), 1,
      anon_sym_if,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1501), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_COLON,
  [37958] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1619), 1,
      anon_sym_COMMA,
    STATE(862), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1662), 2,
      sym__newline,
      sym__semicolon,
  [37981] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1656), 1,
      anon_sym_if,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1666), 1,
      anon_sym_as,
    ACTIONS(1664), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [38002] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1668), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [38021] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1670), 1,
      anon_sym_COMMA,
    STATE(863), 1,
      aux_sym_print_statement_repeat1,
    ACTIONS(1672), 2,
      sym__newline,
      sym__semicolon,
  [38044] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1674), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [38057] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1678), 1,
      sym__string_content,
    ACTIONS(1680), 1,
      sym__string_end,
    ACTIONS(1676), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(723), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [38078] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1505), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
  [38095] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1603), 1,
      sym__string_content,
    ACTIONS(1682), 1,
      sym__string_end,
    ACTIONS(1599), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(735), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [38116] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1564), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [38129] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1684), 1,
      anon_sym_COMMA,
    ACTIONS(1686), 1,
      anon_sym_RBRACE,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(916), 1,
      aux_sym_dictionary_repeat1,
    STATE(1003), 1,
      sym__comprehension_clauses,
  [38154] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1128), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_and,
      anon_sym_or,
  [38167] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1467), 1,
      anon_sym_if,
    ACTIONS(1473), 1,
      anon_sym_and,
    ACTIONS(1475), 1,
      anon_sym_or,
    ACTIONS(1688), 4,
      anon_sym_COMMA,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACE,
  [38186] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1583), 1,
      anon_sym_COMMA,
    STATE(796), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1556), 2,
      sym__newline,
      sym__semicolon,
  [38209] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1690), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_if,
      anon_sym_async,
      anon_sym_for,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [38222] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1491), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
  [38239] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1469), 1,
      anon_sym_async,
    ACTIONS(1471), 1,
      anon_sym_for,
    ACTIONS(1692), 1,
      anon_sym_COMMA,
    ACTIONS(1694), 1,
      anon_sym_RBRACE,
    STATE(702), 1,
      sym_for_in_clause,
    STATE(915), 1,
      aux_sym_dictionary_repeat1,
    STATE(996), 1,
      sym__comprehension_clauses,
  [38264] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1442), 6,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_or,
  [38279] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1696), 1,
      anon_sym_COLON,
    ACTIONS(1698), 1,
      anon_sym_RBRACE,
    ACTIONS(1700), 1,
      sym_type_conversion,
    STATE(1086), 1,
      sym_format_specifier,
  [38304] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1448), 7,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_if,
      anon_sym_COLON,
      anon_sym_and,
      anon_sym_or,
  [38317] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1704), 1,
      sym__string_content,
    ACTIONS(1706), 1,
      sym__string_end,
    ACTIONS(1702), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(734), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [38338] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1708), 1,
      anon_sym_COMMA,
    ACTIONS(1710), 1,
      anon_sym_COLON,
    ACTIONS(1712), 1,
      anon_sym_RBRACK,
    STATE(913), 1,
      aux_sym_subscript_repeat1,
  [38363] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1714), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [38382] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1656), 1,
      anon_sym_if,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1495), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_COLON,
  [38401] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1710), 1,
      anon_sym_COLON,
    ACTIONS(1716), 1,
      anon_sym_COMMA,
    ACTIONS(1718), 1,
      anon_sym_RBRACK,
    STATE(941), 1,
      aux_sym_subscript_repeat1,
  [38426] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1710), 1,
      anon_sym_COLON,
    ACTIONS(1720), 1,
      anon_sym_COMMA,
    ACTIONS(1722), 1,
      anon_sym_RBRACK,
    STATE(900), 1,
      aux_sym_subscript_repeat1,
  [38451] = 6,
    ACTIONS(1597), 1,
      anon_sym_LBRACE,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1726), 1,
      sym__string_content,
    ACTIONS(1728), 1,
      sym__string_end,
    ACTIONS(1724), 2,
      sym_escape_sequence,
      sym__not_escape_sequence,
    STATE(736), 2,
      sym_interpolation,
      aux_sym_string_repeat1,
  [38472] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1730), 1,
      sym_identifier,
    ACTIONS(1732), 1,
      anon_sym_DOT,
    ACTIONS(1734), 1,
      anon_sym___future__,
    STATE(834), 1,
      aux_sym_import_prefix_repeat1,
    STATE(899), 1,
      sym_import_prefix,
    STATE(1020), 2,
      sym_relative_import,
      sym_dotted_name,
  [38495] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1656), 1,
      anon_sym_if,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1485), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
      anon_sym_COLON,
  [38514] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1579), 4,
      sym__newline,
      anon_sym_from,
      anon_sym_COMMA,
      sym__semicolon,
  [38533] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1710), 1,
      anon_sym_COLON,
    ACTIONS(1736), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [38553] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1656), 1,
      anon_sym_if,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1740), 1,
      anon_sym_COLON,
    ACTIONS(1738), 2,
      anon_sym_COMMA,
      anon_sym_as,
  [38573] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1744), 1,
      anon_sym_DOT,
    STATE(771), 1,
      aux_sym_dotted_name_repeat1,
    ACTIONS(1742), 4,
      anon_sym_import,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
  [38589] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1744), 1,
      anon_sym_DOT,
    STATE(776), 1,
      aux_sym_dotted_name_repeat1,
    ACTIONS(1746), 4,
      anon_sym_import,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
  [38605] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1748), 3,
      sym__newline,
      anon_sym_COMMA,
      sym__semicolon,
  [38623] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1750), 1,
      anon_sym_COMMA,
    STATE(774), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(672), 4,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [38639] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1752), 1,
      anon_sym_COMMA,
    STATE(774), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1579), 4,
      anon_sym_RPAREN,
      anon_sym_COLON,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [38655] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1755), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [38673] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1759), 1,
      anon_sym_DOT,
    STATE(776), 1,
      aux_sym_dotted_name_repeat1,
    ACTIONS(1757), 4,
      anon_sym_import,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
  [38689] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1656), 1,
      anon_sym_if,
    ACTIONS(1658), 1,
      anon_sym_and,
    ACTIONS(1660), 1,
      anon_sym_or,
    ACTIONS(1764), 1,
      anon_sym_COLON,
    ACTIONS(1762), 2,
      anon_sym_COMMA,
      anon_sym_as,
  [38709] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1558), 1,
      anon_sym_COMMA,
    ACTIONS(1766), 1,
      anon_sym_COLON,
    STATE(773), 1,
      aux_sym_assert_statement_repeat1,
  [38731] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1558), 1,
      anon_sym_COMMA,
    ACTIONS(1768), 1,
      anon_sym_COLON,
    STATE(773), 1,
      aux_sym_assert_statement_repeat1,
  [38753] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1770), 1,
      anon_sym_COLON,
    ACTIONS(811), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [38773] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1772), 1,
      anon_sym_DOT,
    STATE(786), 1,
      aux_sym_dotted_name_repeat1,
    ACTIONS(1742), 4,
      sym__newline,
      anon_sym_COMMA,
      anon_sym_as,
      sym__semicolon,
  [38789] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1774), 3,
      sym__newline,
      anon_sym_COMMA,
      sym__semicolon,
  [38807] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1558), 1,
      anon_sym_COMMA,
    ACTIONS(1776), 1,
      anon_sym_COLON,
    STATE(773), 1,
      aux_sym_assert_statement_repeat1,
  [38829] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1778), 1,
      anon_sym_DOT,
    STATE(784), 1,
      aux_sym_dotted_name_repeat1,
    ACTIONS(1757), 4,
      sym__newline,
      anon_sym_COMMA,
      anon_sym_as,
      sym__semicolon,
  [38845] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1781), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [38863] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1772), 1,
      anon_sym_DOT,
    STATE(784), 1,
      aux_sym_dotted_name_repeat1,
    ACTIONS(1746), 4,
      sym__newline,
      anon_sym_COMMA,
      anon_sym_as,
      sym__semicolon,
  [38879] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(755), 1,
      anon_sym_COLON,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(753), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [38899] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1654), 3,
      sym__newline,
      anon_sym_EQ,
      sym__semicolon,
  [38917] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1783), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [38935] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1521), 1,
      anon_sym_RPAREN,
    ACTIONS(1523), 1,
      anon_sym_COMMA,
    STATE(937), 1,
      aux_sym_argument_list_repeat1,
  [38957] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1787), 1,
      anon_sym_AT,
    STATE(791), 2,
      sym_decorator,
      aux_sym_decorated_definition_repeat1,
    ACTIONS(1785), 3,
      anon_sym_async,
      anon_sym_def,
      anon_sym_class,
  [38973] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1558), 1,
      anon_sym_COMMA,
    ACTIONS(1790), 1,
      anon_sym_COLON,
    STATE(773), 1,
      aux_sym_assert_statement_repeat1,
  [38995] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1792), 1,
      anon_sym_COMMA,
    STATE(793), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1579), 3,
      sym__newline,
      anon_sym_from,
      sym__semicolon,
  [39010] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1795), 1,
      anon_sym_except,
    ACTIONS(1797), 1,
      anon_sym_finally,
    STATE(414), 1,
      sym_finally_clause,
    STATE(171), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
  [39027] = 3,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1801), 2,
      sym__string_content,
      sym__string_end,
    ACTIONS(1799), 3,
      anon_sym_LBRACE,
      sym_escape_sequence,
      sym__not_escape_sequence,
  [39040] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1803), 1,
      anon_sym_COMMA,
    STATE(793), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(672), 3,
      sym__newline,
      anon_sym_from,
      sym__semicolon,
  [39055] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1805), 1,
      anon_sym_except,
    ACTIONS(1807), 1,
      anon_sym_finally,
    STATE(416), 1,
      sym_finally_clause,
    STATE(164), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
  [39072] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1809), 2,
      sym__newline,
      sym__semicolon,
  [39089] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1811), 1,
      anon_sym_COMMA,
    STATE(804), 1,
      aux_sym__collection_elements_repeat1,
    ACTIONS(562), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [39104] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1813), 2,
      sym__newline,
      sym__semicolon,
  [39121] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(809), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [39138] = 5,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1815), 1,
      anon_sym_LBRACE,
    ACTIONS(1817), 1,
      anon_sym_RBRACE,
    ACTIONS(1819), 1,
      aux_sym_format_specifier_token1,
    STATE(807), 2,
      sym_format_expression,
      aux_sym_format_specifier_repeat1,
  [39155] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1821), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [39172] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1823), 1,
      anon_sym_COMMA,
    STATE(804), 1,
      aux_sym__collection_elements_repeat1,
    ACTIONS(1607), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [39187] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1688), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [39204] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1757), 5,
      sym__newline,
      anon_sym_DOT,
      anon_sym_COMMA,
      anon_sym_as,
      sym__semicolon,
  [39215] = 5,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1815), 1,
      anon_sym_LBRACE,
    ACTIONS(1826), 1,
      anon_sym_RBRACE,
    ACTIONS(1828), 1,
      aux_sym_format_specifier_token1,
    STATE(817), 2,
      sym_format_expression,
      aux_sym_format_specifier_repeat1,
  [39232] = 3,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1832), 2,
      sym__string_content,
      sym__string_end,
    ACTIONS(1830), 3,
      anon_sym_LBRACE,
      sym_escape_sequence,
      sym__not_escape_sequence,
  [39245] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1805), 1,
      anon_sym_except,
    ACTIONS(1807), 1,
      anon_sym_finally,
    STATE(357), 1,
      sym_finally_clause,
    STATE(168), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
  [39262] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1757), 5,
      anon_sym_import,
      anon_sym_DOT,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_as,
  [39273] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1613), 1,
      sym_identifier,
    ACTIONS(1834), 1,
      anon_sym_LPAREN,
    STATE(820), 1,
      sym_dotted_name,
    STATE(847), 1,
      sym_aliased_import,
    STATE(963), 1,
      sym__import_list,
  [39292] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1795), 1,
      anon_sym_except,
    ACTIONS(1797), 1,
      anon_sym_finally,
    STATE(402), 1,
      sym_finally_clause,
    STATE(165), 2,
      sym_except_clause,
      aux_sym_try_statement_repeat1,
  [39309] = 3,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1838), 2,
      sym__string_content,
      sym__string_end,
    ACTIONS(1836), 3,
      anon_sym_LBRACE,
      sym_escape_sequence,
      sym__not_escape_sequence,
  [39322] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
    ACTIONS(1483), 3,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [39337] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1552), 1,
      anon_sym_and,
    ACTIONS(1560), 1,
      anon_sym_if,
    ACTIONS(1562), 1,
      anon_sym_or,
    ACTIONS(1668), 2,
      sym__newline,
      sym__semicolon,
  [39354] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1613), 1,
      sym_identifier,
    STATE(831), 1,
      sym_dotted_name,
    STATE(943), 1,
      sym_aliased_import,
    ACTIONS(1840), 2,
      sym__newline,
      sym__semicolon,
  [39371] = 5,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(1842), 1,
      anon_sym_LBRACE,
    ACTIONS(1845), 1,
      anon_sym_RBRACE,
    ACTIONS(1847), 1,
      aux_sym_format_specifier_token1,
    STATE(817), 2,
      sym_format_expression,
      aux_sym_format_specifier_repeat1,
  [39388] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1613), 1,
      sym_identifier,
    STATE(831), 1,
      sym_dotted_name,
    STATE(943), 1,
      sym_aliased_import,
    ACTIONS(1850), 2,
      sym__newline,
      sym__semicolon,
  [39405] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(811), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [39422] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1852), 1,
      anon_sym_COMMA,
    ACTIONS(1854), 1,
      anon_sym_as,
    STATE(836), 1,
      aux_sym__import_list_repeat1,
    ACTIONS(1856), 2,
      sym__newline,
      sym__semicolon,
  [39439] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1858), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [39456] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1860), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [39473] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1862), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [39490] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1866), 1,
      anon_sym_COMMA,
    STATE(871), 1,
      aux_sym__patterns_repeat1,
    ACTIONS(1864), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [39504] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1868), 1,
      anon_sym_else,
  [39520] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1619), 1,
      anon_sym_COMMA,
    STATE(793), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1870), 2,
      sym__newline,
      sym__semicolon,
  [39534] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1872), 1,
      anon_sym_else,
  [39550] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1874), 1,
      anon_sym_COLON,
  [39566] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1876), 1,
      anon_sym_COLON,
  [39582] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1880), 1,
      anon_sym_COLON,
    ACTIONS(1882), 1,
      anon_sym_EQ,
    ACTIONS(1878), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [39596] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1854), 1,
      anon_sym_as,
    ACTIONS(1884), 3,
      sym__newline,
      anon_sym_COMMA,
      sym__semicolon,
  [39608] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1886), 1,
      anon_sym_RBRACE,
  [39624] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1888), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [39634] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1892), 1,
      anon_sym_DOT,
    STATE(844), 1,
      aux_sym_import_prefix_repeat1,
    ACTIONS(1890), 2,
      anon_sym_import,
      sym_identifier,
  [39648] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1894), 1,
      anon_sym_COMMA,
    STATE(793), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(761), 2,
      sym__newline,
      sym__semicolon,
  [39662] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1896), 1,
      anon_sym_COMMA,
    STATE(841), 1,
      aux_sym__import_list_repeat1,
    ACTIONS(1898), 2,
      sym__newline,
      sym__semicolon,
  [39676] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1900), 1,
      anon_sym_COMMA,
    STATE(837), 1,
      aux_sym_print_statement_repeat1,
    ACTIONS(1903), 2,
      sym__newline,
      sym__semicolon,
  [39690] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1905), 1,
      anon_sym_COMMA,
    STATE(843), 1,
      aux_sym_global_statement_repeat1,
    ACTIONS(1907), 2,
      sym__newline,
      sym__semicolon,
  [39704] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1850), 1,
      anon_sym_RPAREN,
    ACTIONS(1909), 1,
      sym_identifier,
    STATE(890), 1,
      sym_dotted_name,
    STATE(972), 1,
      sym_aliased_import,
  [39720] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1911), 4,
      anon_sym_async,
      anon_sym_def,
      anon_sym_class,
      anon_sym_AT,
  [39730] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1913), 1,
      anon_sym_COMMA,
    STATE(841), 1,
      aux_sym__import_list_repeat1,
    ACTIONS(1916), 2,
      sym__newline,
      sym__semicolon,
  [39744] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1905), 1,
      anon_sym_COMMA,
    STATE(860), 1,
      aux_sym_global_statement_repeat1,
    ACTIONS(1918), 2,
      sym__newline,
      sym__semicolon,
  [39758] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1905), 1,
      anon_sym_COMMA,
    STATE(860), 1,
      aux_sym_global_statement_repeat1,
    ACTIONS(1920), 2,
      sym__newline,
      sym__semicolon,
  [39772] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1924), 1,
      anon_sym_DOT,
    STATE(844), 1,
      aux_sym_import_prefix_repeat1,
    ACTIONS(1922), 2,
      anon_sym_import,
      sym_identifier,
  [39786] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1905), 1,
      anon_sym_COMMA,
    STATE(842), 1,
      aux_sym_global_statement_repeat1,
    ACTIONS(1927), 2,
      sym__newline,
      sym__semicolon,
  [39800] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1909), 1,
      sym_identifier,
    STATE(868), 1,
      sym_dotted_name,
    STATE(874), 1,
      sym_aliased_import,
    STATE(1012), 1,
      sym__import_list,
  [39816] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1852), 1,
      anon_sym_COMMA,
    STATE(836), 1,
      aux_sym__import_list_repeat1,
    ACTIONS(1856), 2,
      sym__newline,
      sym__semicolon,
  [39830] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1929), 1,
      anon_sym_COLON,
  [39846] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1931), 1,
      anon_sym_COLON,
  [39862] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1933), 1,
      anon_sym_else,
  [39878] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1935), 1,
      anon_sym_COLON,
  [39894] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1937), 1,
      anon_sym_else,
  [39910] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1939), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [39920] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1941), 1,
      anon_sym_COLON,
  [39936] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1943), 1,
      anon_sym_COMMA,
    STATE(864), 1,
      aux_sym_print_statement_repeat1,
    ACTIONS(1945), 2,
      sym__newline,
      sym__semicolon,
  [39950] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1947), 1,
      anon_sym_COLON,
  [39966] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1949), 1,
      anon_sym_COLON,
  [39982] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1840), 1,
      anon_sym_RPAREN,
    ACTIONS(1909), 1,
      sym_identifier,
    STATE(890), 1,
      sym_dotted_name,
    STATE(972), 1,
      sym_aliased_import,
  [39998] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1953), 1,
      anon_sym_EQ,
    ACTIONS(1951), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [40010] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1955), 1,
      anon_sym_COMMA,
    STATE(860), 1,
      aux_sym_global_statement_repeat1,
    ACTIONS(1958), 2,
      sym__newline,
      sym__semicolon,
  [40024] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1962), 1,
      anon_sym_COMMA,
    STATE(861), 1,
      aux_sym_with_clause_repeat1,
    ACTIONS(1960), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [40038] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1619), 1,
      anon_sym_COMMA,
    STATE(793), 1,
      aux_sym_assert_statement_repeat1,
    ACTIONS(1965), 2,
      sym__newline,
      sym__semicolon,
  [40052] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1967), 1,
      anon_sym_COMMA,
    STATE(837), 1,
      aux_sym_print_statement_repeat1,
    ACTIONS(1969), 2,
      sym__newline,
      sym__semicolon,
  [40066] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1971), 1,
      anon_sym_COMMA,
    STATE(837), 1,
      aux_sym_print_statement_repeat1,
    ACTIONS(1973), 2,
      sym__newline,
      sym__semicolon,
  [40080] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1477), 1,
      anon_sym_and,
    ACTIONS(1487), 1,
      anon_sym_if,
    ACTIONS(1489), 1,
      anon_sym_or,
    ACTIONS(1975), 1,
      anon_sym_COLON,
  [40096] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1909), 1,
      sym_identifier,
    STATE(868), 1,
      sym_dotted_name,
    STATE(874), 1,
      sym_aliased_import,
    STATE(1013), 1,
      sym__import_list,
  [40112] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1607), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [40122] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1856), 1,
      anon_sym_RPAREN,
    ACTIONS(1977), 1,
      anon_sym_COMMA,
    ACTIONS(1979), 1,
      anon_sym_as,
    STATE(880), 1,
      aux_sym__import_list_repeat1,
  [40138] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1556), 4,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_RBRACE,
  [40148] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1613), 1,
      sym_identifier,
    STATE(820), 1,
      sym_dotted_name,
    STATE(847), 1,
      sym_aliased_import,
    STATE(988), 1,
      sym__import_list,
  [40164] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1981), 1,
      anon_sym_COMMA,
    STATE(641), 1,
      aux_sym__patterns_repeat1,
    ACTIONS(1004), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [40178] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1958), 3,
      sym__newline,
      anon_sym_COMMA,
      sym__semicolon,
  [40187] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1983), 1,
      anon_sym_COMMA,
    ACTIONS(1985), 1,
      anon_sym_COLON,
    STATE(892), 1,
      aux_sym_with_clause_repeat1,
  [40200] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1856), 1,
      anon_sym_RPAREN,
    ACTIONS(1977), 1,
      anon_sym_COMMA,
    STATE(880), 1,
      aux_sym__import_list_repeat1,
  [40213] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1418), 1,
      anon_sym_COMMA,
    ACTIONS(1987), 1,
      anon_sym_in,
    STATE(651), 1,
      aux_sym__patterns_repeat1,
  [40226] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1418), 1,
      anon_sym_COMMA,
    ACTIONS(1989), 1,
      anon_sym_in,
    STATE(651), 1,
      aux_sym__patterns_repeat1,
  [40239] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1418), 1,
      anon_sym_COMMA,
    ACTIONS(1991), 1,
      anon_sym_in,
    STATE(651), 1,
      aux_sym__patterns_repeat1,
  [40252] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1993), 1,
      sym__semicolon,
    ACTIONS(1995), 1,
      sym__newline,
    STATE(895), 1,
      aux_sym__simple_statements_repeat1,
  [40265] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1418), 1,
      anon_sym_COMMA,
    ACTIONS(1997), 1,
      anon_sym_in,
    STATE(651), 1,
      aux_sym__patterns_repeat1,
  [40278] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1898), 1,
      anon_sym_RPAREN,
    ACTIONS(1999), 1,
      anon_sym_COMMA,
    STATE(893), 1,
      aux_sym__import_list_repeat1,
  [40291] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1088), 3,
      sym__newline,
      anon_sym_in,
      sym__semicolon,
  [40300] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1509), 1,
      anon_sym_RPAREN,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
  [40313] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2001), 1,
      sym__semicolon,
    ACTIONS(2003), 1,
      sym__newline,
    STATE(934), 1,
      aux_sym__simple_statements_repeat1,
  [40326] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1983), 1,
      anon_sym_COMMA,
    ACTIONS(2005), 1,
      anon_sym_RPAREN,
    STATE(950), 1,
      aux_sym_with_clause_repeat1,
  [40339] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(2007), 1,
      anon_sym_RPAREN,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
  [40352] = 3,
    ACTIONS(1601), 1,
      sym_comment,
    ACTIONS(2011), 1,
      aux_sym_format_specifier_token1,
    ACTIONS(2009), 2,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
  [40363] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1613), 1,
      sym_identifier,
    STATE(831), 1,
      sym_dotted_name,
    STATE(943), 1,
      sym_aliased_import,
  [40376] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2013), 1,
      anon_sym_RPAREN,
    ACTIONS(2015), 1,
      anon_sym_COMMA,
    STATE(908), 1,
      aux_sym__parameters_repeat1,
  [40389] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(2017), 1,
      anon_sym_RPAREN,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
  [40402] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1979), 1,
      anon_sym_as,
    ACTIONS(1884), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [40413] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1625), 1,
      anon_sym_COMMA,
    ACTIONS(1627), 1,
      anon_sym_RBRACE,
    STATE(910), 1,
      aux_sym_dictionary_repeat1,
  [40426] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1983), 1,
      anon_sym_COMMA,
    ACTIONS(2019), 1,
      anon_sym_COLON,
    STATE(861), 1,
      aux_sym_with_clause_repeat1,
  [40439] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1916), 1,
      anon_sym_RPAREN,
    ACTIONS(2021), 1,
      anon_sym_COMMA,
    STATE(893), 1,
      aux_sym__import_list_repeat1,
  [40452] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2024), 1,
      anon_sym_COMMA,
    ACTIONS(2027), 1,
      anon_sym_RBRACK,
    STATE(894), 1,
      aux_sym_subscript_repeat1,
  [40465] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(425), 1,
      sym__newline,
    ACTIONS(2029), 1,
      sym__semicolon,
    STATE(924), 1,
      aux_sym__simple_statements_repeat1,
  [40478] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1862), 1,
      anon_sym_RPAREN,
    ACTIONS(2031), 1,
      anon_sym_COMMA,
    STATE(896), 1,
      aux_sym_argument_list_repeat1,
  [40491] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2034), 1,
      anon_sym_LPAREN,
    ACTIONS(2036), 1,
      anon_sym_COLON,
    STATE(1045), 1,
      sym_argument_list,
  [40504] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1909), 1,
      sym_identifier,
    STATE(890), 1,
      sym_dotted_name,
    STATE(972), 1,
      sym_aliased_import,
  [40517] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1730), 1,
      sym_identifier,
    ACTIONS(2038), 1,
      anon_sym_import,
    STATE(1017), 1,
      sym_dotted_name,
  [40530] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(722), 1,
      anon_sym_RBRACK,
    ACTIONS(2040), 1,
      anon_sym_COMMA,
    STATE(894), 1,
      aux_sym_subscript_repeat1,
  [40543] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(648), 1,
      anon_sym_RPAREN,
    ACTIONS(2042), 1,
      anon_sym_COMMA,
    STATE(896), 1,
      aux_sym_argument_list_repeat1,
  [40556] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1418), 1,
      anon_sym_COMMA,
    ACTIONS(2044), 1,
      anon_sym_in,
    STATE(651), 1,
      aux_sym__patterns_repeat1,
  [40569] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(626), 1,
      anon_sym_RPAREN,
    ACTIONS(2046), 1,
      anon_sym_COMMA,
    STATE(896), 1,
      aux_sym_argument_list_repeat1,
  [40582] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1720), 1,
      anon_sym_COMMA,
    ACTIONS(1722), 1,
      anon_sym_RBRACK,
    STATE(900), 1,
      aux_sym_subscript_repeat1,
  [40595] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(636), 1,
      anon_sym_RPAREN,
    ACTIONS(2048), 1,
      anon_sym_COMMA,
    STATE(896), 1,
      aux_sym_argument_list_repeat1,
  [40608] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1692), 1,
      anon_sym_COMMA,
    ACTIONS(1694), 1,
      anon_sym_RBRACE,
    STATE(915), 1,
      aux_sym_dictionary_repeat1,
  [40621] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2050), 1,
      anon_sym_RPAREN,
    ACTIONS(2052), 1,
      anon_sym_COMMA,
    STATE(901), 1,
      aux_sym_argument_list_repeat1,
  [40634] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1456), 1,
      anon_sym_RPAREN,
    ACTIONS(2054), 1,
      anon_sym_COMMA,
    STATE(933), 1,
      aux_sym__parameters_repeat1,
  [40647] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1092), 3,
      sym__newline,
      anon_sym_in,
      sym__semicolon,
  [40656] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(718), 1,
      anon_sym_RBRACE,
    ACTIONS(2056), 1,
      anon_sym_COMMA,
    STATE(922), 1,
      aux_sym_dictionary_repeat1,
  [40669] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1456), 1,
      anon_sym_COLON,
    ACTIONS(2058), 1,
      anon_sym_COMMA,
    STATE(928), 1,
      aux_sym__parameters_repeat1,
  [40682] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1517), 1,
      anon_sym_RPAREN,
    ACTIONS(1519), 1,
      anon_sym_COMMA,
    STATE(903), 1,
      aux_sym_argument_list_repeat1,
  [40695] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(757), 1,
      anon_sym_RBRACK,
    ACTIONS(2060), 1,
      anon_sym_COMMA,
    STATE(894), 1,
      aux_sym_subscript_repeat1,
  [40708] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    ACTIONS(1515), 1,
      anon_sym_RPAREN,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
  [40721] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(690), 1,
      anon_sym_RBRACE,
    ACTIONS(2062), 1,
      anon_sym_COMMA,
    STATE(922), 1,
      aux_sym_dictionary_repeat1,
  [40734] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(682), 1,
      anon_sym_RBRACE,
    ACTIONS(2064), 1,
      anon_sym_COMMA,
    STATE(922), 1,
      aux_sym_dictionary_repeat1,
  [40747] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(654), 1,
      anon_sym_RPAREN,
    ACTIONS(2066), 1,
      anon_sym_COMMA,
    STATE(896), 1,
      aux_sym_argument_list_repeat1,
  [40760] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(421), 1,
      sym__newline,
    ACTIONS(2068), 1,
      sym__semicolon,
    STATE(924), 1,
      aux_sym__simple_statements_repeat1,
  [40773] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1521), 1,
      anon_sym_RPAREN,
    ACTIONS(1523), 1,
      anon_sym_COMMA,
    STATE(937), 1,
      aux_sym_argument_list_repeat1,
  [40786] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2070), 1,
      anon_sym_RPAREN,
    ACTIONS(2072), 1,
      anon_sym_COMMA,
    STATE(939), 1,
      aux_sym_argument_list_repeat1,
  [40799] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(824), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [40808] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2074), 1,
      anon_sym_COMMA,
    ACTIONS(2077), 1,
      anon_sym_RBRACE,
    STATE(922), 1,
      aux_sym_dictionary_repeat1,
  [40821] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1716), 1,
      anon_sym_COMMA,
    ACTIONS(1718), 1,
      anon_sym_RBRACK,
    STATE(941), 1,
      aux_sym_subscript_repeat1,
  [40834] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2079), 1,
      sym__semicolon,
    ACTIONS(2082), 1,
      sym__newline,
    STATE(924), 1,
      aux_sym__simple_statements_repeat1,
  [40847] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2084), 1,
      anon_sym_EQ,
    ACTIONS(2086), 2,
      sym__newline,
      sym__semicolon,
  [40858] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1684), 1,
      anon_sym_COMMA,
    ACTIONS(1686), 1,
      anon_sym_RBRACE,
    STATE(916), 1,
      aux_sym_dictionary_repeat1,
  [40871] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1951), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [40880] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2088), 1,
      anon_sym_COMMA,
    ACTIONS(2091), 1,
      anon_sym_COLON,
    STATE(928), 1,
      aux_sym__parameters_repeat1,
  [40893] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1460), 1,
      anon_sym_RPAREN,
    ACTIONS(1479), 1,
      anon_sym_COMMA,
    STATE(799), 1,
      aux_sym__collection_elements_repeat1,
  [40906] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2091), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [40915] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2093), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [40924] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1511), 1,
      anon_sym_RPAREN,
    ACTIONS(1513), 1,
      anon_sym_COMMA,
    STATE(905), 1,
      aux_sym_argument_list_repeat1,
  [40937] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2091), 1,
      anon_sym_RPAREN,
    ACTIONS(2095), 1,
      anon_sym_COMMA,
    STATE(933), 1,
      aux_sym__parameters_repeat1,
  [40950] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(419), 1,
      sym__newline,
    ACTIONS(2098), 1,
      sym__semicolon,
    STATE(924), 1,
      aux_sym__simple_statements_repeat1,
  [40963] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 1,
      anon_sym_RPAREN,
    ACTIONS(2102), 1,
      anon_sym_COMMA,
    STATE(917), 1,
      aux_sym_argument_list_repeat1,
  [40976] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1708), 1,
      anon_sym_COMMA,
    ACTIONS(1712), 1,
      anon_sym_RBRACK,
    STATE(913), 1,
      aux_sym_subscript_repeat1,
  [40989] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(634), 1,
      anon_sym_RPAREN,
    ACTIONS(2104), 1,
      anon_sym_COMMA,
    STATE(896), 1,
      aux_sym_argument_list_repeat1,
  [41002] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2106), 1,
      anon_sym_COLON,
    ACTIONS(1878), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [41013] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(632), 1,
      anon_sym_RPAREN,
    ACTIONS(2108), 1,
      anon_sym_COMMA,
    STATE(896), 1,
      aux_sym_argument_list_repeat1,
  [41026] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1696), 1,
      anon_sym_COLON,
    ACTIONS(2110), 1,
      anon_sym_RBRACE,
    STATE(1009), 1,
      sym_format_specifier,
  [41039] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(759), 1,
      anon_sym_RBRACK,
    ACTIONS(2112), 1,
      anon_sym_COMMA,
    STATE(894), 1,
      aux_sym_subscript_repeat1,
  [41052] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1581), 1,
      anon_sym_from,
    ACTIONS(1585), 2,
      sym__newline,
      sym__semicolon,
  [41063] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1884), 3,
      sym__newline,
      anon_sym_COMMA,
      sym__semicolon,
  [41072] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2034), 1,
      anon_sym_LPAREN,
    ACTIONS(2114), 1,
      anon_sym_COLON,
    STATE(1016), 1,
      sym_argument_list,
  [41085] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2116), 3,
      sym__newline,
      anon_sym_COMMA,
      sym__semicolon,
  [41094] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2118), 1,
      anon_sym_in,
    ACTIONS(2120), 2,
      sym__newline,
      sym__semicolon,
  [41105] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1418), 1,
      anon_sym_COMMA,
    ACTIONS(2122), 1,
      anon_sym_in,
    STATE(651), 1,
      aux_sym__patterns_repeat1,
  [41118] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1878), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [41127] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2013), 1,
      anon_sym_COLON,
    ACTIONS(2124), 1,
      anon_sym_COMMA,
    STATE(911), 1,
      aux_sym__parameters_repeat1,
  [41140] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1983), 1,
      anon_sym_COMMA,
    ACTIONS(2126), 1,
      anon_sym_RPAREN,
    STATE(861), 1,
      aux_sym_with_clause_repeat1,
  [41153] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1960), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_COLON,
  [41162] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1882), 1,
      anon_sym_EQ,
    ACTIONS(1878), 2,
      anon_sym_COMMA,
      anon_sym_COLON,
  [41173] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2128), 1,
      sym__semicolon,
    ACTIONS(2130), 1,
      sym__newline,
    STATE(918), 1,
      aux_sym__simple_statements_repeat1,
  [41186] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2132), 2,
      anon_sym_COLON,
      anon_sym_DASH_GT,
  [41194] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2082), 2,
      sym__newline,
      sym__semicolon,
  [41202] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(856), 2,
      anon_sym_except,
      anon_sym_finally,
  [41210] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2134), 1,
      anon_sym_LPAREN,
    STATE(971), 1,
      sym_parameters,
  [41220] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2136), 2,
      sym__newline,
      sym__semicolon,
  [41228] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1556), 2,
      sym__newline,
      sym__semicolon,
  [41236] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1878), 2,
      anon_sym_COMMA,
      anon_sym_COLON,
  [41244] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2138), 2,
      sym__newline,
      sym__semicolon,
  [41252] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2140), 2,
      sym__newline,
      sym__semicolon,
  [41260] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2142), 2,
      sym__newline,
      sym__semicolon,
  [41268] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2134), 1,
      anon_sym_LPAREN,
    STATE(977), 1,
      sym_parameters,
  [41278] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2144), 2,
      sym__newline,
      sym__semicolon,
  [41286] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2134), 1,
      anon_sym_LPAREN,
    STATE(979), 1,
      sym_parameters,
  [41296] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2146), 2,
      sym__newline,
      sym__semicolon,
  [41304] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(863), 2,
      anon_sym_except,
      anon_sym_finally,
  [41312] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2148), 2,
      sym__newline,
      sym__semicolon,
  [41320] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2134), 1,
      anon_sym_LPAREN,
    STATE(973), 1,
      sym_parameters,
  [41330] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2150), 1,
      anon_sym_COLON,
    ACTIONS(2152), 1,
      anon_sym_DASH_GT,
  [41340] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1884), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [41348] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2154), 1,
      anon_sym_COLON,
    ACTIONS(2156), 1,
      anon_sym_DASH_GT,
  [41358] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2116), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [41366] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2158), 2,
      sym__newline,
      sym__semicolon,
  [41374] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(835), 2,
      anon_sym_except,
      anon_sym_finally,
  [41382] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2160), 1,
      anon_sym_COLON,
    ACTIONS(2162), 1,
      anon_sym_DASH_GT,
  [41392] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1631), 2,
      sym__newline,
      sym__semicolon,
  [41400] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2164), 1,
      anon_sym_COLON,
    ACTIONS(2166), 1,
      anon_sym_DASH_GT,
  [41410] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1629), 2,
      sym__newline,
      sym__semicolon,
  [41418] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1611), 2,
      sym__newline,
      sym__semicolon,
  [41426] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1736), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [41434] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2168), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [41442] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1862), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [41450] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(831), 2,
      anon_sym_except,
      anon_sym_finally,
  [41458] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2170), 2,
      sym__newline,
      sym__semicolon,
  [41466] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2172), 2,
      sym__newline,
      sym__semicolon,
  [41474] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2174), 2,
      sym__newline,
      sym__semicolon,
  [41482] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2176), 2,
      anon_sym_COLON,
      anon_sym_DASH_GT,
  [41490] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2178), 2,
      sym__newline,
      sym__semicolon,
  [41498] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2077), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [41506] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(850), 2,
      anon_sym_except,
      anon_sym_finally,
  [41514] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2180), 1,
      sym_identifier,
  [41521] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2182), 1,
      sym_identifier,
  [41528] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2184), 1,
      sym_identifier,
  [41535] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2186), 1,
      anon_sym_RBRACE,
  [41542] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2188), 1,
      anon_sym_in,
  [41549] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2190), 1,
      anon_sym_RPAREN,
  [41556] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2192), 1,
      anon_sym_RBRACK,
  [41563] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2194), 1,
      anon_sym_RPAREN,
  [41570] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2196), 1,
      anon_sym_RBRACE,
  [41577] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2198), 1,
      anon_sym_RBRACE,
  [41584] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2200), 1,
      anon_sym_RBRACE,
  [41591] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1521), 1,
      anon_sym_RPAREN,
  [41598] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2202), 1,
      sym_identifier,
  [41605] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1517), 1,
      anon_sym_RPAREN,
  [41612] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2204), 1,
      anon_sym_RBRACE,
  [41619] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2206), 1,
      anon_sym_RBRACE,
  [41626] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2208), 1,
      anon_sym_RBRACE,
  [41633] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2210), 1,
      anon_sym_RBRACK,
  [41640] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2212), 1,
      sym_identifier,
  [41647] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2214), 1,
      anon_sym_RPAREN,
  [41654] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2216), 1,
      anon_sym_RPAREN,
  [41661] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2218), 1,
      anon_sym_RPAREN,
  [41668] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2220), 1,
      anon_sym_RBRACK,
  [41675] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2222), 1,
      anon_sym_COLON,
  [41682] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2224), 1,
      anon_sym_import,
  [41689] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2044), 1,
      anon_sym_in,
  [41696] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2226), 1,
      anon_sym_RPAREN,
  [41703] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2228), 1,
      anon_sym_import,
  [41710] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1511), 1,
      anon_sym_RPAREN,
  [41717] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2230), 1,
      anon_sym_import,
  [41724] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2232), 1,
      anon_sym_COLON,
  [41731] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1776), 1,
      anon_sym_COLON,
  [41738] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2234), 1,
      anon_sym_RBRACE,
  [41745] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2236), 1,
      anon_sym_COLON,
  [41752] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2238), 1,
      anon_sym_RBRACE,
  [41759] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2240), 1,
      anon_sym_COLON,
  [41766] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2242), 1,
      anon_sym_in,
  [41773] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2244), 1,
      anon_sym_COLON,
  [41780] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1686), 1,
      anon_sym_RBRACE,
  [41787] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2246), 1,
      sym_identifier,
  [41794] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2248), 1,
      anon_sym_COLON,
  [41801] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2250), 1,
      anon_sym_RBRACK,
  [41808] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2252), 1,
      anon_sym_in,
  [41815] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2254), 1,
      anon_sym_COLON,
  [41822] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2256), 1,
      anon_sym_in,
  [41829] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2258), 1,
      anon_sym_RPAREN,
  [41836] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2260), 1,
      sym_identifier,
  [41843] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(678), 1,
      anon_sym_def,
  [41850] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2262), 1,
      sym_identifier,
  [41857] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2264), 1,
      anon_sym_COLON,
  [41864] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2266), 1,
      anon_sym_for,
  [41871] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2268), 1,
      anon_sym_RPAREN,
  [41878] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2270), 1,
      anon_sym_COLON,
  [41885] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2272), 1,
      anon_sym_RPAREN,
  [41892] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1768), 1,
      anon_sym_COLON,
  [41899] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2274), 1,
      anon_sym_COLON,
  [41906] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2276), 1,
      anon_sym_COLON,
  [41913] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2278), 1,
      anon_sym_COLON,
  [41920] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2280), 1,
      anon_sym_COLON,
  [41927] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1766), 1,
      anon_sym_COLON,
  [41934] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2282), 1,
      sym_identifier,
  [41941] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2284), 1,
      anon_sym_RBRACE,
  [41948] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2286), 1,
      anon_sym_COLON,
  [41955] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2122), 1,
      anon_sym_in,
  [41962] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2288), 1,
      anon_sym_COLON,
  [41969] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2290), 1,
      anon_sym_COLON,
  [41976] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2292), 1,
      anon_sym_COLON,
  [41983] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1627), 1,
      anon_sym_RBRACE,
  [41990] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2294), 1,
      anon_sym_COLON,
  [41997] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2296), 1,
      anon_sym_COLON,
  [42004] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1987), 1,
      anon_sym_in,
  [42011] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2298), 1,
      anon_sym_RBRACK,
  [42018] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2300), 1,
      ts_builtin_sym_end,
  [42025] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2302), 1,
      anon_sym_COLON,
  [42032] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2304), 1,
      anon_sym_RPAREN,
  [42039] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2306), 1,
      sym_identifier,
  [42046] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2308), 1,
      sym_identifier,
  [42053] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1790), 1,
      anon_sym_COLON,
  [42060] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2310), 1,
      anon_sym_RBRACK,
  [42067] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2312), 1,
      sym_identifier,
  [42074] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2314), 1,
      sym_identifier,
  [42081] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2316), 1,
      sym_identifier,
  [42088] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2318), 1,
      anon_sym_COLON,
  [42095] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2320), 1,
      anon_sym_RBRACK,
  [42102] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2322), 1,
      anon_sym_COLON,
  [42109] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2324), 1,
      anon_sym_COLON,
  [42116] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2326), 1,
      sym_identifier,
  [42123] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2328), 1,
      sym_identifier,
  [42130] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1694), 1,
      anon_sym_RBRACE,
  [42137] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1997), 1,
      anon_sym_in,
  [42144] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(668), 1,
      anon_sym_def,
  [42151] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2330), 1,
      anon_sym_COLON,
  [42158] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1991), 1,
      anon_sym_in,
  [42165] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2110), 1,
      anon_sym_RBRACE,
  [42172] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1989), 1,
      anon_sym_in,
};

static uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(101)] = 0,
  [SMALL_STATE(102)] = 113,
  [SMALL_STATE(103)] = 226,
  [SMALL_STATE(104)] = 335,
  [SMALL_STATE(105)] = 450,
  [SMALL_STATE(106)] = 567,
  [SMALL_STATE(107)] = 680,
  [SMALL_STATE(108)] = 789,
  [SMALL_STATE(109)] = 904,
  [SMALL_STATE(110)] = 1013,
  [SMALL_STATE(111)] = 1125,
  [SMALL_STATE(112)] = 1237,
  [SMALL_STATE(113)] = 1349,
  [SMALL_STATE(114)] = 1447,
  [SMALL_STATE(115)] = 1545,
  [SMALL_STATE(116)] = 1644,
  [SMALL_STATE(117)] = 1705,
  [SMALL_STATE(118)] = 1766,
  [SMALL_STATE(119)] = 1869,
  [SMALL_STATE(120)] = 1968,
  [SMALL_STATE(121)] = 2071,
  [SMALL_STATE(122)] = 2132,
  [SMALL_STATE(123)] = 2235,
  [SMALL_STATE(124)] = 2340,
  [SMALL_STATE(125)] = 2443,
  [SMALL_STATE(126)] = 2504,
  [SMALL_STATE(127)] = 2600,
  [SMALL_STATE(128)] = 2700,
  [SMALL_STATE(129)] = 2796,
  [SMALL_STATE(130)] = 2896,
  [SMALL_STATE(131)] = 2996,
  [SMALL_STATE(132)] = 3096,
  [SMALL_STATE(133)] = 3196,
  [SMALL_STATE(134)] = 3300,
  [SMALL_STATE(135)] = 3402,
  [SMALL_STATE(136)] = 3504,
  [SMALL_STATE(137)] = 3604,
  [SMALL_STATE(138)] = 3704,
  [SMALL_STATE(139)] = 3804,
  [SMALL_STATE(140)] = 3904,
  [SMALL_STATE(141)] = 4008,
  [SMALL_STATE(142)] = 4108,
  [SMALL_STATE(143)] = 4208,
  [SMALL_STATE(144)] = 4308,
  [SMALL_STATE(145)] = 4404,
  [SMALL_STATE(146)] = 4504,
  [SMALL_STATE(147)] = 4606,
  [SMALL_STATE(148)] = 4706,
  [SMALL_STATE(149)] = 4808,
  [SMALL_STATE(150)] = 4908,
  [SMALL_STATE(151)] = 5008,
  [SMALL_STATE(152)] = 5104,
  [SMALL_STATE(153)] = 5199,
  [SMALL_STATE(154)] = 5296,
  [SMALL_STATE(155)] = 5366,
  [SMALL_STATE(156)] = 5456,
  [SMALL_STATE(157)] = 5546,
  [SMALL_STATE(158)] = 5616,
  [SMALL_STATE(159)] = 5710,
  [SMALL_STATE(160)] = 5803,
  [SMALL_STATE(161)] = 5896,
  [SMALL_STATE(162)] = 5989,
  [SMALL_STATE(163)] = 6082,
  [SMALL_STATE(164)] = 6175,
  [SMALL_STATE(165)] = 6244,
  [SMALL_STATE(166)] = 6313,
  [SMALL_STATE(167)] = 6380,
  [SMALL_STATE(168)] = 6473,
  [SMALL_STATE(169)] = 6542,
  [SMALL_STATE(170)] = 6635,
  [SMALL_STATE(171)] = 6728,
  [SMALL_STATE(172)] = 6797,
  [SMALL_STATE(173)] = 6853,
  [SMALL_STATE(174)] = 6945,
  [SMALL_STATE(175)] = 7001,
  [SMALL_STATE(176)] = 7065,
  [SMALL_STATE(177)] = 7157,
  [SMALL_STATE(178)] = 7247,
  [SMALL_STATE(179)] = 7339,
  [SMALL_STATE(180)] = 7429,
  [SMALL_STATE(181)] = 7519,
  [SMALL_STATE(182)] = 7611,
  [SMALL_STATE(183)] = 7667,
  [SMALL_STATE(184)] = 7757,
  [SMALL_STATE(185)] = 7847,
  [SMALL_STATE(186)] = 7903,
  [SMALL_STATE(187)] = 7993,
  [SMALL_STATE(188)] = 8085,
  [SMALL_STATE(189)] = 8177,
  [SMALL_STATE(190)] = 8264,
  [SMALL_STATE(191)] = 8329,
  [SMALL_STATE(192)] = 8418,
  [SMALL_STATE(193)] = 8505,
  [SMALL_STATE(194)] = 8592,
  [SMALL_STATE(195)] = 8681,
  [SMALL_STATE(196)] = 8746,
  [SMALL_STATE(197)] = 8811,
  [SMALL_STATE(198)] = 8870,
  [SMALL_STATE(199)] = 8957,
  [SMALL_STATE(200)] = 9046,
  [SMALL_STATE(201)] = 9133,
  [SMALL_STATE(202)] = 9194,
  [SMALL_STATE(203)] = 9259,
  [SMALL_STATE(204)] = 9346,
  [SMALL_STATE(205)] = 9433,
  [SMALL_STATE(206)] = 9498,
  [SMALL_STATE(207)] = 9563,
  [SMALL_STATE(208)] = 9650,
  [SMALL_STATE(209)] = 9711,
  [SMALL_STATE(210)] = 9776,
  [SMALL_STATE(211)] = 9865,
  [SMALL_STATE(212)] = 9952,
  [SMALL_STATE(213)] = 10041,
  [SMALL_STATE(214)] = 10128,
  [SMALL_STATE(215)] = 10215,
  [SMALL_STATE(216)] = 10304,
  [SMALL_STATE(217)] = 10365,
  [SMALL_STATE(218)] = 10430,
  [SMALL_STATE(219)] = 10489,
  [SMALL_STATE(220)] = 10578,
  [SMALL_STATE(221)] = 10665,
  [SMALL_STATE(222)] = 10726,
  [SMALL_STATE(223)] = 10813,
  [SMALL_STATE(224)] = 10900,
  [SMALL_STATE(225)] = 10989,
  [SMALL_STATE(226)] = 11075,
  [SMALL_STATE(227)] = 11129,
  [SMALL_STATE(228)] = 11215,
  [SMALL_STATE(229)] = 11269,
  [SMALL_STATE(230)] = 11355,
  [SMALL_STATE(231)] = 11415,
  [SMALL_STATE(232)] = 11501,
  [SMALL_STATE(233)] = 11587,
  [SMALL_STATE(234)] = 11673,
  [SMALL_STATE(235)] = 11759,
  [SMALL_STATE(236)] = 11813,
  [SMALL_STATE(237)] = 11899,
  [SMALL_STATE(238)] = 11953,
  [SMALL_STATE(239)] = 12007,
  [SMALL_STATE(240)] = 12093,
  [SMALL_STATE(241)] = 12147,
  [SMALL_STATE(242)] = 12207,
  [SMALL_STATE(243)] = 12261,
  [SMALL_STATE(244)] = 12347,
  [SMALL_STATE(245)] = 12433,
  [SMALL_STATE(246)] = 12487,
  [SMALL_STATE(247)] = 12573,
  [SMALL_STATE(248)] = 12627,
  [SMALL_STATE(249)] = 12713,
  [SMALL_STATE(250)] = 12799,
  [SMALL_STATE(251)] = 12853,
  [SMALL_STATE(252)] = 12939,
  [SMALL_STATE(253)] = 13022,
  [SMALL_STATE(254)] = 13105,
  [SMALL_STATE(255)] = 13158,
  [SMALL_STATE(256)] = 13241,
  [SMALL_STATE(257)] = 13324,
  [SMALL_STATE(258)] = 13407,
  [SMALL_STATE(259)] = 13490,
  [SMALL_STATE(260)] = 13573,
  [SMALL_STATE(261)] = 13626,
  [SMALL_STATE(262)] = 13709,
  [SMALL_STATE(263)] = 13792,
  [SMALL_STATE(264)] = 13875,
  [SMALL_STATE(265)] = 13958,
  [SMALL_STATE(266)] = 14041,
  [SMALL_STATE(267)] = 14124,
  [SMALL_STATE(268)] = 14177,
  [SMALL_STATE(269)] = 14260,
  [SMALL_STATE(270)] = 14343,
  [SMALL_STATE(271)] = 14426,
  [SMALL_STATE(272)] = 14509,
  [SMALL_STATE(273)] = 14592,
  [SMALL_STATE(274)] = 14645,
  [SMALL_STATE(275)] = 14728,
  [SMALL_STATE(276)] = 14811,
  [SMALL_STATE(277)] = 14894,
  [SMALL_STATE(278)] = 14977,
  [SMALL_STATE(279)] = 15060,
  [SMALL_STATE(280)] = 15113,
  [SMALL_STATE(281)] = 15196,
  [SMALL_STATE(282)] = 15249,
  [SMALL_STATE(283)] = 15332,
  [SMALL_STATE(284)] = 15415,
  [SMALL_STATE(285)] = 15498,
  [SMALL_STATE(286)] = 15581,
  [SMALL_STATE(287)] = 15664,
  [SMALL_STATE(288)] = 15747,
  [SMALL_STATE(289)] = 15832,
  [SMALL_STATE(290)] = 15915,
  [SMALL_STATE(291)] = 15998,
  [SMALL_STATE(292)] = 16081,
  [SMALL_STATE(293)] = 16164,
  [SMALL_STATE(294)] = 16247,
  [SMALL_STATE(295)] = 16330,
  [SMALL_STATE(296)] = 16383,
  [SMALL_STATE(297)] = 16466,
  [SMALL_STATE(298)] = 16549,
  [SMALL_STATE(299)] = 16632,
  [SMALL_STATE(300)] = 16715,
  [SMALL_STATE(301)] = 16798,
  [SMALL_STATE(302)] = 16881,
  [SMALL_STATE(303)] = 16964,
  [SMALL_STATE(304)] = 17017,
  [SMALL_STATE(305)] = 17100,
  [SMALL_STATE(306)] = 17183,
  [SMALL_STATE(307)] = 17266,
  [SMALL_STATE(308)] = 17349,
  [SMALL_STATE(309)] = 17432,
  [SMALL_STATE(310)] = 17485,
  [SMALL_STATE(311)] = 17568,
  [SMALL_STATE(312)] = 17651,
  [SMALL_STATE(313)] = 17734,
  [SMALL_STATE(314)] = 17817,
  [SMALL_STATE(315)] = 17870,
  [SMALL_STATE(316)] = 17923,
  [SMALL_STATE(317)] = 18006,
  [SMALL_STATE(318)] = 18089,
  [SMALL_STATE(319)] = 18172,
  [SMALL_STATE(320)] = 18255,
  [SMALL_STATE(321)] = 18338,
  [SMALL_STATE(322)] = 18391,
  [SMALL_STATE(323)] = 18474,
  [SMALL_STATE(324)] = 18557,
  [SMALL_STATE(325)] = 18640,
  [SMALL_STATE(326)] = 18723,
  [SMALL_STATE(327)] = 18779,
  [SMALL_STATE(328)] = 18835,
  [SMALL_STATE(329)] = 18891,
  [SMALL_STATE(330)] = 18947,
  [SMALL_STATE(331)] = 19003,
  [SMALL_STATE(332)] = 19055,
  [SMALL_STATE(333)] = 19107,
  [SMALL_STATE(334)] = 19163,
  [SMALL_STATE(335)] = 19215,
  [SMALL_STATE(336)] = 19271,
  [SMALL_STATE(337)] = 19327,
  [SMALL_STATE(338)] = 19379,
  [SMALL_STATE(339)] = 19435,
  [SMALL_STATE(340)] = 19487,
  [SMALL_STATE(341)] = 19543,
  [SMALL_STATE(342)] = 19599,
  [SMALL_STATE(343)] = 19655,
  [SMALL_STATE(344)] = 19707,
  [SMALL_STATE(345)] = 19763,
  [SMALL_STATE(346)] = 19819,
  [SMALL_STATE(347)] = 19875,
  [SMALL_STATE(348)] = 19931,
  [SMALL_STATE(349)] = 19986,
  [SMALL_STATE(350)] = 20037,
  [SMALL_STATE(351)] = 20092,
  [SMALL_STATE(352)] = 20147,
  [SMALL_STATE(353)] = 20198,
  [SMALL_STATE(354)] = 20249,
  [SMALL_STATE(355)] = 20300,
  [SMALL_STATE(356)] = 20350,
  [SMALL_STATE(357)] = 20430,
  [SMALL_STATE(358)] = 20480,
  [SMALL_STATE(359)] = 20530,
  [SMALL_STATE(360)] = 20580,
  [SMALL_STATE(361)] = 20630,
  [SMALL_STATE(362)] = 20680,
  [SMALL_STATE(363)] = 20730,
  [SMALL_STATE(364)] = 20780,
  [SMALL_STATE(365)] = 20830,
  [SMALL_STATE(366)] = 20880,
  [SMALL_STATE(367)] = 20930,
  [SMALL_STATE(368)] = 20980,
  [SMALL_STATE(369)] = 21030,
  [SMALL_STATE(370)] = 21080,
  [SMALL_STATE(371)] = 21130,
  [SMALL_STATE(372)] = 21180,
  [SMALL_STATE(373)] = 21230,
  [SMALL_STATE(374)] = 21310,
  [SMALL_STATE(375)] = 21360,
  [SMALL_STATE(376)] = 21410,
  [SMALL_STATE(377)] = 21460,
  [SMALL_STATE(378)] = 21510,
  [SMALL_STATE(379)] = 21560,
  [SMALL_STATE(380)] = 21610,
  [SMALL_STATE(381)] = 21660,
  [SMALL_STATE(382)] = 21710,
  [SMALL_STATE(383)] = 21760,
  [SMALL_STATE(384)] = 21810,
  [SMALL_STATE(385)] = 21860,
  [SMALL_STATE(386)] = 21910,
  [SMALL_STATE(387)] = 21960,
  [SMALL_STATE(388)] = 22010,
  [SMALL_STATE(389)] = 22060,
  [SMALL_STATE(390)] = 22110,
  [SMALL_STATE(391)] = 22160,
  [SMALL_STATE(392)] = 22210,
  [SMALL_STATE(393)] = 22260,
  [SMALL_STATE(394)] = 22310,
  [SMALL_STATE(395)] = 22360,
  [SMALL_STATE(396)] = 22410,
  [SMALL_STATE(397)] = 22460,
  [SMALL_STATE(398)] = 22510,
  [SMALL_STATE(399)] = 22560,
  [SMALL_STATE(400)] = 22610,
  [SMALL_STATE(401)] = 22660,
  [SMALL_STATE(402)] = 22710,
  [SMALL_STATE(403)] = 22760,
  [SMALL_STATE(404)] = 22810,
  [SMALL_STATE(405)] = 22860,
  [SMALL_STATE(406)] = 22910,
  [SMALL_STATE(407)] = 22960,
  [SMALL_STATE(408)] = 23010,
  [SMALL_STATE(409)] = 23060,
  [SMALL_STATE(410)] = 23110,
  [SMALL_STATE(411)] = 23160,
  [SMALL_STATE(412)] = 23210,
  [SMALL_STATE(413)] = 23260,
  [SMALL_STATE(414)] = 23310,
  [SMALL_STATE(415)] = 23360,
  [SMALL_STATE(416)] = 23410,
  [SMALL_STATE(417)] = 23460,
  [SMALL_STATE(418)] = 23510,
  [SMALL_STATE(419)] = 23560,
  [SMALL_STATE(420)] = 23610,
  [SMALL_STATE(421)] = 23660,
  [SMALL_STATE(422)] = 23710,
  [SMALL_STATE(423)] = 23760,
  [SMALL_STATE(424)] = 23810,
  [SMALL_STATE(425)] = 23892,
  [SMALL_STATE(426)] = 23942,
  [SMALL_STATE(427)] = 23992,
  [SMALL_STATE(428)] = 24042,
  [SMALL_STATE(429)] = 24121,
  [SMALL_STATE(430)] = 24200,
  [SMALL_STATE(431)] = 24279,
  [SMALL_STATE(432)] = 24328,
  [SMALL_STATE(433)] = 24377,
  [SMALL_STATE(434)] = 24456,
  [SMALL_STATE(435)] = 24535,
  [SMALL_STATE(436)] = 24614,
  [SMALL_STATE(437)] = 24690,
  [SMALL_STATE(438)] = 24738,
  [SMALL_STATE(439)] = 24786,
  [SMALL_STATE(440)] = 24834,
  [SMALL_STATE(441)] = 24882,
  [SMALL_STATE(442)] = 24930,
  [SMALL_STATE(443)] = 24978,
  [SMALL_STATE(444)] = 25026,
  [SMALL_STATE(445)] = 25074,
  [SMALL_STATE(446)] = 25150,
  [SMALL_STATE(447)] = 25232,
  [SMALL_STATE(448)] = 25280,
  [SMALL_STATE(449)] = 25362,
  [SMALL_STATE(450)] = 25410,
  [SMALL_STATE(451)] = 25458,
  [SMALL_STATE(452)] = 25506,
  [SMALL_STATE(453)] = 25554,
  [SMALL_STATE(454)] = 25602,
  [SMALL_STATE(455)] = 25650,
  [SMALL_STATE(456)] = 25698,
  [SMALL_STATE(457)] = 25746,
  [SMALL_STATE(458)] = 25794,
  [SMALL_STATE(459)] = 25842,
  [SMALL_STATE(460)] = 25890,
  [SMALL_STATE(461)] = 25938,
  [SMALL_STATE(462)] = 26009,
  [SMALL_STATE(463)] = 26074,
  [SMALL_STATE(464)] = 26145,
  [SMALL_STATE(465)] = 26202,
  [SMALL_STATE(466)] = 26271,
  [SMALL_STATE(467)] = 26338,
  [SMALL_STATE(468)] = 26403,
  [SMALL_STATE(469)] = 26466,
  [SMALL_STATE(470)] = 26527,
  [SMALL_STATE(471)] = 26584,
  [SMALL_STATE(472)] = 26655,
  [SMALL_STATE(473)] = 26718,
  [SMALL_STATE(474)] = 26775,
  [SMALL_STATE(475)] = 26832,
  [SMALL_STATE(476)] = 26889,
  [SMALL_STATE(477)] = 26946,
  [SMALL_STATE(478)] = 27017,
  [SMALL_STATE(479)] = 27078,
  [SMALL_STATE(480)] = 27145,
  [SMALL_STATE(481)] = 27214,
  [SMALL_STATE(482)] = 27262,
  [SMALL_STATE(483)] = 27308,
  [SMALL_STATE(484)] = 27354,
  [SMALL_STATE(485)] = 27402,
  [SMALL_STATE(486)] = 27472,
  [SMALL_STATE(487)] = 27518,
  [SMALL_STATE(488)] = 27597,
  [SMALL_STATE(489)] = 27642,
  [SMALL_STATE(490)] = 27687,
  [SMALL_STATE(491)] = 27732,
  [SMALL_STATE(492)] = 27781,
  [SMALL_STATE(493)] = 27830,
  [SMALL_STATE(494)] = 27879,
  [SMALL_STATE(495)] = 27945,
  [SMALL_STATE(496)] = 28003,
  [SMALL_STATE(497)] = 28079,
  [SMALL_STATE(498)] = 28133,
  [SMALL_STATE(499)] = 28195,
  [SMALL_STATE(500)] = 28259,
  [SMALL_STATE(501)] = 28323,
  [SMALL_STATE(502)] = 28371,
  [SMALL_STATE(503)] = 28439,
  [SMALL_STATE(504)] = 28503,
  [SMALL_STATE(505)] = 28567,
  [SMALL_STATE(506)] = 28615,
  [SMALL_STATE(507)] = 28669,
  [SMALL_STATE(508)] = 28723,
  [SMALL_STATE(509)] = 28771,
  [SMALL_STATE(510)] = 28831,
  [SMALL_STATE(511)] = 28895,
  [SMALL_STATE(512)] = 28963,
  [SMALL_STATE(513)] = 29024,
  [SMALL_STATE(514)] = 29077,
  [SMALL_STATE(515)] = 29138,
  [SMALL_STATE(516)] = 29199,
  [SMALL_STATE(517)] = 29260,
  [SMALL_STATE(518)] = 29303,
  [SMALL_STATE(519)] = 29364,
  [SMALL_STATE(520)] = 29425,
  [SMALL_STATE(521)] = 29486,
  [SMALL_STATE(522)] = 29547,
  [SMALL_STATE(523)] = 29608,
  [SMALL_STATE(524)] = 29669,
  [SMALL_STATE(525)] = 29730,
  [SMALL_STATE(526)] = 29791,
  [SMALL_STATE(527)] = 29852,
  [SMALL_STATE(528)] = 29897,
  [SMALL_STATE(529)] = 29942,
  [SMALL_STATE(530)] = 29987,
  [SMALL_STATE(531)] = 30032,
  [SMALL_STATE(532)] = 30085,
  [SMALL_STATE(533)] = 30146,
  [SMALL_STATE(534)] = 30207,
  [SMALL_STATE(535)] = 30268,
  [SMALL_STATE(536)] = 30329,
  [SMALL_STATE(537)] = 30390,
  [SMALL_STATE(538)] = 30437,
  [SMALL_STATE(539)] = 30482,
  [SMALL_STATE(540)] = 30543,
  [SMALL_STATE(541)] = 30588,
  [SMALL_STATE(542)] = 30649,
  [SMALL_STATE(543)] = 30710,
  [SMALL_STATE(544)] = 30771,
  [SMALL_STATE(545)] = 30832,
  [SMALL_STATE(546)] = 30893,
  [SMALL_STATE(547)] = 30954,
  [SMALL_STATE(548)] = 31019,
  [SMALL_STATE(549)] = 31084,
  [SMALL_STATE(550)] = 31145,
  [SMALL_STATE(551)] = 31190,
  [SMALL_STATE(552)] = 31251,
  [SMALL_STATE(553)] = 31312,
  [SMALL_STATE(554)] = 31373,
  [SMALL_STATE(555)] = 31434,
  [SMALL_STATE(556)] = 31495,
  [SMALL_STATE(557)] = 31556,
  [SMALL_STATE(558)] = 31617,
  [SMALL_STATE(559)] = 31678,
  [SMALL_STATE(560)] = 31739,
  [SMALL_STATE(561)] = 31798,
  [SMALL_STATE(562)] = 31865,
  [SMALL_STATE(563)] = 31918,
  [SMALL_STATE(564)] = 31979,
  [SMALL_STATE(565)] = 32036,
  [SMALL_STATE(566)] = 32101,
  [SMALL_STATE(567)] = 32164,
  [SMALL_STATE(568)] = 32225,
  [SMALL_STATE(569)] = 32286,
  [SMALL_STATE(570)] = 32353,
  [SMALL_STATE(571)] = 32414,
  [SMALL_STATE(572)] = 32475,
  [SMALL_STATE(573)] = 32518,
  [SMALL_STATE(574)] = 32565,
  [SMALL_STATE(575)] = 32626,
  [SMALL_STATE(576)] = 32687,
  [SMALL_STATE(577)] = 32729,
  [SMALL_STATE(578)] = 32771,
  [SMALL_STATE(579)] = 32813,
  [SMALL_STATE(580)] = 32857,
  [SMALL_STATE(581)] = 32899,
  [SMALL_STATE(582)] = 32941,
  [SMALL_STATE(583)] = 32985,
  [SMALL_STATE(584)] = 33027,
  [SMALL_STATE(585)] = 33069,
  [SMALL_STATE(586)] = 33111,
  [SMALL_STATE(587)] = 33153,
  [SMALL_STATE(588)] = 33197,
  [SMALL_STATE(589)] = 33241,
  [SMALL_STATE(590)] = 33283,
  [SMALL_STATE(591)] = 33325,
  [SMALL_STATE(592)] = 33367,
  [SMALL_STATE(593)] = 33411,
  [SMALL_STATE(594)] = 33453,
  [SMALL_STATE(595)] = 33497,
  [SMALL_STATE(596)] = 33539,
  [SMALL_STATE(597)] = 33581,
  [SMALL_STATE(598)] = 33623,
  [SMALL_STATE(599)] = 33665,
  [SMALL_STATE(600)] = 33707,
  [SMALL_STATE(601)] = 33749,
  [SMALL_STATE(602)] = 33791,
  [SMALL_STATE(603)] = 33833,
  [SMALL_STATE(604)] = 33875,
  [SMALL_STATE(605)] = 33917,
  [SMALL_STATE(606)] = 33959,
  [SMALL_STATE(607)] = 34001,
  [SMALL_STATE(608)] = 34043,
  [SMALL_STATE(609)] = 34084,
  [SMALL_STATE(610)] = 34125,
  [SMALL_STATE(611)] = 34166,
  [SMALL_STATE(612)] = 34207,
  [SMALL_STATE(613)] = 34248,
  [SMALL_STATE(614)] = 34293,
  [SMALL_STATE(615)] = 34334,
  [SMALL_STATE(616)] = 34375,
  [SMALL_STATE(617)] = 34416,
  [SMALL_STATE(618)] = 34457,
  [SMALL_STATE(619)] = 34498,
  [SMALL_STATE(620)] = 34539,
  [SMALL_STATE(621)] = 34580,
  [SMALL_STATE(622)] = 34621,
  [SMALL_STATE(623)] = 34662,
  [SMALL_STATE(624)] = 34707,
  [SMALL_STATE(625)] = 34748,
  [SMALL_STATE(626)] = 34789,
  [SMALL_STATE(627)] = 34830,
  [SMALL_STATE(628)] = 34871,
  [SMALL_STATE(629)] = 34912,
  [SMALL_STATE(630)] = 34953,
  [SMALL_STATE(631)] = 34994,
  [SMALL_STATE(632)] = 35035,
  [SMALL_STATE(633)] = 35076,
  [SMALL_STATE(634)] = 35117,
  [SMALL_STATE(635)] = 35158,
  [SMALL_STATE(636)] = 35200,
  [SMALL_STATE(637)] = 35242,
  [SMALL_STATE(638)] = 35282,
  [SMALL_STATE(639)] = 35322,
  [SMALL_STATE(640)] = 35362,
  [SMALL_STATE(641)] = 35402,
  [SMALL_STATE(642)] = 35432,
  [SMALL_STATE(643)] = 35469,
  [SMALL_STATE(644)] = 35518,
  [SMALL_STATE(645)] = 35555,
  [SMALL_STATE(646)] = 35584,
  [SMALL_STATE(647)] = 35609,
  [SMALL_STATE(648)] = 35634,
  [SMALL_STATE(649)] = 35663,
  [SMALL_STATE(650)] = 35688,
  [SMALL_STATE(651)] = 35713,
  [SMALL_STATE(652)] = 35741,
  [SMALL_STATE(653)] = 35787,
  [SMALL_STATE(654)] = 35821,
  [SMALL_STATE(655)] = 35855,
  [SMALL_STATE(656)] = 35886,
  [SMALL_STATE(657)] = 35911,
  [SMALL_STATE(658)] = 35949,
  [SMALL_STATE(659)] = 35987,
  [SMALL_STATE(660)] = 36025,
  [SMALL_STATE(661)] = 36063,
  [SMALL_STATE(662)] = 36085,
  [SMALL_STATE(663)] = 36123,
  [SMALL_STATE(664)] = 36145,
  [SMALL_STATE(665)] = 36167,
  [SMALL_STATE(666)] = 36202,
  [SMALL_STATE(667)] = 36234,
  [SMALL_STATE(668)] = 36266,
  [SMALL_STATE(669)] = 36298,
  [SMALL_STATE(670)] = 36330,
  [SMALL_STATE(671)] = 36367,
  [SMALL_STATE(672)] = 36396,
  [SMALL_STATE(673)] = 36415,
  [SMALL_STATE(674)] = 36452,
  [SMALL_STATE(675)] = 36481,
  [SMALL_STATE(676)] = 36504,
  [SMALL_STATE(677)] = 36527,
  [SMALL_STATE(678)] = 36552,
  [SMALL_STATE(679)] = 36589,
  [SMALL_STATE(680)] = 36614,
  [SMALL_STATE(681)] = 36637,
  [SMALL_STATE(682)] = 36662,
  [SMALL_STATE(683)] = 36699,
  [SMALL_STATE(684)] = 36720,
  [SMALL_STATE(685)] = 36741,
  [SMALL_STATE(686)] = 36762,
  [SMALL_STATE(687)] = 36785,
  [SMALL_STATE(688)] = 36808,
  [SMALL_STATE(689)] = 36842,
  [SMALL_STATE(690)] = 36876,
  [SMALL_STATE(691)] = 36910,
  [SMALL_STATE(692)] = 36944,
  [SMALL_STATE(693)] = 36978,
  [SMALL_STATE(694)] = 37012,
  [SMALL_STATE(695)] = 37046,
  [SMALL_STATE(696)] = 37080,
  [SMALL_STATE(697)] = 37114,
  [SMALL_STATE(698)] = 37137,
  [SMALL_STATE(699)] = 37156,
  [SMALL_STATE(700)] = 37175,
  [SMALL_STATE(701)] = 37194,
  [SMALL_STATE(702)] = 37217,
  [SMALL_STATE(703)] = 37240,
  [SMALL_STATE(704)] = 37258,
  [SMALL_STATE(705)] = 37276,
  [SMALL_STATE(706)] = 37290,
  [SMALL_STATE(707)] = 37306,
  [SMALL_STATE(708)] = 37324,
  [SMALL_STATE(709)] = 37348,
  [SMALL_STATE(710)] = 37362,
  [SMALL_STATE(711)] = 37376,
  [SMALL_STATE(712)] = 37396,
  [SMALL_STATE(713)] = 37414,
  [SMALL_STATE(714)] = 37432,
  [SMALL_STATE(715)] = 37450,
  [SMALL_STATE(716)] = 37468,
  [SMALL_STATE(717)] = 37488,
  [SMALL_STATE(718)] = 37512,
  [SMALL_STATE(719)] = 37532,
  [SMALL_STATE(720)] = 37558,
  [SMALL_STATE(721)] = 37582,
  [SMALL_STATE(722)] = 37600,
  [SMALL_STATE(723)] = 37620,
  [SMALL_STATE(724)] = 37641,
  [SMALL_STATE(725)] = 37660,
  [SMALL_STATE(726)] = 37683,
  [SMALL_STATE(727)] = 37706,
  [SMALL_STATE(728)] = 37719,
  [SMALL_STATE(729)] = 37742,
  [SMALL_STATE(730)] = 37765,
  [SMALL_STATE(731)] = 37790,
  [SMALL_STATE(732)] = 37813,
  [SMALL_STATE(733)] = 37836,
  [SMALL_STATE(734)] = 37857,
  [SMALL_STATE(735)] = 37878,
  [SMALL_STATE(736)] = 37899,
  [SMALL_STATE(737)] = 37920,
  [SMALL_STATE(738)] = 37939,
  [SMALL_STATE(739)] = 37958,
  [SMALL_STATE(740)] = 37981,
  [SMALL_STATE(741)] = 38002,
  [SMALL_STATE(742)] = 38021,
  [SMALL_STATE(743)] = 38044,
  [SMALL_STATE(744)] = 38057,
  [SMALL_STATE(745)] = 38078,
  [SMALL_STATE(746)] = 38095,
  [SMALL_STATE(747)] = 38116,
  [SMALL_STATE(748)] = 38129,
  [SMALL_STATE(749)] = 38154,
  [SMALL_STATE(750)] = 38167,
  [SMALL_STATE(751)] = 38186,
  [SMALL_STATE(752)] = 38209,
  [SMALL_STATE(753)] = 38222,
  [SMALL_STATE(754)] = 38239,
  [SMALL_STATE(755)] = 38264,
  [SMALL_STATE(756)] = 38279,
  [SMALL_STATE(757)] = 38304,
  [SMALL_STATE(758)] = 38317,
  [SMALL_STATE(759)] = 38338,
  [SMALL_STATE(760)] = 38363,
  [SMALL_STATE(761)] = 38382,
  [SMALL_STATE(762)] = 38401,
  [SMALL_STATE(763)] = 38426,
  [SMALL_STATE(764)] = 38451,
  [SMALL_STATE(765)] = 38472,
  [SMALL_STATE(766)] = 38495,
  [SMALL_STATE(767)] = 38514,
  [SMALL_STATE(768)] = 38533,
  [SMALL_STATE(769)] = 38553,
  [SMALL_STATE(770)] = 38573,
  [SMALL_STATE(771)] = 38589,
  [SMALL_STATE(772)] = 38605,
  [SMALL_STATE(773)] = 38623,
  [SMALL_STATE(774)] = 38639,
  [SMALL_STATE(775)] = 38655,
  [SMALL_STATE(776)] = 38673,
  [SMALL_STATE(777)] = 38689,
  [SMALL_STATE(778)] = 38709,
  [SMALL_STATE(779)] = 38731,
  [SMALL_STATE(780)] = 38753,
  [SMALL_STATE(781)] = 38773,
  [SMALL_STATE(782)] = 38789,
  [SMALL_STATE(783)] = 38807,
  [SMALL_STATE(784)] = 38829,
  [SMALL_STATE(785)] = 38845,
  [SMALL_STATE(786)] = 38863,
  [SMALL_STATE(787)] = 38879,
  [SMALL_STATE(788)] = 38899,
  [SMALL_STATE(789)] = 38917,
  [SMALL_STATE(790)] = 38935,
  [SMALL_STATE(791)] = 38957,
  [SMALL_STATE(792)] = 38973,
  [SMALL_STATE(793)] = 38995,
  [SMALL_STATE(794)] = 39010,
  [SMALL_STATE(795)] = 39027,
  [SMALL_STATE(796)] = 39040,
  [SMALL_STATE(797)] = 39055,
  [SMALL_STATE(798)] = 39072,
  [SMALL_STATE(799)] = 39089,
  [SMALL_STATE(800)] = 39104,
  [SMALL_STATE(801)] = 39121,
  [SMALL_STATE(802)] = 39138,
  [SMALL_STATE(803)] = 39155,
  [SMALL_STATE(804)] = 39172,
  [SMALL_STATE(805)] = 39187,
  [SMALL_STATE(806)] = 39204,
  [SMALL_STATE(807)] = 39215,
  [SMALL_STATE(808)] = 39232,
  [SMALL_STATE(809)] = 39245,
  [SMALL_STATE(810)] = 39262,
  [SMALL_STATE(811)] = 39273,
  [SMALL_STATE(812)] = 39292,
  [SMALL_STATE(813)] = 39309,
  [SMALL_STATE(814)] = 39322,
  [SMALL_STATE(815)] = 39337,
  [SMALL_STATE(816)] = 39354,
  [SMALL_STATE(817)] = 39371,
  [SMALL_STATE(818)] = 39388,
  [SMALL_STATE(819)] = 39405,
  [SMALL_STATE(820)] = 39422,
  [SMALL_STATE(821)] = 39439,
  [SMALL_STATE(822)] = 39456,
  [SMALL_STATE(823)] = 39473,
  [SMALL_STATE(824)] = 39490,
  [SMALL_STATE(825)] = 39504,
  [SMALL_STATE(826)] = 39520,
  [SMALL_STATE(827)] = 39534,
  [SMALL_STATE(828)] = 39550,
  [SMALL_STATE(829)] = 39566,
  [SMALL_STATE(830)] = 39582,
  [SMALL_STATE(831)] = 39596,
  [SMALL_STATE(832)] = 39608,
  [SMALL_STATE(833)] = 39624,
  [SMALL_STATE(834)] = 39634,
  [SMALL_STATE(835)] = 39648,
  [SMALL_STATE(836)] = 39662,
  [SMALL_STATE(837)] = 39676,
  [SMALL_STATE(838)] = 39690,
  [SMALL_STATE(839)] = 39704,
  [SMALL_STATE(840)] = 39720,
  [SMALL_STATE(841)] = 39730,
  [SMALL_STATE(842)] = 39744,
  [SMALL_STATE(843)] = 39758,
  [SMALL_STATE(844)] = 39772,
  [SMALL_STATE(845)] = 39786,
  [SMALL_STATE(846)] = 39800,
  [SMALL_STATE(847)] = 39816,
  [SMALL_STATE(848)] = 39830,
  [SMALL_STATE(849)] = 39846,
  [SMALL_STATE(850)] = 39862,
  [SMALL_STATE(851)] = 39878,
  [SMALL_STATE(852)] = 39894,
  [SMALL_STATE(853)] = 39910,
  [SMALL_STATE(854)] = 39920,
  [SMALL_STATE(855)] = 39936,
  [SMALL_STATE(856)] = 39950,
  [SMALL_STATE(857)] = 39966,
  [SMALL_STATE(858)] = 39982,
  [SMALL_STATE(859)] = 39998,
  [SMALL_STATE(860)] = 40010,
  [SMALL_STATE(861)] = 40024,
  [SMALL_STATE(862)] = 40038,
  [SMALL_STATE(863)] = 40052,
  [SMALL_STATE(864)] = 40066,
  [SMALL_STATE(865)] = 40080,
  [SMALL_STATE(866)] = 40096,
  [SMALL_STATE(867)] = 40112,
  [SMALL_STATE(868)] = 40122,
  [SMALL_STATE(869)] = 40138,
  [SMALL_STATE(870)] = 40148,
  [SMALL_STATE(871)] = 40164,
  [SMALL_STATE(872)] = 40178,
  [SMALL_STATE(873)] = 40187,
  [SMALL_STATE(874)] = 40200,
  [SMALL_STATE(875)] = 40213,
  [SMALL_STATE(876)] = 40226,
  [SMALL_STATE(877)] = 40239,
  [SMALL_STATE(878)] = 40252,
  [SMALL_STATE(879)] = 40265,
  [SMALL_STATE(880)] = 40278,
  [SMALL_STATE(881)] = 40291,
  [SMALL_STATE(882)] = 40300,
  [SMALL_STATE(883)] = 40313,
  [SMALL_STATE(884)] = 40326,
  [SMALL_STATE(885)] = 40339,
  [SMALL_STATE(886)] = 40352,
  [SMALL_STATE(887)] = 40363,
  [SMALL_STATE(888)] = 40376,
  [SMALL_STATE(889)] = 40389,
  [SMALL_STATE(890)] = 40402,
  [SMALL_STATE(891)] = 40413,
  [SMALL_STATE(892)] = 40426,
  [SMALL_STATE(893)] = 40439,
  [SMALL_STATE(894)] = 40452,
  [SMALL_STATE(895)] = 40465,
  [SMALL_STATE(896)] = 40478,
  [SMALL_STATE(897)] = 40491,
  [SMALL_STATE(898)] = 40504,
  [SMALL_STATE(899)] = 40517,
  [SMALL_STATE(900)] = 40530,
  [SMALL_STATE(901)] = 40543,
  [SMALL_STATE(902)] = 40556,
  [SMALL_STATE(903)] = 40569,
  [SMALL_STATE(904)] = 40582,
  [SMALL_STATE(905)] = 40595,
  [SMALL_STATE(906)] = 40608,
  [SMALL_STATE(907)] = 40621,
  [SMALL_STATE(908)] = 40634,
  [SMALL_STATE(909)] = 40647,
  [SMALL_STATE(910)] = 40656,
  [SMALL_STATE(911)] = 40669,
  [SMALL_STATE(912)] = 40682,
  [SMALL_STATE(913)] = 40695,
  [SMALL_STATE(914)] = 40708,
  [SMALL_STATE(915)] = 40721,
  [SMALL_STATE(916)] = 40734,
  [SMALL_STATE(917)] = 40747,
  [SMALL_STATE(918)] = 40760,
  [SMALL_STATE(919)] = 40773,
  [SMALL_STATE(920)] = 40786,
  [SMALL_STATE(921)] = 40799,
  [SMALL_STATE(922)] = 40808,
  [SMALL_STATE(923)] = 40821,
  [SMALL_STATE(924)] = 40834,
  [SMALL_STATE(925)] = 40847,
  [SMALL_STATE(926)] = 40858,
  [SMALL_STATE(927)] = 40871,
  [SMALL_STATE(928)] = 40880,
  [SMALL_STATE(929)] = 40893,
  [SMALL_STATE(930)] = 40906,
  [SMALL_STATE(931)] = 40915,
  [SMALL_STATE(932)] = 40924,
  [SMALL_STATE(933)] = 40937,
  [SMALL_STATE(934)] = 40950,
  [SMALL_STATE(935)] = 40963,
  [SMALL_STATE(936)] = 40976,
  [SMALL_STATE(937)] = 40989,
  [SMALL_STATE(938)] = 41002,
  [SMALL_STATE(939)] = 41013,
  [SMALL_STATE(940)] = 41026,
  [SMALL_STATE(941)] = 41039,
  [SMALL_STATE(942)] = 41052,
  [SMALL_STATE(943)] = 41063,
  [SMALL_STATE(944)] = 41072,
  [SMALL_STATE(945)] = 41085,
  [SMALL_STATE(946)] = 41094,
  [SMALL_STATE(947)] = 41105,
  [SMALL_STATE(948)] = 41118,
  [SMALL_STATE(949)] = 41127,
  [SMALL_STATE(950)] = 41140,
  [SMALL_STATE(951)] = 41153,
  [SMALL_STATE(952)] = 41162,
  [SMALL_STATE(953)] = 41173,
  [SMALL_STATE(954)] = 41186,
  [SMALL_STATE(955)] = 41194,
  [SMALL_STATE(956)] = 41202,
  [SMALL_STATE(957)] = 41210,
  [SMALL_STATE(958)] = 41220,
  [SMALL_STATE(959)] = 41228,
  [SMALL_STATE(960)] = 41236,
  [SMALL_STATE(961)] = 41244,
  [SMALL_STATE(962)] = 41252,
  [SMALL_STATE(963)] = 41260,
  [SMALL_STATE(964)] = 41268,
  [SMALL_STATE(965)] = 41278,
  [SMALL_STATE(966)] = 41286,
  [SMALL_STATE(967)] = 41296,
  [SMALL_STATE(968)] = 41304,
  [SMALL_STATE(969)] = 41312,
  [SMALL_STATE(970)] = 41320,
  [SMALL_STATE(971)] = 41330,
  [SMALL_STATE(972)] = 41340,
  [SMALL_STATE(973)] = 41348,
  [SMALL_STATE(974)] = 41358,
  [SMALL_STATE(975)] = 41366,
  [SMALL_STATE(976)] = 41374,
  [SMALL_STATE(977)] = 41382,
  [SMALL_STATE(978)] = 41392,
  [SMALL_STATE(979)] = 41400,
  [SMALL_STATE(980)] = 41410,
  [SMALL_STATE(981)] = 41418,
  [SMALL_STATE(982)] = 41426,
  [SMALL_STATE(983)] = 41434,
  [SMALL_STATE(984)] = 41442,
  [SMALL_STATE(985)] = 41450,
  [SMALL_STATE(986)] = 41458,
  [SMALL_STATE(987)] = 41466,
  [SMALL_STATE(988)] = 41474,
  [SMALL_STATE(989)] = 41482,
  [SMALL_STATE(990)] = 41490,
  [SMALL_STATE(991)] = 41498,
  [SMALL_STATE(992)] = 41506,
  [SMALL_STATE(993)] = 41514,
  [SMALL_STATE(994)] = 41521,
  [SMALL_STATE(995)] = 41528,
  [SMALL_STATE(996)] = 41535,
  [SMALL_STATE(997)] = 41542,
  [SMALL_STATE(998)] = 41549,
  [SMALL_STATE(999)] = 41556,
  [SMALL_STATE(1000)] = 41563,
  [SMALL_STATE(1001)] = 41570,
  [SMALL_STATE(1002)] = 41577,
  [SMALL_STATE(1003)] = 41584,
  [SMALL_STATE(1004)] = 41591,
  [SMALL_STATE(1005)] = 41598,
  [SMALL_STATE(1006)] = 41605,
  [SMALL_STATE(1007)] = 41612,
  [SMALL_STATE(1008)] = 41619,
  [SMALL_STATE(1009)] = 41626,
  [SMALL_STATE(1010)] = 41633,
  [SMALL_STATE(1011)] = 41640,
  [SMALL_STATE(1012)] = 41647,
  [SMALL_STATE(1013)] = 41654,
  [SMALL_STATE(1014)] = 41661,
  [SMALL_STATE(1015)] = 41668,
  [SMALL_STATE(1016)] = 41675,
  [SMALL_STATE(1017)] = 41682,
  [SMALL_STATE(1018)] = 41689,
  [SMALL_STATE(1019)] = 41696,
  [SMALL_STATE(1020)] = 41703,
  [SMALL_STATE(1021)] = 41710,
  [SMALL_STATE(1022)] = 41717,
  [SMALL_STATE(1023)] = 41724,
  [SMALL_STATE(1024)] = 41731,
  [SMALL_STATE(1025)] = 41738,
  [SMALL_STATE(1026)] = 41745,
  [SMALL_STATE(1027)] = 41752,
  [SMALL_STATE(1028)] = 41759,
  [SMALL_STATE(1029)] = 41766,
  [SMALL_STATE(1030)] = 41773,
  [SMALL_STATE(1031)] = 41780,
  [SMALL_STATE(1032)] = 41787,
  [SMALL_STATE(1033)] = 41794,
  [SMALL_STATE(1034)] = 41801,
  [SMALL_STATE(1035)] = 41808,
  [SMALL_STATE(1036)] = 41815,
  [SMALL_STATE(1037)] = 41822,
  [SMALL_STATE(1038)] = 41829,
  [SMALL_STATE(1039)] = 41836,
  [SMALL_STATE(1040)] = 41843,
  [SMALL_STATE(1041)] = 41850,
  [SMALL_STATE(1042)] = 41857,
  [SMALL_STATE(1043)] = 41864,
  [SMALL_STATE(1044)] = 41871,
  [SMALL_STATE(1045)] = 41878,
  [SMALL_STATE(1046)] = 41885,
  [SMALL_STATE(1047)] = 41892,
  [SMALL_STATE(1048)] = 41899,
  [SMALL_STATE(1049)] = 41906,
  [SMALL_STATE(1050)] = 41913,
  [SMALL_STATE(1051)] = 41920,
  [SMALL_STATE(1052)] = 41927,
  [SMALL_STATE(1053)] = 41934,
  [SMALL_STATE(1054)] = 41941,
  [SMALL_STATE(1055)] = 41948,
  [SMALL_STATE(1056)] = 41955,
  [SMALL_STATE(1057)] = 41962,
  [SMALL_STATE(1058)] = 41969,
  [SMALL_STATE(1059)] = 41976,
  [SMALL_STATE(1060)] = 41983,
  [SMALL_STATE(1061)] = 41990,
  [SMALL_STATE(1062)] = 41997,
  [SMALL_STATE(1063)] = 42004,
  [SMALL_STATE(1064)] = 42011,
  [SMALL_STATE(1065)] = 42018,
  [SMALL_STATE(1066)] = 42025,
  [SMALL_STATE(1067)] = 42032,
  [SMALL_STATE(1068)] = 42039,
  [SMALL_STATE(1069)] = 42046,
  [SMALL_STATE(1070)] = 42053,
  [SMALL_STATE(1071)] = 42060,
  [SMALL_STATE(1072)] = 42067,
  [SMALL_STATE(1073)] = 42074,
  [SMALL_STATE(1074)] = 42081,
  [SMALL_STATE(1075)] = 42088,
  [SMALL_STATE(1076)] = 42095,
  [SMALL_STATE(1077)] = 42102,
  [SMALL_STATE(1078)] = 42109,
  [SMALL_STATE(1079)] = 42116,
  [SMALL_STATE(1080)] = 42123,
  [SMALL_STATE(1081)] = 42130,
  [SMALL_STATE(1082)] = 42137,
  [SMALL_STATE(1083)] = 42144,
  [SMALL_STATE(1084)] = 42151,
  [SMALL_STATE(1085)] = 42158,
  [SMALL_STATE(1086)] = 42165,
  [SMALL_STATE(1087)] = 42172,
};

static TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 0),
  [7] = {.entry = {.count = 1, .reusable = false}}, SHIFT(175),
  [9] = {.entry = {.count = 1, .reusable = false}}, SHIFT(870),
  [11] = {.entry = {.count = 1, .reusable = false}}, SHIFT(765),
  [13] = {.entry = {.count = 1, .reusable = true}}, SHIFT(104),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(548),
  [17] = {.entry = {.count = 1, .reusable = false}}, SHIFT(46),
  [19] = {.entry = {.count = 1, .reusable = false}}, SHIFT(256),
  [21] = {.entry = {.count = 1, .reusable = false}}, SHIFT(180),
  [23] = {.entry = {.count = 1, .reusable = false}}, SHIFT(251),
  [25] = {.entry = {.count = 1, .reusable = false}}, SHIFT(161),
  [27] = {.entry = {.count = 1, .reusable = false}}, SHIFT(958),
  [29] = {.entry = {.count = 1, .reusable = false}}, SHIFT(961),
  [31] = {.entry = {.count = 1, .reusable = false}}, SHIFT(962),
  [33] = {.entry = {.count = 1, .reusable = false}}, SHIFT(263),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(157),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(433),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(264),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1075),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(194),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1074),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1073),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1072),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(166),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1069),
  [55] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [57] = {.entry = {.count = 1, .reusable = true}}, SHIFT(101),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(265),
  [61] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(662),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(162),
  [67] = {.entry = {.count = 1, .reusable = true}}, SHIFT(605),
  [69] = {.entry = {.count = 1, .reusable = true}}, SHIFT(112),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(605),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(47),
  [75] = {.entry = {.count = 1, .reusable = true}}, SHIFT(744),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(272),
  [79] = {.entry = {.count = 1, .reusable = false}}, SHIFT(154),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(435),
  [83] = {.entry = {.count = 1, .reusable = false}}, SHIFT(268),
  [85] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1066),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(210),
  [89] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1079),
  [91] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1068),
  [93] = {.entry = {.count = 1, .reusable = true}}, SHIFT(247),
  [95] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [97] = {.entry = {.count = 1, .reusable = true}}, SHIFT(976),
  [99] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(175),
  [102] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(870),
  [105] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(765),
  [108] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(104),
  [111] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(548),
  [114] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(46),
  [117] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(256),
  [120] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(180),
  [123] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(251),
  [126] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(161),
  [129] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(958),
  [132] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(961),
  [135] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(962),
  [138] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(272),
  [141] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(154),
  [144] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(435),
  [147] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(268),
  [150] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1066),
  [153] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(210),
  [156] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1079),
  [159] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1073),
  [162] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1072),
  [165] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(166),
  [168] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1068),
  [171] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(515),
  [174] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(101),
  [177] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(265),
  [180] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(516),
  [183] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(662),
  [186] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(162),
  [189] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(605),
  [192] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(112),
  [195] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(605),
  [198] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(47),
  [201] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2),
  [203] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(744),
  [206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(237),
  [208] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(263),
  [211] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(157),
  [214] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(433),
  [217] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(264),
  [220] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1075),
  [223] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(194),
  [226] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1074),
  [229] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_module_repeat1, 2), SHIFT_REPEAT(1069),
  [232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(992),
  [234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [236] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 1),
  [238] = {.entry = {.count = 1, .reusable = false}}, SHIFT(540),
  [240] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_primary_expression, 1, .production_id = 1),
  [242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_primary_expression, 1, .production_id = 1),
  [244] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_pattern, 1, .production_id = 1), REDUCE(sym_primary_expression, 1, .production_id = 1),
  [247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(591),
  [249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(297),
  [251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pattern, 1, .production_id = 1),
  [253] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pattern, 1, .production_id = 1),
  [255] = {.entry = {.count = 1, .reusable = false}}, SHIFT(95),
  [257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [261] = {.entry = {.count = 1, .reusable = false}}, SHIFT(516),
  [263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(221),
  [265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(421),
  [267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(7),
  [269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2),
  [273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(409),
  [275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(33),
  [277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(328),
  [279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(279),
  [283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(29),
  [289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(371),
  [291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(19),
  [293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(398),
  [295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(13),
  [297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3),
  [301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(339),
  [307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(9),
  [309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(16),
  [313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(209),
  [319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(427),
  [327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(37),
  [329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(413),
  [331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(34),
  [333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(35),
  [337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(381),
  [339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(25),
  [341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(205),
  [347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
  [349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(403),
  [355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(27),
  [361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(254),
  [367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(28),
  [373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(393),
  [375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(26),
  [381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(14),
  [385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(375),
  [387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(21),
  [389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(315),
  [391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(39),
  [393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(11),
  [397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(407),
  [399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(273),
  [403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(32),
  [405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(17),
  [409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(24),
  [413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(392),
  [415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(36),
  [417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(242),
  [419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(250),
  [421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(985),
  [425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(968),
  [427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(484),
  [431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(135),
  [433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(486),
  [435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(306),
  [439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(657),
  [443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(110),
  [447] = {.entry = {.count = 1, .reusable = false}}, SHIFT(443),
  [449] = {.entry = {.count = 1, .reusable = false}}, SHIFT(93),
  [451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(758),
  [453] = {.entry = {.count = 1, .reusable = false}}, SHIFT(481),
  [455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(146),
  [457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(488),
  [459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [461] = {.entry = {.count = 1, .reusable = false}}, SHIFT(283),
  [463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [465] = {.entry = {.count = 1, .reusable = false}}, SHIFT(660),
  [467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(94),
  [469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(588),
  [471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(134),
  [473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(127),
  [475] = {.entry = {.count = 1, .reusable = false}}, SHIFT(287),
  [477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(658),
  [481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(628),
  [483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(111),
  [485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(628),
  [487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(97),
  [489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(733),
  [491] = {.entry = {.count = 1, .reusable = false}}, SHIFT(305),
  [493] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_primary_expression, 1, .production_id = 1), REDUCE(sym_list_splat_pattern, 2, .production_id = 4),
  [496] = {.entry = {.count = 1, .reusable = false}}, SHIFT(573),
  [498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(105),
  [500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [502] = {.entry = {.count = 1, .reusable = false}}, SHIFT(594),
  [504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(201),
  [508] = {.entry = {.count = 1, .reusable = false}}, SHIFT(152),
  [510] = {.entry = {.count = 1, .reusable = false}}, SHIFT(96),
  [512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(587),
  [514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(530),
  [520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(529),
  [522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1060),
  [526] = {.entry = {.count = 1, .reusable = false}}, SHIFT(252),
  [528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(323),
  [530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1031),
  [534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(609),
  [536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1081),
  [538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(585),
  [540] = {.entry = {.count = 1, .reusable = false}}, SHIFT(527),
  [542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [544] = {.entry = {.count = 1, .reusable = false}}, SHIFT(548),
  [546] = {.entry = {.count = 1, .reusable = false}}, SHIFT(550),
  [548] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pattern_list, 2),
  [550] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pattern_list, 2),
  [552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(106),
  [554] = {.entry = {.count = 1, .reusable = false}}, SHIFT(546),
  [556] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pattern_list, 3),
  [558] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pattern_list, 3),
  [560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__collection_elements, 2),
  [564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(252),
  [566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subscript, 5, .production_id = 40),
  [568] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subscript, 5, .production_id = 40),
  [570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subscript, 4, .production_id = 40),
  [572] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subscript, 4, .production_id = 40),
  [574] = {.entry = {.count = 1, .reusable = false}}, SHIFT(537),
  [576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(590),
  [578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1021),
  [580] = {.entry = {.count = 1, .reusable = false}}, SHIFT(592),
  [582] = {.entry = {.count = 1, .reusable = false}}, SHIFT(98),
  [584] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__collection_elements, 3),
  [586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1004),
  [590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, .production_id = 21),
  [592] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attribute, 3, .production_id = 21),
  [594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(634),
  [596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [600] = {.entry = {.count = 1, .reusable = false}}, SHIFT(613),
  [602] = {.entry = {.count = 1, .reusable = false}}, SHIFT(99),
  [604] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subscript, 6, .production_id = 72),
  [606] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subscript, 6, .production_id = 72),
  [608] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 7, .production_id = 82),
  [610] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_in_clause, 7, .production_id = 82),
  [612] = {.entry = {.count = 1, .reusable = false}}, SHIFT(659),
  [614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [616] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 6, .production_id = 73),
  [618] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_in_clause, 6, .production_id = 73),
  [620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(616),
  [622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(439),
  [624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(440),
  [626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(624),
  [628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(451),
  [634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(453),
  [636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(603),
  [638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(617),
  [640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(583),
  [642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(580),
  [644] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 6, .production_id = 74),
  [646] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_in_clause, 6, .production_id = 74),
  [648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(612),
  [650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(581),
  [654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(604),
  [656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 5, .production_id = 59),
  [658] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_in_clause, 5, .production_id = 59),
  [660] = {.entry = {.count = 1, .reusable = false}}, SHIFT(296),
  [662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield, 1),
  [664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_list, 3),
  [672] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_list, 2),
  [674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(429),
  [676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(199),
  [678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1053),
  [680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(632),
  [682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(626),
  [684] = {.entry = {.count = 1, .reusable = false}}, SHIFT(308),
  [686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raise_statement, 1),
  [688] = {.entry = {.count = 1, .reusable = false}}, SHIFT(322),
  [690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(597),
  [692] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_statement, 5, .production_id = 48),
  [694] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_statement, 5, .production_id = 48),
  [696] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1023),
  [698] = {.entry = {.count = 1, .reusable = false}}, SHIFT(232),
  [700] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1049),
  [702] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_statement, 4, .production_id = 33),
  [704] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_statement, 4, .production_id = 33),
  [706] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1051),
  [708] = {.entry = {.count = 1, .reusable = false}}, SHIFT(231),
  [710] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1050),
  [712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(764),
  [714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(437),
  [716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(602),
  [718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(455),
  [720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [724] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_primary_expression, 1),
  [726] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_pattern, 1), REDUCE(sym_primary_expression, 1),
  [729] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_primary_expression, 1),
  [731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(325),
  [733] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pattern, 1),
  [735] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pattern, 1),
  [737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(619),
  [739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice, 1),
  [743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [745] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 1),
  [747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(125),
  [749] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_list, 3),
  [751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_list, 2),
  [753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice, 2),
  [755] = {.entry = {.count = 1, .reusable = true}}, SHIFT(211),
  [757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(116),
  [761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 2),
  [763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 5, .production_id = 44),
  [765] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 5, .production_id = 44),
  [767] = {.entry = {.count = 1, .reusable = false}}, SHIFT(316),
  [769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(123),
  [771] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 5, .production_id = 43),
  [773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 5, .production_id = 43),
  [775] = {.entry = {.count = 1, .reusable = false}}, SHIFT(261),
  [777] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_try_statement_repeat1, 2),
  [779] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_try_statement_repeat1, 2),
  [781] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_try_statement_repeat1, 2), SHIFT_REPEAT(232),
  [784] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 3),
  [786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list, 2),
  [788] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_list_pattern, 2), REDUCE(sym_list, 2),
  [791] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_list, 2),
  [793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list_pattern, 2),
  [795] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_list_pattern, 2),
  [797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 6, .production_id = 63),
  [799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 6, .production_id = 63),
  [801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 3),
  [803] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 3, .production_id = 5),
  [805] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 4, .production_id = 31),
  [807] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 4, .production_id = 31),
  [809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice, 4),
  [811] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice, 3),
  [813] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 4, .production_id = 14),
  [815] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 4, .production_id = 13),
  [817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple, 2),
  [819] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_tuple_pattern, 2), REDUCE(sym_tuple, 2),
  [822] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple, 2),
  [824] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 2),
  [826] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple_pattern, 2),
  [828] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_try_statement_repeat1, 2), SHIFT_REPEAT(231),
  [831] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_statements, 4),
  [833] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_statements, 4),
  [835] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 1),
  [837] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 1),
  [839] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_if_statement_repeat1, 2, .production_id = 61),
  [841] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_if_statement_repeat1, 2, .production_id = 61),
  [843] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_if_statement_repeat1, 2, .production_id = 61), SHIFT_REPEAT(261),
  [846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(73),
  [848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(56),
  [850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 2),
  [852] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 2),
  [854] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_statements, 2),
  [856] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_statements, 2),
  [858] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_if_statement_repeat1, 2, .production_id = 61), SHIFT_REPEAT(316),
  [861] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_statements, 3),
  [863] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_statements, 3),
  [865] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_except_clause, 3, .production_id = 68),
  [867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_except_clause, 3, .production_id = 68),
  [869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_except_clause, 4),
  [871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_except_clause, 4),
  [873] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_except_clause, 4, .production_id = 80),
  [875] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_except_clause, 4, .production_id = 80),
  [877] = {.entry = {.count = 1, .reusable = false}}, SHIFT(623),
  [879] = {.entry = {.count = 1, .reusable = false}}, SHIFT(636),
  [881] = {.entry = {.count = 1, .reusable = false}}, SHIFT(100),
  [883] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_except_clause, 6, .production_id = 90),
  [885] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_except_clause, 6, .production_id = 90),
  [887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_except_clause, 7),
  [889] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_except_clause, 7),
  [891] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_except_clause, 5),
  [893] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_except_clause, 5),
  [895] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 6, .production_id = 66),
  [897] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 6, .production_id = 66),
  [899] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_statement, 6, .production_id = 48),
  [901] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_statement, 6, .production_id = 48),
  [903] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 7, .production_id = 76),
  [905] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 7, .production_id = 76),
  [907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_statement, 5, .production_id = 33),
  [909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_statement, 5, .production_id = 33),
  [911] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_elif_clause, 5, .production_id = 44),
  [913] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elif_clause, 5, .production_id = 44),
  [915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_if_statement_repeat1, 1, .production_id = 41),
  [917] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_if_statement_repeat1, 1, .production_id = 41),
  [919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 8, .production_id = 84),
  [921] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 8, .production_id = 84),
  [923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elif_clause, 4, .production_id = 31),
  [925] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_elif_clause, 4, .production_id = 31),
  [927] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_statement, 5, .production_id = 47),
  [929] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_statement, 5, .production_id = 47),
  [931] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 7, .production_id = 79),
  [933] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 7, .production_id = 79),
  [935] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_statement, 4, .production_id = 32),
  [937] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_statement, 4, .production_id = 32),
  [939] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else_clause, 3, .production_id = 33),
  [941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else_clause, 3, .production_id = 33),
  [943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_concatenated_string_repeat1, 2),
  [945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_concatenated_string_repeat1, 2),
  [947] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_concatenated_string_repeat1, 2), SHIFT_REPEAT(758),
  [950] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concatenated_string, 2),
  [952] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concatenated_string, 2),
  [954] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else_clause, 4, .production_id = 48),
  [956] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else_clause, 4, .production_id = 48),
  [958] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 6, .production_id = 62),
  [960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 6, .production_id = 62),
  [962] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__patterns, 3),
  [964] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 6, .production_id = 70),
  [966] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class_definition, 6, .production_id = 70),
  [968] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 7, .production_id = 78),
  [970] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 7, .production_id = 78),
  [972] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 77),
  [974] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 77),
  [976] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_statement, 4, .production_id = 34),
  [978] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_with_statement, 4, .production_id = 34),
  [980] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 69),
  [982] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 6, .production_id = 69),
  [984] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 7, .production_id = 75),
  [986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 7, .production_id = 75),
  [988] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_with_statement, 5, .production_id = 45),
  [990] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_statement, 5, .production_id = 45),
  [992] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_statement, 6, .production_id = 33),
  [994] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_statement, 6, .production_id = 33),
  [996] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 4, .production_id = 36),
  [998] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class_definition, 4, .production_id = 36),
  [1000] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 9, .production_id = 88),
  [1002] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 9, .production_id = 88),
  [1004] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__patterns, 2),
  [1006] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_finally_clause, 3, .production_id = 68),
  [1008] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_finally_clause, 3, .production_id = 68),
  [1010] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_statement, 6, .production_id = 67),
  [1012] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_statement, 6, .production_id = 67),
  [1014] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 6, .production_id = 65),
  [1016] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 6, .production_id = 65),
  [1018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 9, .production_id = 89),
  [1020] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 9, .production_id = 89),
  [1022] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_with_statement, 6, .production_id = 64),
  [1024] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_statement, 6, .production_id = 64),
  [1026] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_decorated_definition, 2, .production_id = 11),
  [1028] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_decorated_definition, 2, .production_id = 11),
  [1030] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 87),
  [1032] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 8, .production_id = 87),
  [1034] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_finally_clause, 4),
  [1036] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_finally_clause, 4),
  [1038] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 6, .production_id = 60),
  [1040] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 6, .production_id = 60),
  [1042] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_try_statement, 7, .production_id = 48),
  [1044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_try_statement, 7, .production_id = 48),
  [1046] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 7, .production_id = 81),
  [1048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 7, .production_id = 81),
  [1050] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 8, .production_id = 86),
  [1052] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 8, .production_id = 86),
  [1054] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_statement, 5, .production_id = 46),
  [1056] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_statement, 5, .production_id = 46),
  [1058] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_statement, 5, .production_id = 49),
  [1060] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_with_statement, 5, .production_id = 49),
  [1062] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 8, .production_id = 85),
  [1064] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 8, .production_id = 85),
  [1066] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 8, .production_id = 83),
  [1068] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 8, .production_id = 83),
  [1070] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class_definition, 5, .production_id = 52),
  [1072] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 52),
  [1074] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class_definition, 5, .production_id = 51),
  [1076] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class_definition, 5, .production_id = 51),
  [1078] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 5, .production_id = 50),
  [1080] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, .production_id = 50),
  [1082] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 5, .production_id = 42),
  [1084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 5, .production_id = 42),
  [1086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [1088] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 3),
  [1090] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 3),
  [1092] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 2),
  [1094] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 2),
  [1096] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary, 5),
  [1098] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dictionary, 5),
  [1100] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3),
  [1102] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3),
  [1104] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 5, .production_id = 27),
  [1106] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 5, .production_id = 27),
  [1108] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 5),
  [1110] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 5),
  [1112] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list, 3),
  [1114] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_list, 3),
  [1116] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 2),
  [1118] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 2),
  [1120] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple, 3),
  [1122] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple, 3),
  [1124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1041),
  [1126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(120),
  [1128] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression, 1),
  [1130] = {.entry = {.count = 1, .reusable = false}}, SHIFT(533),
  [1132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [1134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(535),
  [1136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(536),
  [1138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(533),
  [1140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(212),
  [1142] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression, 1),
  [1144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(997),
  [1146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(539),
  [1148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(541),
  [1150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [1152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [1154] = {.entry = {.count = 1, .reusable = false}}, SHIFT(535),
  [1156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [1158] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary, 2),
  [1160] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dictionary, 2),
  [1162] = {.entry = {.count = 1, .reusable = false}}, SHIFT(526),
  [1164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [1166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [1168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [1170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(526),
  [1172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1037),
  [1174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [1176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [1178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [1180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [1182] = {.entry = {.count = 1, .reusable = false}}, SHIFT(524),
  [1184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [1186] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generator_expression, 4, .production_id = 28),
  [1188] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generator_expression, 4, .production_id = 28),
  [1190] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 2, .production_id = 10),
  [1192] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 2, .production_id = 10),
  [1194] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 4, .production_id = 27),
  [1196] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 4, .production_id = 27),
  [1198] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set, 3),
  [1200] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_set, 3),
  [1202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 4),
  [1204] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 4),
  [1206] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_comprehension, 4, .production_id = 28),
  [1208] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_set_comprehension, 4, .production_id = 28),
  [1210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary, 4),
  [1212] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dictionary, 4),
  [1214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list_comprehension, 4, .production_id = 28),
  [1216] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_list_comprehension, 4, .production_id = 28),
  [1218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 3, .production_id = 27),
  [1220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 3, .production_id = 27),
  [1222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 3),
  [1224] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 3),
  [1226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary, 3),
  [1228] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dictionary, 3),
  [1230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary_comprehension, 4, .production_id = 28),
  [1232] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_dictionary_comprehension, 4, .production_id = 28),
  [1234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 3),
  [1236] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_comparison_operator_repeat1, 3),
  [1238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary_operator, 3, .production_id = 20),
  [1240] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary_operator, 3, .production_id = 20),
  [1242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2),
  [1244] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_comparison_operator_repeat1, 2),
  [1246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_operator, 2, .production_id = 8),
  [1248] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_operator, 2, .production_id = 8),
  [1250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(294),
  [1252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(301),
  [1254] = {.entry = {.count = 1, .reusable = false}}, SHIFT(538),
  [1256] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 1, .production_id = 9),
  [1258] = {.entry = {.count = 1, .reusable = false}}, SHIFT(528),
  [1260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(994),
  [1262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(118),
  [1264] = {.entry = {.count = 1, .reusable = false}}, SHIFT(512),
  [1266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [1268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(552),
  [1270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [1272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [1274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [1276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1035),
  [1278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [1280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
  [1282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [1284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(574),
  [1286] = {.entry = {.count = 1, .reusable = false}}, SHIFT(552),
  [1288] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [1290] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_concatenated_string_repeat1, 2), SHIFT_REPEAT(744),
  [1293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(995),
  [1295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [1297] = {.entry = {.count = 1, .reusable = false}}, SHIFT(549),
  [1299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [1301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [1303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [1305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [1307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(224),
  [1309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1029),
  [1311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(555),
  [1313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(556),
  [1315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [1317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [1319] = {.entry = {.count = 1, .reusable = false}}, SHIFT(553),
  [1321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [1323] = {.entry = {.count = 1, .reusable = false}}, SHIFT(563),
  [1325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(532),
  [1327] = {.entry = {.count = 1, .reusable = false}}, SHIFT(514),
  [1329] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_concatenated_string_repeat1, 2), SHIFT_REPEAT(733),
  [1332] = {.entry = {.count = 1, .reusable = false}}, SHIFT(545),
  [1334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list_splat_pattern, 2, .production_id = 4),
  [1336] = {.entry = {.count = 1, .reusable = false}}, SHIFT(310),
  [1338] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list_splat_pattern, 2),
  [1340] = {.entry = {.count = 1, .reusable = false}}, SHIFT(648),
  [1342] = {.entry = {.count = 1, .reusable = false}}, SHIFT(645),
  [1344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(282),
  [1346] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_primary_expression, 1), REDUCE(sym_list_splat_pattern, 2),
  [1349] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(524),
  [1352] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(1037),
  [1355] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(524),
  [1358] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(504),
  [1361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_comparison_operator, 2),
  [1363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_comparison_operator, 2),
  [1365] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(535),
  [1368] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(997),
  [1371] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(535),
  [1374] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(510),
  [1377] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__patterns_repeat1, 2),
  [1379] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__patterns_repeat1, 2), SHIFT_REPEAT(445),
  [1382] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(552),
  [1385] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(1035),
  [1388] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(552),
  [1391] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(500),
  [1394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(840),
  [1396] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary_splat_pattern, 2, .production_id = 4),
  [1398] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary_splat_pattern, 2),
  [1400] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pattern, 3),
  [1402] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list_pattern, 3),
  [1404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [1406] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(553),
  [1409] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(1029),
  [1412] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(553),
  [1415] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_operator_repeat1, 2), SHIFT_REPEAT(499),
  [1418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(113),
  [1420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [1422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(103),
  [1424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(107),
  [1426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(952),
  [1428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [1430] = {.entry = {.count = 1, .reusable = false}}, SHIFT(485),
  [1432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(302),
  [1434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(547),
  [1436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(278),
  [1438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(220),
  [1440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [1442] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_boolean_operator, 3, .production_id = 20),
  [1444] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_boolean_operator, 3, .production_id = 20),
  [1446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(319),
  [1448] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_await, 2),
  [1450] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_await, 2),
  [1452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [1454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(954),
  [1456] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__parameters, 2),
  [1458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__parameters, 3),
  [1460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(631),
  [1462] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_with_item, 1, .dynamic_precedence = -1, .production_id = 6), SHIFT(115),
  [1465] = {.entry = {.count = 1, .reusable = false}}, SHIFT(436),
  [1467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(277),
  [1469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1043),
  [1471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(434),
  [1473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(289),
  [1475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [1477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(298),
  [1479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [1481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [1483] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__collection_elements, 1),
  [1485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_conditional_expression, 5),
  [1487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(276),
  [1489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(292),
  [1491] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_not_operator, 2, .production_id = 5),
  [1493] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_not_operator, 2, .production_id = 5),
  [1495] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda, 4, .production_id = 38),
  [1497] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda, 4, .production_id = 38),
  [1499] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_conditional_expression, 5),
  [1501] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda, 3, .production_id = 16),
  [1503] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda, 3, .production_id = 16),
  [1505] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_named_expression, 3, .production_id = 17),
  [1507] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_named_expression, 3, .production_id = 17),
  [1509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(438),
  [1511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(606),
  [1513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(139),
  [1515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [1517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(622),
  [1519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(132),
  [1521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(458),
  [1523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [1525] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__comprehension_clauses, 2),
  [1527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(318),
  [1529] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression_within_for_in_clause, 1),
  [1531] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__comprehension_clauses_repeat1, 2),
  [1533] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__comprehension_clauses_repeat1, 2), SHIFT_REPEAT(318),
  [1536] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__comprehension_clauses_repeat1, 2), SHIFT_REPEAT(1043),
  [1539] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__comprehension_clauses_repeat1, 2), SHIFT_REPEAT(434),
  [1542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__comprehension_clauses, 1),
  [1544] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 5, .production_id = 74),
  [1546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [1548] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 4, .production_id = 59),
  [1550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(151),
  [1552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [1554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_clause, 2),
  [1556] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield, 2),
  [1558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(156),
  [1560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [1562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [1564] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_for_in_clause_repeat1, 2),
  [1566] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_for_in_clause_repeat1, 2), SHIFT_REPEAT(198),
  [1569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 6, .production_id = 82),
  [1571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(126),
  [1573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1083),
  [1575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1079),
  [1577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1068),
  [1579] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_assert_statement_repeat1, 2),
  [1581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(311),
  [1583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(184),
  [1585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raise_statement, 2),
  [1587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1040),
  [1589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1074),
  [1591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1069),
  [1593] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_in_clause, 5, .production_id = 73),
  [1595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(128),
  [1597] = {.entry = {.count = 1, .reusable = false}}, SHIFT(280),
  [1599] = {.entry = {.count = 1, .reusable = false}}, SHIFT(735),
  [1601] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [1603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [1605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(572),
  [1607] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__collection_elements_repeat1, 2),
  [1609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(189),
  [1611] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 1),
  [1613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [1615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(866),
  [1617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [1619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [1621] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_exec_statement, 4, .production_id = 7),
  [1623] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__right_hand_side, 1),
  [1625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(170),
  [1627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(459),
  [1629] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 2),
  [1631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_delete_statement, 2),
  [1633] = {.entry = {.count = 1, .reusable = false}}, SHIFT(746),
  [1635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [1637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(593),
  [1639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [1641] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_string_repeat1, 2), SHIFT_REPEAT(280),
  [1644] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_string_repeat1, 2), SHIFT_REPEAT(735),
  [1647] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_repeat1, 2), SHIFT_REPEAT(735),
  [1650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_repeat1, 2),
  [1652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [1654] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type, 1),
  [1656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(274),
  [1658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [1660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [1662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assert_statement, 2),
  [1664] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_item, 1, .dynamic_precedence = -1, .production_id = 6),
  [1666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [1668] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield, 3),
  [1670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [1672] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 2, .production_id = 5),
  [1674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_within_for_in_clause, 3, .production_id = 16),
  [1676] = {.entry = {.count = 1, .reusable = false}}, SHIFT(723),
  [1678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [1680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [1682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(600),
  [1684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(160),
  [1686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [1688] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pair, 3, .production_id = 39),
  [1690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_within_for_in_clause, 4, .production_id = 38),
  [1692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(163),
  [1694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(586),
  [1696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [1698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [1700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [1702] = {.entry = {.count = 1, .reusable = false}}, SHIFT(734),
  [1704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [1706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(432),
  [1708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(187),
  [1710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(186),
  [1712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [1714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_list_splat, 2),
  [1716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(188),
  [1718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [1720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [1722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(615),
  [1724] = {.entry = {.count = 1, .reusable = false}}, SHIFT(736),
  [1726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(736),
  [1728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(909),
  [1730] = {.entry = {.count = 1, .reusable = false}}, SHIFT(770),
  [1732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [1734] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1022),
  [1736] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_subscript_repeat1, 2, .production_id = 55),
  [1738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(257),
  [1740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(52),
  [1742] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dotted_name, 1),
  [1744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1032),
  [1746] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dotted_name, 2),
  [1748] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_chevron, 2),
  [1750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(155),
  [1752] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_assert_statement_repeat1, 2), SHIFT_REPEAT(300),
  [1755] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_typed_default_parameter, 5, .production_id = 71),
  [1757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_dotted_name_repeat1, 2),
  [1759] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_dotted_name_repeat1, 2), SHIFT_REPEAT(1032),
  [1762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [1764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(82),
  [1766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [1768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(72),
  [1770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(207),
  [1772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1005),
  [1774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_print_statement_repeat1, 2, .production_id = 5),
  [1776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(60),
  [1778] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_dotted_name_repeat1, 2), SHIFT_REPEAT(1005),
  [1781] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_dictionary_splat, 2),
  [1783] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_default_parameter, 3, .production_id = 17),
  [1785] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_decorated_definition_repeat1, 2),
  [1787] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_decorated_definition_repeat1, 2), SHIFT_REPEAT(515),
  [1790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [1792] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_assert_statement_repeat1, 2), SHIFT_REPEAT(307),
  [1795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [1797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1050),
  [1799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interpolation, 3),
  [1801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpolation, 3),
  [1803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(183),
  [1805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(232),
  [1807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1049),
  [1809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raise_statement, 3, .production_id = 15),
  [1811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(119),
  [1813] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_raise_statement, 4, .production_id = 30),
  [1815] = {.entry = {.count = 1, .reusable = false}}, SHIFT(324),
  [1817] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_format_specifier, 1),
  [1819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [1821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_slice, 5),
  [1823] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__collection_elements_repeat1, 2), SHIFT_REPEAT(158),
  [1826] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_format_specifier, 2),
  [1828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [1830] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interpolation, 4),
  [1832] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpolation, 4),
  [1834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(846),
  [1836] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interpolation, 5),
  [1838] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpolation, 5),
  [1840] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_list, 2, .production_id = 3),
  [1842] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_format_specifier_repeat1, 2), SHIFT_REPEAT(324),
  [1845] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_format_specifier_repeat1, 2),
  [1847] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_format_specifier_repeat1, 2), SHIFT_REPEAT(817),
  [1850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_list, 3, .production_id = 12),
  [1852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [1854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1011),
  [1856] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_list, 1, .production_id = 3),
  [1858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_argument, 3, .production_id = 54),
  [1860] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_argument, 3, .production_id = 17),
  [1862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2),
  [1864] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__patterns, 1),
  [1866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(373),
  [1868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [1870] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_exec_statement, 5, .production_id = 7),
  [1872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(271),
  [1874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [1876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(69),
  [1878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter, 1),
  [1880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(243),
  [1882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(270),
  [1884] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__import_list_repeat1, 2, .production_id = 22),
  [1886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(886),
  [1888] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_list_splat, 3),
  [1890] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_import_prefix, 1),
  [1892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(844),
  [1894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(200),
  [1896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [1898] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__import_list, 2, .production_id = 12),
  [1900] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_print_statement_repeat1, 2, .production_id = 29), SHIFT_REPEAT(317),
  [1903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_print_statement_repeat1, 2, .production_id = 29),
  [1905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1039),
  [1907] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_global_statement, 2),
  [1909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [1911] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_decorator, 3),
  [1913] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__import_list_repeat1, 2, .production_id = 23), SHIFT_REPEAT(887),
  [1916] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__import_list_repeat1, 2, .production_id = 23),
  [1918] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_nonlocal_statement, 3),
  [1920] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_global_statement, 3),
  [1922] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_import_prefix_repeat1, 2),
  [1924] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_import_prefix_repeat1, 2), SHIFT_REPEAT(844),
  [1927] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_nonlocal_statement, 2),
  [1929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [1931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(58),
  [1933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [1935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(79),
  [1937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [1939] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_list_splat, 3, .production_id = 27),
  [1941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(62),
  [1943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(203),
  [1945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 2),
  [1947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [1949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(59),
  [1951] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_typed_parameter, 3, .production_id = 37),
  [1953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [1955] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_global_statement_repeat1, 2), SHIFT_REPEAT(1039),
  [1958] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_global_statement_repeat1, 2),
  [1960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_with_clause_repeat1, 2),
  [1962] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_with_clause_repeat1, 2), SHIFT_REPEAT(227),
  [1965] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assert_statement, 3),
  [1967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [1969] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 3, .production_id = 14),
  [1971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [1973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_print_statement, 3, .production_id = 13),
  [1975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(61),
  [1977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [1979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(993),
  [1981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(356),
  [1983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(227),
  [1985] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_clause, 1),
  [1987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(223),
  [1989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(225),
  [1991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [1993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(90),
  [1995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(956),
  [1997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [1999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(839),
  [2001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(87),
  [2003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [2005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1048),
  [2007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(853),
  [2009] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_format_expression, 3),
  [2011] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_format_expression, 3),
  [2013] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__parameters, 1),
  [2015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(669),
  [2017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [2019] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_clause, 2),
  [2021] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__import_list_repeat1, 2, .production_id = 23), SHIFT_REPEAT(898),
  [2024] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_subscript_repeat1, 2, .production_id = 56), SHIFT_REPEAT(191),
  [2027] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_subscript_repeat1, 2, .production_id = 56),
  [2029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(89),
  [2031] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2), SHIFT_REPEAT(153),
  [2034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(124),
  [2036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [2038] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_relative_import, 1),
  [2040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(176),
  [2042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(141),
  [2044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(192),
  [2046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(129),
  [2048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [2050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(625),
  [2052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(145),
  [2054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(668),
  [2056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [2058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(667),
  [2060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [2062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(169),
  [2064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(159),
  [2066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [2068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(86),
  [2070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [2072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(137),
  [2074] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_dictionary_repeat1, 2), SHIFT_REPEAT(177),
  [2077] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_dictionary_repeat1, 2),
  [2079] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__simple_statements_repeat1, 2), SHIFT_REPEAT(92),
  [2082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__simple_statements_repeat1, 2),
  [2084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [2086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment, 3, .production_id = 18),
  [2088] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__parameters_repeat1, 2), SHIFT_REPEAT(671),
  [2091] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__parameters_repeat1, 2),
  [2093] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_item, 3, .dynamic_precedence = -1, .production_id = 35),
  [2095] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__parameters_repeat1, 2), SHIFT_REPEAT(674),
  [2098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(91),
  [2100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [2102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(150),
  [2104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(131),
  [2106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(248),
  [2108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(130),
  [2110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(808),
  [2112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [2114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [2116] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_aliased_import, 3, .production_id = 24),
  [2118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(290),
  [2120] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_exec_statement, 2, .production_id = 7),
  [2122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [2124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(666),
  [2126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1026),
  [2128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(88),
  [2130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [2132] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 2),
  [2134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [2136] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pass_statement, 1),
  [2138] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_statement, 1),
  [2140] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_statement, 1),
  [2142] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_future_import_statement, 4, .production_id = 25),
  [2144] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_wildcard_import, 1),
  [2146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_from_statement, 4, .production_id = 26),
  [2148] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_from_statement, 6, .production_id = 58),
  [2150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(77),
  [2152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [2154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [2156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(239),
  [2158] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_future_import_statement, 6, .production_id = 57),
  [2160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(53),
  [2162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [2164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(50),
  [2166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(246),
  [2168] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2, .production_id = 27),
  [2170] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_augmented_assignment, 3, .production_id = 20),
  [2172] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment, 5, .production_id = 53),
  [2174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_import_statement, 2, .production_id = 2),
  [2176] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 3),
  [2178] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment, 3, .production_id = 19),
  [2180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(974),
  [2182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(174),
  [2184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [2186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(599),
  [2188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(545),
  [2190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(621),
  [2192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(608),
  [2194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(578),
  [2196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(598),
  [2198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(627),
  [2200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(629),
  [2202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [2204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(460),
  [2206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(454),
  [2208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(813),
  [2210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(456),
  [2212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(945),
  [2214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(975),
  [2216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(969),
  [2218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [2220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(596),
  [2222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [2224] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_relative_import, 2),
  [2226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [2228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(726),
  [2230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(811),
  [2232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(67),
  [2234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [2236] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_clause, 4),
  [2238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(610),
  [2240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [2242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [2244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(75),
  [2246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [2248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [2250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [2252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(532),
  [2254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(70),
  [2256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [2258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(633),
  [2260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [2262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(121),
  [2264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(74),
  [2266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [2268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(989),
  [2270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(81),
  [2272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(595),
  [2274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_with_clause, 3),
  [2276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(66),
  [2278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(78),
  [2280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(76),
  [2282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [2284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(452),
  [2286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [2288] = {.entry = {.count = 1, .reusable = true}}, SHIFT(83),
  [2290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(57),
  [2292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(55),
  [2294] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [2296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [2298] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [2300] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [2302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(65),
  [2304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(444),
  [2306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [2308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(944),
  [2310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [2312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(845),
  [2314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(838),
  [2316] = {.entry = {.count = 1, .reusable = true}}, SHIFT(970),
  [2318] = {.entry = {.count = 1, .reusable = true}}, SHIFT(80),
  [2320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(584),
  [2322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(269),
  [2324] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda_parameters, 1),
  [2326] = {.entry = {.count = 1, .reusable = true}}, SHIFT(966),
  [2328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(964),
  [2330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(63),
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_python_external_scanner_create(void);
void tree_sitter_python_external_scanner_destroy(void *);
bool tree_sitter_python_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_python_external_scanner_serialize(void *, char *);
void tree_sitter_python_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef _WIN32
#define extern __declspec(dllexport)
#endif

extern const TSLanguage *tree_sitter_python(void) {
  static TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = (const uint16_t *)ts_parse_table,
    .small_parse_table = (const uint16_t *)ts_small_parse_table,
    .small_parse_table_map = (const uint32_t *)ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = (const TSFieldMapSlice *)ts_field_map_slices,
    .field_map_entries = (const TSFieldMapEntry *)ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = (const TSSymbol *)ts_alias_sequences,
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      (const bool *)ts_external_scanner_states,
      ts_external_scanner_symbol_map,
      tree_sitter_python_external_scanner_create,
      tree_sitter_python_external_scanner_destroy,
      tree_sitter_python_external_scanner_scan,
      tree_sitter_python_external_scanner_serialize,
      tree_sitter_python_external_scanner_deserialize,
    },
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
